#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "arte/Arte.h"
#include "auto/DDEF.h"
#include "arte/mecCount.h"

#define FIELD_SIZE 80

#define OUTPUT_DIRECTORY "new.inc"

/* 
   WRITE_SEPARATE_HEADERS: if this variable is defined, a header
   for each table is created. If the variable is not defined, 
   tables definitions are collected in menu headers.
 */

#define WRITE_SEPARATE_HEADERS
				
static struct _menu {
	char name[FIELD_SIZE]; /* menu name */
	char supp[FIELD_SIZE]; /* upper menu */
	int  cflg;             /* 0 - selfconsistent, 1 - within supp, 2 - no */
	struct _menu *next;    /* next struct in the list */
	struct _mlst *mlst;    /* list of menus linked by relations */
	} * menus = NULL;

struct _mlst {
	struct _menu *menu;
	struct _mlst *next;
	};
	
static struct _table {
	char name[FIELD_SIZE]; /* table name */
	int first, last;       /* first and last lines in DEFF */
	struct _menu *menu;    /* menu */
	struct _rrel *rrel;    /* list of relations */
	struct _table *next;   /* next table in the list */
	} * tables = NULL;

struct _rrel {
	char fname[FIELD_SIZE]; /* name of the field */
	char tname[FIELD_SIZE]; /* name of relation table */
	int  type;              /* type (as in DDEF(i).flag) */
	int  sametbl;           /* relation to the same table ? 0/1 */
	struct _table *itbl;    /* related table */
	struct _rrel *invr;     /* inverse relation (in itbl->rrel) */
	struct _rrel *next;     /* next relation in the list */
	} ;

static int wrt_tbl(struct _table *, FILE *);
	
static void add_menu(struct _menu *menu, struct _menu *dep)
{
	struct _mlst *i, *j;
	
	for(i=menu->mlst;i;i=i->next)
		if(i->menu==dep)
			break;
	if(i!=NULL)
		return;
	i=malloc(sizeof(struct _mlst));
	i->next=NULL;
	i->menu=dep;
	if(menu->mlst==NULL)
	{
		menu->mlst=i;
		return;
	}
	for(j=menu->mlst;j->next;j=j->next);
	j->next=i;
}

	
static void write_blank(FILE *f, int n)
{
	while(n>0)
	{
		fprintf(f," ");
		n--;
	}
}	

		
static void f2c(char *s, int l)
{
	int i;
	for(i=0;s[i] && s[i]!=' ' && i<l;i++);
	if(i==l) i--;
	s[i]=0;
}	

static void f3c(char *s, int l)
{
	int i;
	for(i=l-1; s[i]==' ' && i>=0;i--)
		s[i]=0;
}	

void ddmkleda_(void)
{
	
  /*************************************************************************
   *
   * Author:  A.Semenov (JINR, Dubna)
   * Created: 03.05.99
   *
   * Description: Composes LEDA header files from DD (got from
   *           table DDEF). Files created: OUTPUT_DIRECTORY/LEDA_<menu>.h
   *
   * Called from: DDEXEC
   *
   * External references: none
   *
   * Arguments: none 
   *
   *
   ************************************************************************/
	
	
	
	
int i;

struct _menu *omn=NULL;
struct _table *otbl=NULL;
struct _rrel *orel=NULL;

   /* collect menus */

for(i=1;i<=NDDEF();i++)
{
if(DDEF(i).flag==3 && DDEF(i).name[0]!=' ')
	{
	struct _menu *mn;
	mn=malloc(sizeof(struct _menu));
	strncpy(mn->name,DDEF(i).name,FIELD_SIZE);
	f2c(mn->name,FIELD_SIZE);
	strncpy(mn->supp,DDEF(i).nam2,FIELD_SIZE);
	f2c(mn->supp,FIELD_SIZE);
	if(mn->supp[0]==0)
		strcpy(mn->supp,mn->name);
	mn->cflg=0;
	mn->mlst=NULL;
	mn->next=NULL;
	if(menus==NULL)
		menus=mn;
	else
		omn->next=mn;
	omn=mn;
	}
}

   /* Collect tables and relations */

for(i=1;i<=NDDEF();i++)
{
	int j; 
	struct _table *tbl;
	struct _menu *mn;

	if(DDEF(i).flag!=1)
		continue;
	
	tbl=malloc(sizeof(struct _table));
	strncpy(tbl->name,DDEF(i).nam2,FIELD_SIZE);
	f2c(tbl->name,FIELD_SIZE);
	
	for(mn=menus;mn;mn=mn->next)
		if(strcmp(mn->name,tbl->name)==0)
			break;
	if(mn==NULL)
	{
		static int errPrt;
		static char errMsg[80];
		sprintf(errMsg,"ddmkleda: table %10.10s: unknown menu %s\n",
				DDEF(i).name,tbl->name);
		mecErrorCount(&errPrt,errMsg);
		return;
	}
	tbl->menu=mn;
	
	strncpy(tbl->name,DDEF(i).name,FIELD_SIZE);
	f2c(tbl->name,FIELD_SIZE);
	tbl->next=NULL;
	tbl->rrel=NULL;
	tbl->first=i;
	if(tables==NULL)
		tables=tbl;
	else
		otbl->next=tbl;
	otbl=tbl;
	
	for(j=i+1;j<=NDDEF();j++)
	{
		if(DDEF(j).flag<5)
		{
			tbl->last=j-1;
			break;
		}
		
		if(DDEF(j).flag>6 && DDEF(j).flag<11)
		{
			struct _rrel *rel;
			rel=malloc(sizeof(struct _rrel));
			strncpy(rel->fname,DDEF(j).name,FIELD_SIZE);
			f2c(rel->fname,FIELD_SIZE);
			strncpy(rel->tname,DDEF(j).nam3,FIELD_SIZE);
			f2c(rel->tname,FIELD_SIZE);
			rel->type=DDEF(j).flag;
			rel->sametbl=0;
			rel->itbl=NULL;
			rel->invr=NULL;
			rel->next=NULL;
			if(tbl->rrel==NULL)
				tbl->rrel=rel;
			else
				orel->next=rel;
			orel=rel;
		}
	}
	
}

 /* Establish inverse relations */

for(otbl=tables;otbl;otbl=otbl->next)
	for(orel=otbl->rrel;orel;orel=orel->next)
	{
		struct _table *itbl;
		struct _rrel *irel;
		int sflg=0;
		
		if(orel->invr)
			continue;
		
		for(itbl=otbl; itbl && sflg==0; itbl=itbl->next)
			for(irel=itbl->rrel;irel;irel=irel->next)
			{
				if(irel==orel)
					continue;
				if(irel->invr)
					continue;
				if(strcmp(irel->tname,orel->tname)==0)
				{
					orel->invr=irel;
					irel->invr=orel;
					orel->itbl=itbl;
					irel->itbl=otbl;
					if(itbl==otbl)
						irel->sametbl=1;
					sflg=1;
					if(strcmp(otbl->menu->name,itbl->menu->name)==0)
						break;
					add_menu(otbl->menu,itbl->menu);
					add_menu(itbl->menu,otbl->menu);
					if(strcmp(otbl->menu->supp,itbl->menu->supp)==0)
					{
						if(otbl->menu->cflg==0)
							otbl->menu->cflg=1;
						if(itbl->menu->cflg==0)
							itbl->menu->cflg=1;
						break;
					}
					otbl->menu->cflg=2;
					itbl->menu->cflg=2;
					break;
				}
			}
		
		if(sflg==0)
		{
			static int errPrt;
			static char errMsg[80];
			sprintf(errMsg,"ddmkleda: table %s, reltable %s: no match\n",
					otbl->name,orel->tname);
			mecErrorCount(&errPrt,errMsg);
			return;
		}
	}


		
  /*  Finally write header files */


for(omn=menus;omn;omn=omn->next)
{
	FILE *fout;
	char cbuf[180];
	sprintf(cbuf,"%s/%s.leda",OUTPUT_DIRECTORY,omn->name);
	fout=fopen(cbuf,"w");
	if(fout==NULL)
	{
		static int errPrt;
		static char errMsg[80];
		sprintf(errMsg,"ddmkleda: can not write to file %s\n",cbuf);
		mecErrorCount(&errPrt,errMsg);
		perror("ddmkleda:");
		return;
	}
	
	fprintf(fout,"\n#ifndef LEDA_%s_HEADER\n#define LEDA_%s_HEADER\n\n",
			omn->name, omn->name);
	
	for(otbl=tables;otbl;otbl=otbl->next)
	{
				
#ifdef WRITE_SEPARATE_HEADERS
		
		FILE *fout1;
		
		if(otbl->menu!=omn)
				continue;

		
		sprintf(cbuf,"%s/%s.leda",OUTPUT_DIRECTORY,
				otbl->name);
		fout1=fopen(cbuf,"w");
		if(fout1==NULL)
		{
			static int errPrt;
			static char errMsg[80];
			sprintf(errMsg,"ddmkleda: can not write to file %s\n",cbuf);
			mecErrorCount(&errPrt,errMsg);
			perror("ddmkleda:");
			return;
		}
		
		if(wrt_tbl(otbl,fout1)==0)
			return ;
		
		fclose(fout1);
		
		fprintf(fout,"#include \"auto/%s.leda\"\n",otbl->name);
		
#else
			if(otbl->menu!=omn)
				continue;

			if(wrt_tbl(otbl,fout)==0)
				return ;	
#endif

	}
	
	if(omn->mlst)
	{
		struct _mlst *l;
		fprintf(fout,"\n\n\t/* Menus involved by relations */\n\n");
		for(l=omn->mlst;l;l=l->next)
			fprintf(fout,"#include \"auto/%s.leda\"\n",l->menu->name);
	}
	
	{
		struct _menu *pmn;
		i=0;
		for(pmn=menus;pmn;pmn=pmn->next)
		{
			if(pmn==omn)
				continue;
			if(strcmp(omn->name,pmn->supp)==0)
				i++;
		}
		if(i)
		{
			fprintf(fout,"\n\n\t/* Subordinate menus */\n\n");
			for(pmn=menus;pmn;pmn=pmn->next)
			{
				if(pmn==omn)
					continue;
				if(strcmp(omn->name,pmn->supp)==0)
					fprintf(fout,"#include \"auto/%s.leda\"\n",pmn->name);
			}
		}
	}
	
	fprintf(fout,"\n#endif\n\n");
	fclose(fout);
}

	/* printout */
	
{
	struct _menu *mn;
	for(mn=menus;mn;mn=mn->next)
	{
		struct _mlst *l;
		printf(" === DDMKLEDA: wrote %s/%s.leda ",OUTPUT_DIRECTORY,mn->name);
		if(mn->mlst==NULL)
			printf("\n");
		else
		{
			printf("(related menus: ");
			for(l=mn->mlst;l;l=l->next)
				printf("%s ",l->menu->name);
			printf(")\n");
		}
	}
}	


{ FILE *f;
f=fopen("output.my","w");

for(i=1;i<=NDDEF();i++)
{

fprintf(f,"%2d %2d %2d %2d %2d %8.8s %8.8s %8.8s %16.16s\n",DDEF(i).flag,
	DDEF(i).col, DDEF(i).dim, DDEF(i).rflg, DDEF(i).div,
	DDEF(i).name, DDEF(i).nam2, DDEF(i).nam3, DDEF(i).text);
}

fclose(f);
}
		
  /* Free used memory */

omn=menus;
while(omn)
{
	struct _menu *mn;
	struct _mlst *ml;
	
	ml=omn->mlst;
	while(ml)
	{
		struct _mlst *ml1;
		ml1=ml->next;
		free(ml);
		ml=ml1;
	}
	
	mn=omn->next;
	free(omn);
	omn=mn;
}

otbl=tables;
while(otbl)
{
	struct _table *tbl;
	struct _rrel *rel;
	
	rel=otbl->rrel;
	while(rel)
	{
		struct _rrel *rel1;
		rel1=rel->next;
		free(rel);
		rel=rel1;
	}
	
	tbl=otbl->next;
	free(otbl);
	otbl=tbl;
}


}

static int next_prime(int n)
{
	if(!(n%2))
		n++;
	while(!((n%3) && (n%5) && (n%7) && (n%11) && (n%13) && (n%19) && (n%23)))
		n+=2;
	return n;
}

static int wrt_tbl(struct _table *otbl, FILE *fout)
{
		
		struct _rrel *orel=NULL;
		char cbuf[180];
		int i;
		int kno=0;
		int rno=0;
		int pos=0;
		
		strncpy(cbuf,DDEF(otbl->first).text,80);
		f3c(cbuf,80);
		fprintf(fout,"\nDefineLedaTable( %s, %d, '%s',\n",
			otbl->name,next_prime(DDEF(otbl->first).dim),cbuf);
		
		for(i=otbl->first+1;i<=otbl->last;i++)
		{
			if(DDEF(i).flag==5)
			{
				char t;
				t=DDEF(i).nam2[0];
				switch(t)
				{
					case 'i':
					case 'I':
							fprintf(fout,"\tDBint   "); break;
					case 'f':
					case 'F':
							fprintf(fout,"\tDBfloat "); break;
					case 'h':
					case 'H':
					case 'c':
					case 'C':
							fprintf(fout,"\tDBascii "); break;
					case 'b':
					case 'B':
							fprintf(fout,"\tDBuint  "); break;
					case 's':
					case 'S':
							fprintf(fout,"\tDBshort "); break;
					case 'u':
					case 'U':
							fprintf(fout,"\tDBushort "); break;
					default:
						{
							static int errPrt;
							static char errMsg[80];
							sprintf(errMsg,
							"ddmkleda: table %s: unknown attribute type %c.\n",
									otbl->name,t);
							mecErrorCount(&errPrt,errMsg);
							return 0;
						}
							
				}
				
				strncpy(cbuf,DDEF(i).name,80);
				f2c(cbuf,80);
				pos=fprintf(fout,"%s",cbuf);
				
				if(t != 'h' && t != 'H' && t != 'c' && t != 'C' && DDEF(i).dim==0)
					pos+=fprintf(fout,";  ");
				else if(t != 'h' && t != 'H' && t != 'c' && t != 'C')
					pos+=fprintf(fout,"[%d];",DDEF(i).dim);
				else
					pos+=fprintf(fout,"[%d];",(DDEF(i).dim?DDEF(i).dim:1)*4);
				strncpy(cbuf,DDEF(i).text,80);
				f3c(cbuf,80);
				write_blank(fout,12-pos);
				if(cbuf[0])
					fprintf(fout," /* %s */\n",cbuf);
				else
					fprintf(fout,"\n");
				if(DDEF(i).key)
					kno++;
				continue;
			}
			if(DDEF(i).flag>6 && DDEF(i).flag<11)
			{
				int pos=0;
				strncpy(cbuf,DDEF(i).name,80);
				f2c(cbuf,80);
				if(DDEF(i).flag==7 || DDEF(i).flag==9)
					pos=fprintf(fout,"\tDBrelone  %s;",cbuf);
				else
					pos=fprintf(fout,"\tDBrelmany %s;",cbuf);
				strncpy(cbuf,DDEF(i).text,80);
				f3c(cbuf,80);
				write_blank(fout,21-pos);
				if(cbuf[0])
					fprintf(fout," /* %s */\n",cbuf);
				else
					fprintf(fout,"\n");
				rno++;
				continue;
			}
			
		}
		
		
		fprintf(fout,"\t);\n\n");
		
		fprintf(fout,"SetLedaArteCompatibility(%s);\n\n",otbl->name);

		if(kno==0)
			fprintf(fout,"SetLedaImplicitKey(%s);\n\n",otbl->name);
		else
		{
			fprintf(fout,"SetLedaKey(%d) (%s,\n",kno,otbl->name);
			for(i=otbl->first+1;i<=otbl->last;i++)
			{
				if(DDEF(i).flag==5 && DDEF(i).key)
				{
					char t;
					kno--;
					t=DDEF(i).nam2[0];
					switch(t)
					{
						case 'i':
						case 'I':
								fprintf(fout,"\tDBint, "); break;
						case 'f':
						case 'F':
								fprintf(fout,"\tDBfloat, "); break;
						case 'h':
						case 'H':
						case 'c':
						case 'C':
								fprintf(fout,"\tDBascii, "); break;
						case 'b':
						case 'B':
								fprintf(fout,"\tDBuint, "); break;
						case 's':
						case 'S':
								fprintf(fout,"\tDBshort, "); break;
						case 'u':
						case 'U':
								fprintf(fout,"\tDBushort, "); break;
					}

					strncpy(cbuf,DDEF(i).name,80);
					f2c(cbuf,80);
					fprintf(fout,"%s",cbuf);

					if(t=='h' || t=='H' || t=='c' || t=='C' || DDEF(i).dim)
						fprintf(fout,"[0]");
					if(kno)
						fprintf(fout,",\n");
					else
						fprintf(fout,");\n\n");
				}
			}
		}
		
		rno=0;
		for(orel=otbl->rrel;orel;orel=orel->next)
			if(!orel->sametbl)
				rno++;
		
		if(rno)
		{
			struct _rrel *rel;
			
			fprintf(fout,"SetLedaRelation(%d) (%s,\n",
					rno,otbl->name);
			
			for(rel=otbl->rrel;rel;rel=rel->next)
			{
				if(rel->sametbl)
					continue;
				rno--;
				fprintf(fout,"\t%s, %s, %s, %s",
						rel->fname,
						rel->itbl->name,rel->invr->fname,
						rel->tname);
				if(rno)
					fprintf(fout,",\n");
				else
					fprintf(fout,");\n\n");
			}
		}
	return 1;

}	
