//                              -*- Mode: C++ -*-
//: ChronoManager.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 3/4/97
// Last Update ........
//
#ifndef CHRONOMANAGER_HH_
#define CHRONOMANAGER_HH_

//#include <string>
#include <list.h>


class Chrono;  // tell compiler that somewhere a Chrono class exists

/*
CLASS
  ChronoManager
     Singleton class that is used to store Chrono objects.

RESPONSABILITIES
  <pre>
  It keeps a list of all used clocks.
  It can create Chronos on request.
  It can print information for all clocks.
  </pre>

OPERATIONS
  createChrono()
  deleteChrono()
  printInformations()

WARNING
  Sometimes including the header file for this class (ChronoManager.hh) can 
  give rise to compile warnings. These can be avoided if the header is
  the first to be included.

*/

class ChronoManager {
public:
  //--  Default Constructor
  // 
  ChronoManager();
  ~ChronoManager();  

  //--
  //  Initialization of the ChronoManager is done calling this
  //  function
  static void instance();

  //-- Operations
  // Return a brand new Chrono
  Chrono* createChrono(char*s);
  //--
  // Return a brand new Chrono
  //  Chrono* createChrono(string &s);
  //--
  // Delete the current chrono
  void deleteChrono(Chrono* ch);

  // --
  // Delete ChronoManager static data. Calling this function
  // all stored Chronos are deleted and active ChronoManager instance
  // becomes invalid
  static void deleteInstance(); 


  //--
  // Print informations for all stored Chronos
  void printInformations();

protected:
private:
  /////////////////////////// Data Part /////////////////////
  //-- 
  // static pointer to a list of Chronos 
  static list<Chrono*> * plist;
  // static number 
  static int numberOfChronoManagers;
};


#endif // CHRONOMANAGER_HH_





