//                              -*- Mode: C++ -*-
//: CmdCareECDBMIZ.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#include "care/CmdCareECDBMIZ.hh"
#include "arte/arte.hh"
#include "care/care.hh"
#include "arte/ArteMessage.hh"
#include "care/ECHF_status.hh"
#include "care/caredata.hh"
#include "care/MMBank.hh"

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include "care/EcalDB.hh"

/////////////// The function seen by kuip
#ifdef USEKUIP
extern "C" {

  void ecdbmiz_(){
    CmdCareECDBMIZ boardfile;
    boardfile.readAndExe();
    ECalInfo.dbflag = 2;
  }
}

#endif

/////////////// Static data definition

char CmdCareECDBMIZ::filename[256] = "\0";

/////////////// CmdCareECDBMIZ class member function definition

CmdCareECDBMIZ::CmdCareECDBMIZ(){}
CmdCareECDBMIZ::~CmdCareECDBMIZ(){}

void CmdCareECDBMIZ::defValues(){
  strcpy(filename,"ECAL/EcalCalibMiz.db");
  action = none;
  match = 0;
  major = 1;
  minor = 10;
  tbset = TALL;
  info.firstTime = 0;
  info.lastTime = 0;
  info.storageTime = 0;
  info.version = 0;
  strcpy(info.name," ");

}


//--
//   Execute stored command // read or write from the DB
void CmdCareECDBMIZ::execute(){
  switch(action){
  case dbread:
    readDB(tbset, match);
    break;
  case dbwrite:
    writeDB(tbset, match);
    break;
  default:
    break;
  }
}

#ifdef USEKUIP
//--
//   Read acommand from external
void CmdCareECDBMIZ::read(){
  char* name = readString();
  int i=0;
  for(; i<256 && name[i]!=' ' && name[i]!='\0'; i++)
    filename[i] = name[i];
  filename[i] = '\0';

  // reading the TB set
  int itbset = readInt();
  if( itbset>=0 && itbset<=(int)THOT ) tbset = (CmdCareECDBMIZ::Tbset)itbset;
  
  int act = readInt();
  action = none;
  if( act==1) action = dbread;
  else if( act==2) action = dbwrite;
  //
  // DB version numbers
  //
  major = readInt();
  minor = readInt();

  // 
  // Match parameter 
  //
  char* smatch = readUppercaseString();
  const char tests[] = "EQGTLTGELE";
  for(i=0; i<5; i++){
    if( !strncmp(smatch,tests+i*2,2) ) break;
  }
  if( i<5 ) match = i;
}
#endif

char* CmdCareECDBMIZ::calibfile(){
  if( filename[0]==0) 
    defValues();
  return filename;
}

void CmdCareECDBMIZ::dump(){
  arteMsg<<"Calibration file name :"<<filename;
  arteMsg.allp();
  arteMsg<<" Tables:"<<tbset<<" action:"<<action
         <<" Tags: "<<major<<","<<minor<<"  match:"<<match;
  arteMsg.allp();
}


void CmdCareECDBMIZ::setInfo(int itime1, int itime2,
			  char* iname, int iversion){
  info.firstTime = itime1;
  info.lastTime = itime2;
  info.storageTime = 0;
  info.version = iversion;
  strncpy(info.name,iname,12);
}


void CmdCareECDBMIZ::readDB(Tbset whichtable, int access){

  int mymajor = -1;
  int myminor = -1;

  arteMsg<< "Opening  " << calibfile();
  arteMsg.allp();
  EcalDB & mydb = *EcalDB::instance();
  mydb.Open(calibfile(), 0);

  switch(whichtable){
  case TSLOW:
    arteMsg<<"Reading calibration data (SLOW): ";
    arteMsg.allp();
    readECBO( mydb, access);
    readECHS( mydb, access);
    break;
  case TECBO:
    arteMsg<<"Reading calibration data (ECBO): ";
    arteMsg.allp(); 
    readECBO( mydb, access);
    break;
  case TECHS:
    arteMsg<<"Reading calibration data (ECHS): ";
    arteMsg.allp();
    readECHS( mydb, access);
    break;
  case TFAST:
    readECHF( mydb, access);
    break;
  case TPED:
    readECPD( mydb, access);
    break;
  case TSENS:
    readECSN( mydb, access);
    break;
  case THOT:
    readECHT( mydb, access);
    break;
  case TALL:
    arteMsg<<"Reading calibration data (ALL): ";
    arteMsg.allp();    
    readECBO( mydb, access);
    readECHS( mydb, access);
    readECHF( mydb, access);
    break;
  }
  arteMsg << "Closing " << calibfile();
  arteMsg.allp();
  mydb.Close();
  minor = myminor;
  major = mymajor;
}

void CmdCareECDBMIZ::writeDB(Tbset whichtable, int access){
  arteMsg<<"Opening "<<calibfile();
  arteMsg.allp();
  cout<<"Before EcalDB constructor"<<endl;

  EcalDB & mydb = *EcalDB::instance();
  cout<<"Before EcalDB::Open"<<endl;
  mydb.Open(calibfile(), 0);
  cout<<"Open done! Banks to write:"<<whichtable<<endl;
  switch(whichtable){
  case TSLOW:
    writeECBO(mydb, access);
    writeECHS(mydb, access);
    break;
  case TECBO:
    writeECBO(mydb, access);
    break;
  case TECHS:
    writeECHS(mydb, access);
    break;
  case TFAST:
    writeECHF(mydb, access);
    break;
  case TPED:
    writeECPD( mydb, access);
    break;
  case TSENS:
    writeECSN( mydb, access);
    break;
  case THOT:
    cout<<"Tring to write THOT"<<endl;
    writeECHT( mydb, access);
    break;
  case TALL:
    writeECBO(mydb, access);
    writeECHS(mydb, access);
    writeECHF(mydb, access);
    break;
  }
  arteMsg << "Closing " << calibfile();
  arteMsg.allp();
  mydb.Close();
}

void CmdCareECDBMIZ::setDB( char *name, int imaj, int imin){
  major = imaj;
  minor = imin;
  int i=0;
  for(; i<256 && name[i]!=' ' && name[i]!='\0'; i++)
      filename[i] = name[i];
  filename[i] = '\0';
}

 
void  CmdCareECDBMIZ::readECBO(EcalDB & mydb, int access){

  MMBank<ECBO>::Drop();
  mydb.ECBO.Book();
  
  if( mydb.ECBO.Match(major, minor,access)==0 ){
    arteMsg<<"This ECBO table version does not exists";
    arteMsg.warn();
  } else {
    MMBank<ECBO>::Book(mydb.ECBO.Nrow(major, minor,access));
    mydb.ECBO.Fetch(&(*(ArtePointer<ECBO>(1))));
    arteMsg<<"ECBO: Tags ("<< mydb.ECBO.Major()<<","<<mydb.ECBO.Minor()
	   <<")   Entries: "<<ArtePointer<ECBO>::nrow();
    arteMsg.allp();
    mydb.ECBO.Unbook();
    info = mydb.ECBO.Info();
  }
}

void  CmdCareECDBMIZ::readECHS(EcalDB & mydb, int access){
  MMBank<ECHS>::Drop();
  mydb.ECHS.Book();
  if( mydb.ECHS.Match(major, minor,access)==0){
    arteMsg<<"This ECHS table version does not exists";
    arteMsg.warn();
  } else {
    
    MMBank<ECHS>::Book(mydb.ECHS.Nrow(major, minor,access));
    mydb.ECHS.Fetch(&(*(ArtePointer<ECHS>(1))));
    arteMsg<<"ECHS: Tags ("<< mydb.ECHS.Major()<<","<<mydb.ECHS.Minor()
	   <<")   Entries: "<<ArtePointer<ECHS>::nrow();
    arteMsg.allp();
    mydb.ECHS.Unbook();
    info = mydb.ECHS.Info();
  }
}

void  CmdCareECDBMIZ::readECHF(EcalDB & mydb, int access){
  readECPD( mydb, access);
  readECSN( mydb, access);
  readECHT( mydb, access);
  makeECHF();
}


void  CmdCareECDBMIZ::makeECHF(){
  cout<<"Now making ECHF with "<<ArtePointer<ECPD>::nrow()<<" entries"<<endl;
  cout<<"Dropping"<<endl;
  MMBank<ECHF>::Drop();
  cout<<"Booking "<<endl;
  //  MMBank<ECHF>::Book(ArtePointer<ECPD>::nrow());  
  MMBank<ECHF>::Book(5956);
  ArteTable<ECHF>::iterator echf=ArteTable<ECHF>::begin();
  ArteTable<ECPD>::iterator echped=ArteTable<ECPD>::begin();
  ArteTable<ECSN>::iterator echsens=ArteTable<ECSN>::begin();
  for(; echf!=ArteTable<ECHF>::end(); ++echf, ++echped, ++echsens){
    //cout<<" Channel "<<echf<<endl;
    echf->stat = (echped->stat & 0xFFFF)+ ((echsens->stat & 0xFFFF)<<16);
    if( (echsens->stat & 1)==0 ) echf->stat =echf->stat & 0xFFFFFFFE; 
    echf->pedf = echped->pedf;
    echf->wid = echped->wid;
    echf->sens = echsens->sens;
    echf->esen = echsens->esen;
  }
  int entries = ArtePointer<ECHF>::nrow();
  for(ArteTable<ECHT>::iterator echotc=ArteTable<ECHT>::begin(); 
      echotc!=ArteTable<ECHT>::end(); ++echotc){
    if( echotc->chan>0 && echotc->chan<=entries ){
      ArtePointer<ECHF> echfx(echotc->chan);
      echfx->stat =  echfx->stat & 0xFFFFFFFE;
    }
  }
}

void  CmdCareECDBMIZ::readECPD(EcalDB & mydb, int access){
  MMBank<ECPD>::Drop();
  mydb.ECPD.Book();
  if( mydb.ECPD.Match(major, minor,access)==0){
    arteMsg<<"This ECPD table version does not exists";
    arteMsg.warn();
  } else {
    MMBank<ECPD>::Book(mydb.ECPD.Nrow(major, minor,access));
    mydb.ECPD.Fetch(&(*(ArtePointer<ECPD>(1))));
    arteMsg<<"ECPD: Tags ("<< mydb.ECPD.Major()<<","<<mydb.ECPD.Minor()
	   <<")   Entries: "<<ArtePointer<ECPD>::nrow();
    arteMsg.allp();
    mydb.ECPD.Unbook();
    info = mydb.ECPD.Info();
  }
}

void  CmdCareECDBMIZ::readECSN(EcalDB & mydb, int access){
  MMBank<ECSN>::Drop();
  mydb.ECSN.Book();
  if( mydb.ECSN.Match(major, minor,access)==0){
    arteMsg<<"This ECSN table version does not exists";
    arteMsg.warn();
  } else {
    MMBank<ECSN>::Book(mydb.ECSN.Nrow(major, minor,access));
    mydb.ECSN.Fetch(&(*(ArtePointer<ECSN>(1))));
    arteMsg<<"ECSN: Tags ("<< mydb.ECSN.Major()<<","<<mydb.ECSN.Minor()
	   <<")   Entries: "<<ArtePointer<ECSN>::nrow();
    arteMsg.allp();
    mydb.ECSN.Unbook();
    info = mydb.ECSN.Info();
  }
}

void  CmdCareECDBMIZ::readECHT(EcalDB & mydb, int access){

  MMBank<ECHT>::Drop();
  mydb.ECHT.Book();
  if( mydb.ECHT.Match(major, minor,access)==0){
    arteMsg<<"This ECHT table version does not exists";
    arteMsg.warn();
  } else {
    int entries = mydb.ECHT.Nrow(major, minor,access);
    if( entries>0) {
      MMBank<ECHT>::Book(entries);
      mydb.ECHT.Fetch(&(*(ArtePointer<ECHT>(1))));
    }
    arteMsg<<"ECHT: Tags ("<< mydb.ECHT.Major()<<","<<mydb.ECHT.Minor()
	   <<")   Entries: "<<ArtePointer<ECHT>::nrow();
    arteMsg.allp();
    mydb.ECHT.Unbook();
    info = mydb.ECHT.Info();
    arteMsg<<"ECHT: infos ("<< info.storageTime<<","<<info.name<<")";

    arteMsg.allp();
  }
}

void  CmdCareECDBMIZ::writeECBO(EcalDB & mydb, int access){
  
  mydb.ECBO.Book();
  mydb.ECBO.Store(ArtePointer<ECBO>::nrow(),
		  &(*(ArtePointer<ECBO>(1))),
		  major, minor, info, access);
  mydb.ECBO.Unbook();
}

void  CmdCareECDBMIZ::writeECHS(EcalDB & mydb, int access){
  
  mydb.ECHS.Book();
  
  mydb.ECHS.Store(ArtePointer<ECHS>::nrow(),
		  &(*(ArtePointer<ECHS>(1))),
		  major, minor, info, access);
  mydb.ECHS.Unbook();
}

void  CmdCareECDBMIZ::writeECHF(EcalDB & mydb, int access){
  writeECPD(mydb, access);
  writeECSN(mydb, access);
  writeECHT(mydb, access);
}


void  CmdCareECDBMIZ::writeECPD(EcalDB & mydb, int access){
  
  mydb.ECPD.Book();
  
  mydb.ECPD.Store(ArtePointer<ECPD>::nrow(),
		  &(*(ArtePointer<ECPD>(1))),
		  major, minor, info, access);
  mydb.ECPD.Unbook();
}

void  CmdCareECDBMIZ::writeECSN(EcalDB & mydb, int access){
   
  mydb.ECSN.Book();
  
  mydb.ECSN.Store(ArtePointer<ECSN>::nrow(),
		  &(*(ArtePointer<ECSN>(1))),
		  major, minor, info, access);
  mydb.ECSN.Unbook();
}

void  CmdCareECDBMIZ::writeECHT(EcalDB & mydb, int access){
  cout<<" Booking "<<endl;
  mydb.ECHT.Book();
  cout<<" Storing "<<endl;
  mydb.ECHT.Store(ArtePointer<ECHT>::nrow(),
		  &(*(ArtePointer<ECHT>(1))),
		  major, minor, info, access);
  cout<<" unbooking"<<endl;
  mydb.ECHT.Unbook();
}
