//                              -*- Mode: C++ -*-
//:EcalDB.C
//
//
// Author: Vincenzo Vagnoni & Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
//

#include <time.h>
#include "care/EcalDB.hh"

//#include <iostream.h>

// ------------ Static data definitions

char *EcalDB::tableName[5]  = { "ECBO", "ECHS", "ECPD", "ECSN", "ECHT" };
char EcalDB::infoDesc[] = "I:INFO-COLUMN I:firstTime I:lastTime I:storageTime A:name I:version \0";
int   EcalDB::infoDim[6] = {1,1,1,1,12,1};
int   EcalDB::infoType[6] = {TABLEint, TABLEint, TABLEint, TABLEint,
			     TABLEascii, TABLEint};
 
char *EcalDB::tableDesc[5] = {
  // ECBO description string for Mizzi
  "I:FDID I:STAT I:NUMB I:LINK I:CHAN",
  // ECHS description string for Mizzi
  "I:MOD F:X F:Y F:R I:FDID I:CHAN I:ECBO F:PCAL F:ECAL F:PEDS F:THRS",
  // ECPD description string for Mizzi
  "I:STAT F:PEDF F:WID",
 // ECSN description string for Mizzi
  "I:STAT F:SENS F:ESENS",
 // ECHT description string for Mizzi
  "I:STAT I:CHAN"
};

char *EcalDB::tableComment[5]= {
    "Ecal readout BOard data",
    "Ecal readout CHannel Calibration table (Slow) - Slow update rate",
    "Ecal readout CHannel Status and Pedestal table ",
    "Ecal readout CHannel Sensitivity table",
    "Ecal readout HOT Channel table"
  };
int   EcalDB::fieldDim[5][11] = {
    // ECBO field dimensions
    { 1, 1, 1, 1, 32 },
    // ECHS field dimensions
    { 1, 1, 1, 1, 1, 1, 1, 6, 6, 1, 1},
    // ECPD field dimensions
    { 1, 1, 1 },
    // ECSN field dimensions
    { 1, 1, 1 },
    // ECHT field dimensions
    { 1, 1}
  };

int   EcalDB::fieldNum[5] = { 5, 11, 3, 3, 2 };
int   EcalDB::fieldType[5][11]  = {
    // ECBO field types
    { TABLEint, TABLEint, TABLEint, TABLEint, TABLEint },
    // ECHS field types
    { TABLEint,   TABLEfloat, TABLEfloat, TABLEfloat,
      TABLEint,   TABLEint,   TABLEint,
      TABLEfloat, TABLEfloat, TABLEfloat, TABLEfloat},
    // ECPD field types
    { TABLEint, TABLEfloat, TABLEfloat },
    // ECSN field types
    { TABLEint, TABLEfloat, TABLEfloat },
    // ECHT field types
    { TABLEint, TABLEint }
  };


EcalDB *EcalDB::theEcalDB =0;

EcalDB::EcalDB() : ECBO(0,*this), ECHS(1,*this), ECPD(2,*this),
    ECSN(3,*this), ECHT(4,*this) {};

EcalDB::~EcalDB() {};


Tables EcalDB::Open(char *dbName, int access) {

  x=OpenTables(dbName, access);

  return x;
};

int EcalDB::Close(void) {
  int ret=CloseTables(x);
  x=NULL;
  return ret;
};



EcalDB::TableMeth::TableMeth(int i, EcalDB &T) : 
  xeCur(i), EcalDBase(T), tSize(0), nrow(0), major(0), minor(0), info() {

  for(int j=0; j<EcalDBase.fieldNum[xeCur]; j++) {
    tSize += EcalDBase.fieldDim[xeCur][j]*(fieldType[xeCur][j] & 0xF);
  }

};

EcalDB::TableMeth::~TableMeth() {};

int EcalDB::TableMeth::Book() {

  char buffer[256];
  buffer[0] = '\0';
  strcpy(buffer, &EcalDBase.infoDesc[0]);
  strcat(buffer, &(EcalDBase.tableDesc[xeCur][0]));

  EcalDBase.xe[xeCur]=
    BookTable(EcalDBase.x,
	      EcalDBase.tableName[xeCur],
	      buffer,
	      EcalDBase.tableComment[xeCur]);
  return EcalDBase.xe[xeCur];

}

int EcalDB::TableMeth::Unbook() {
  int ret;
  ret=UnbookTable(EcalDBase.x,EcalDBase.xe[xeCur]);
  EcalDBase.xe[xeCur]=0;
  return ret;
}

int EcalDB::TableMeth::Store
(int nrow, void *ptr, int maj, int min, const TableInfo & inf, int access) {

  char *table=(char*)ptr;
  info = inf;
  //  struct timeval tv;
  //  gettimeofday(&tv, NULL );
  time_t tim = time(NULL);
  info.storageTime = tim;

  TableStamp(EcalDBase.x,maj,min);
  SkipTableField(EcalDBase.x,1);
  PutTableField(EcalDBase.x,TABLEint,1,&nrow);
  PutTableField(EcalDBase.x,TABLEint,1,&info.firstTime);
  PutTableField(EcalDBase.x,TABLEint,1,&info.lastTime);
  PutTableField(EcalDBase.x,TABLEint,1,&info.storageTime);
  PutTableField(EcalDBase.x,TABLEascii,12,&info.name[0]);
  PutTableField(EcalDBase.x,TABLEint,1,&info.version);

  for(int i=0; i<fieldNum[xeCur]; i++) {
    int entries = fieldDim[xeCur][i];
    int dim = entries *(fieldType[xeCur][i] & 0xF);
    char *buffer=new char[dim*nrow];
    int bufferCursor=0;
    int tableCursor=0;
    for(int j=0; j<nrow; j++) {
      memcpy(&buffer[bufferCursor],&table[tableCursor],dim);
      bufferCursor+=dim;
      tableCursor+=tSize;
    }

    PutTableField(EcalDBase.x,EcalDBase.fieldType[xeCur][i],
		  nrow*entries,buffer);
    delete buffer;

    table+=dim;
  }

  if(!StoreTable(EcalDBase.x,EcalDBase.xe[xeCur],access)) return 0;
  major=TableMajor(EcalDBase.x);
  minor=TableMinor(EcalDBase.x);

  return 1;
}

int EcalDB::TableMeth::Nrow(int maj, int min, int access) {
  TableStamp(EcalDBase.x,maj,min);
  if(!FetchTable(EcalDBase.x,EcalDBase.xe[xeCur],access)) return 0;

  SkipTableField(EcalDBase.x,1);
  GetTableField(EcalDBase.x,TABLEint,1,&nrow);
  GetTableField(EcalDBase.x,TABLEint,1,&info.firstTime);
  GetTableField(EcalDBase.x,TABLEint,1,&info.lastTime);
  GetTableField(EcalDBase.x,TABLEint,1,&info.storageTime);
  GetTableField(EcalDBase.x,TABLEascii,12,&info.name[0]);
  GetTableField(EcalDBase.x,TABLEint,1,&info.version);
  return nrow;
}

int EcalDB::TableMeth::Fetch(void *ptr) {

  char *table=(char*)ptr;

  for(int i=0; i<fieldNum[xeCur]; i++) {
    int entries = fieldDim[xeCur][i];
    int dim = entries*(fieldType[xeCur][i] & 0xF);
    char *buffer=new char[dim*nrow];
    int bufferCursor=0;
    int tableCursor=0;

    GetTableField(EcalDBase.x,EcalDBase.fieldType[xeCur][i],nrow*entries,buffer);

    for(int j=0; j<nrow; j++) {
      memcpy(&table[tableCursor],&buffer[bufferCursor],dim);
      bufferCursor+=dim;
      tableCursor+=tSize;
    }

    delete buffer;

    table+=dim;
  }
  major=TableMajor(EcalDBase.x);
  minor=TableMinor(EcalDBase.x);
  return 1;

}

int EcalDB::TableMeth::Match(int maj, int min, int access) {
  TableStamp(EcalDBase.x,maj,min);
  if(!MatchTable(EcalDBase.x,EcalDBase.xe[xeCur],access)) return 0;
  major=TableMajor(EcalDBase.x);
  minor=TableMinor(EcalDBase.x);
  return 1;
}

int EcalDB::TableMeth::Major() const {
  return major;
}

int EcalDB::TableMeth::Minor() const {
  return minor;
}

const EcalDB::TableInfo & EcalDB::TableMeth::Info() const {
  return info;
}
