//                              -*- Mode: C++ -*-
//: CmdCareECDB.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 1/4/98
// Last Update ........
//

#ifndef CMDCAREECDB_HH_
#define CMDCAREECDB_HH_

#include "care/CmdBase.hh" 
#include "care/EcalDB.hh"

class ifstream;
class ofstream;


/*
CLASS
  CmdCareECDB
  Class to deal with the care/ECDB kuip command
  Define the name of the ASCII file containing the calibration DB

*/

class CmdCareECDB : public CmdBase {
public:
  enum Tbset {TALL, TECBO, TECHS, TPED, TSENS, THOT};

  //-- Constructors
  CmdCareECDB();
  //-- Destructor
  // Destructor: currently void
  virtual ~CmdCareECDB();
  
  //-- Behavioural operations
  //   Reset stored data to default values
  virtual void defValues();
  //--
  //   Execute stored command
  virtual void execute();

# ifdef USEKUIP
  //--
  //   Read acommand from external
  virtual void read();
# endif
  //--
  //   Dump internal data
  virtual void dump();
  //--
  //   Static access member function
  char* calibfile();


  //--
  // Read the database
  void readDB(Tbset whichtable);
  //--
  // Write the database
  void writeDB(Tbset whichtable);

  void setDB(char *nam, int imajor, int iminor);

  //  Set the main Infos
  //-- 
  void setInfo(int time1, int time2, int time3, const char* name, int version);

  // get the new info
  inline const EcalDB::TableInfo & Info() const;

  //-- 
  //  Get functions
  inline int Major() const;
  inline int Minor() const;

private:
 
  int readECBO(ifstream & is);
  int readECHS(ifstream & is);
  int readECHF(ifstream & is);
  int readECPD(ifstream & is);
  int readECSN(ifstream & is);
  int readECHT(ifstream & is);

  int writeECBO(ofstream & os);
  int writeECHS(ofstream & os);
  int writeECHF(ofstream & os);
  int writeECPD(ofstream & os);
  int writeECSN(ofstream & os);
  int writeECHT(ofstream & os);
  void writeInfo(ofstream & os, const char *tablename);
  void makeECHF();

  int gccenum( int fed, int chan);
  ///////////////////// Data Part /////////////////////////////////
  //--
  static  char filename[256];
  enum  {none, dbread, dbwrite} action;
  int major;
  int minor;
  EcalDB::TableInfo info;
  int verbose;
};

int CmdCareECDB::Major() const {
    return major;
}

int CmdCareECDB::Minor() const {
    return minor;
}

// get the new info
const EcalDB::TableInfo & CmdCareECDB::Info() const{
  return info;
}

#endif  //   CMDCAREECDB_HH_

