//                              -*- Mode: C++ -*-
//: ESPID04.hh
//
//
// Author: Spighi, Villa
// ECAL group 
// HERA-B experiment, Desy
// e-mail: spighi@bo.infn.it, villa@bo.infn.it
//
// Created 3/7/97
// Last Update ........
//

#ifndef ESPID04_HH_
#define ESPID04_HH_
#include "care/ESPIDStrategy.hh"

/**
class  ESPID04.
   Class for PID (pion/electron discrimination) analysis: 
   it evaluates particular quantities to separate pion and electron
   signals using the different lateral spread of the clusters. 
   In particular the program evaluate te quantity
                      Rpid = (sum on towers ri*(Ei**0.4))
		             -----------------------------
                             (sum on towers Ei**0.4)

  Reference: N.I.M.  A305 (1991) 55-70 
*/

class ESPID04 : public ESPIDStrategy {
public:
  /// Constructor
  ESPID04();
  /// Destructor
  virtual ~ESPID04();
  //--
  /// ClusterTree Analyzer
  virtual void doAnalysis(const EMClusterTree* tree);
  //--
  /// ClusterMember Analyzer
  virtual void doAnalysis(const EMClusterMember* memb);
  //--
  /// EMTrackOnEcal Analyzer   
  virtual void doAnalysis(const EMTrackOnEcal* track);
  //--
  /// Cluster Analyzer
  virtual void doAnalysis(const EMCluster* clus);
  //--
  /// EMHitTower Analyzer
  virtual void doAnalysis(const EMHitCell* hitC);
  //--
  ///  Start a cluster analysis
  virtual void beginAnalysis(const EMCluster* clus, ArtePointer<RCCL> rccl,
			     const EMHitCell* hitc );
  //--
  /// Evaluation likelihood for electron
  virtual void likeElectron();

  /// Evaluation likelihood for hadron
  virtual void likeHadron();

  /// set the parameter of likelihood curves for electrons and pions
  static void setparlike04(int cal,int particle,float like04[]);

private:
  float  energySum04;
  float rsum04;
  //
  static float eparinner[9];
  static float pparinner[9];
  static float eparmiddle[9];
  static float pparmiddle[9];
  static float eparouter[9];
  static float pparouter[9];
};

#endif // ESPID04_HH_

































