//                              -*- Mode: C++ -*-
//: ESPosition.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#ifndef ESPOSITION_HH_
#define ESPOSITION_HH_
#include "care/ECALtypes.hh"
class EMHitCell;


//--
///  Types of positions that have different time and resolution performances.
enum PosTyp {Peak, COG, E_CorCOG, P_CorCOG }; 

/**
class  ESPosition.
  This class is used to store informations on showers:
  a point of the shower, its energy, the number of data and a
  link to the most energetic cell
*/


class ESPosition {
public:

  //
  /// Default constructor: It initialize all data to zero
  ESPosition();
  //
  /// Copy constructor
  ESPosition( const ESPosition & pos);
  //-- Destructor
  /// Destructor: currently void
  ~ESPosition();
  //--
  ///  Operator function for assigning the pos struct:
  const ESPosition & operator = (const ESPosition & pos);
  
  //-- Behavioural operations
  ///   Reset stored data to default
  inline void reset();
  //   Access functions 
  //--
  /// direct access to x, y and z
  inline void setPoint(float x, float y,float z){ 
    xf = x; yf = y; zf = z; };

  inline void setVar(float vxx, float vxy, float vyy){ 
    varf[0] = vxx; varf[1] = vxy; varf[2] = vyy; }
  inline void setEnergy( Energy_t energy){ energyf = energy;};
  inline void setSqEnErr( Energy_t sqenerr){ sqenerrf = sqenerr;};
  inline void setWidth(int width){ ndata = width;};
  inline void setType(PosTyp itype){ type = itype;};
  inline void setCell(const EMHitCell * icell){ maxHitC = icell;};

  //   Access functions 
  //--
  /// direct access to x, y and z
  inline Coord_t  x() const { return xf;}
  inline Coord_t  y() const { return yf;}
  inline Coord_t  z() const { return zf;}
  inline Coord_t  var(int i) const { return varf[i];};
  inline Energy_t energy() const { return energyf;};
  inline Energy_t sqenerr() const { return sqenerrf;};

  inline unsigned int width() const { return ndata;};
  inline PosTyp posType() const { return type;};
  inline  const EMHitCell* cell() const { return maxHitC;};

protected:
  //     Data Part 
  //--
  /// The position data
   //--
  /// public structure that is used to store position data.
  /// Coordinate of a point on ECAL
  Coord_t zf;
  Coord_t xf;
  Coord_t yf;
  /// Variances on x,y
  Coord_t varf[3];  
  /// Energy and its squared error
  Energy_t energyf;
  Energy_t sqenerrf;
  ///
  unsigned int ndata;
  PosTyp  type;
  const EMHitCell* maxHitC;

};

#include "care/ESPosition.icc"
#endif  //   ESPOSITION_HH_
