//
//  Authors:  Silvia Masciocchi                     Created:  05-Feb-99
//            Ivan Kisel                            Modified: 07-Jun-99
//            Max-Planck-Institut fuer Physik                 01-Jul-99
//            Munchen        
//  Email:    sma@mppmu.mpg.de ikisel@mppmu.mpg.de  HERA-B Collaboration
//
// fill_banks.C
// VDS reconstruction program - filling banks after tracking program Cats.
// Called from catsevnt().
//

#include <vector>

#include <iostream.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"

#define MAXNUMBEROFHITS     20000
#define MAXNUMBEROFTRACKS   4000

#ifdef XXX
extern int   Ntk, nh_trk[MAXNUMBEROFTRACKS+1];
extern float Xp_trk[MAXNUMBEROFTRACKS+1], X0_trk[MAXNUMBEROFTRACKS+1];
extern float Yp_trk[MAXNUMBEROFTRACKS+1], Y0_trk[MAXNUMBEROFTRACKS+1];
static float C_trk[10*(MAXNUMBEROFTRACKS+1)], Chi2_trk[MAXNUMBEROFTRACKS+1];
extern int   hits[MAXNUMBEROFHITS+MAXNUMBEROFTRACKS+10];
#endif

extern "C" {
  void fill_banks(int Ntk, int *nh_trk, float *Xp_trk, float *X0_trk, float
		  *Yp_trk, float *Y0_trk, int *hits,
		  float *C_trk, float *Chi2_trk){
    
    int nh, itk, ih, i;
    vector<ArtePointer<RHIT> > rhits;
    RSEG r;
    RHIT rh;
   
    /*fill the RSEG table*/
    
    nh = 0;
    for (itk=0;itk<Ntk;itk++){ // loop over 3D tracks

      // a track is parametrized as a line y(or x)=A*z+B in projections
      // for x proj. A = Xp_trk[itk]; B = X0_trk[itk];   
      // for y proj. A = Yp_trk[itk]; B = Y0_trk[itk];
      
      r.zf = ArtePointer<HITB>(hits[nh])->z;
      r.xf = Xp_trk[itk]*r.zf + X0_trk[itk];
      r.yf = Yp_trk[itk]*r.zf + Y0_trk[itk];

      r.ze = ArtePointer<HITB>(hits[nh+nh_trk[itk]-1])->z;
      r.xe = Xp_trk[itk]*r.ze + X0_trk[itk];
      r.ye = Yp_trk[itk]*r.ze + Y0_trk[itk];

      r.txf = r.txe = Xp_trk[itk];
      r.tyf = r.tye = Yp_trk[itk];
      r.pf  = r.pe  = 0.1; 

      for (i= 0; i<10; ++i ) r.cvf[i] = r.cve[i] = C_trk[10*itk+i];
      for (i=10; i<14; ++i ) r.cvf[i] = r.cve[i] = 0.0;
      r.cvf[14] = r.cve[14] = 1e4;

      r.time = 0.0; 
      r.dtim = 0.0; 
      r.chi  = Chi2_trk[itk]; 
      r.fit  = 0; 
      r.cmp  = 1; 
      r.cmp  = r.cmp | Rsegc::bit16;

      ArteTable<RSEG>::push_back( r );
      ArtePointer<RSEG> rseg( ArteTable<RSEG>::size() );

      for (ih=nh; ih<nh+nh_trk[itk]; ih++){
	ArtePointer<HITB> hitb (hits[ih]);
	createRelation( hitb, rseg, &RSEG::hitb );
      }

      for( ArteRelationIterator<HITB> hitb = rseg->hitb.begin();
	   hitb != rseg->hitb.end(); ++hitb ) {
    
	rh.x  = rseg->xf+((*hitb)->z-rseg->zf)*rseg->txf;
	rh.y  = rseg->yf+((*hitb)->z-rseg->zf)*rseg->tyf;
	rh.tx = rseg->txf;
	rh.ty = rseg->tyf;
	rh.p  = 0;
	rh.q  = 0;
	ArtePointer<RHIT> rhit = ArteTable<RHIT>::push_back( rh );
    
	// createRelation( rseg,  rhit, &RSEG::rhit );
	rhits.push_back( rhit );
	createRelation( *hitb, rhit, &HITB::rhit );
      }

      /*
      for( vector<ArtePointer<RHIT> >::reverse_iterator rhit
	     = rhits.rbegin(); rhit!=rhits.rend(); ++rhit ) {
	createRelation( rseg, *rhit, &RSEG::rhit );
      }
      */

      nh=nh+nh_trk[itk]+1; //skip (-1): hits of diff. tracks are
                           //separated by (-1)
			     
    }
    return;
  }
}





