#include <stdio.h>
#include <strings.h>
#include <ctype.h>
#include <stdlib.h>
#include "arte/Arte.h"
#include "auto/MEIO.h"
#include "dbarte/DbArteBridge.h"
#include "arte/ArteFlag.h"
#include "arte/mecCount.h"


extern DbFileDesc *DbFileDescList;


void mmbook_(int *, int *), mmrupd_(int *), mmdrop_(int *), mmfree_(int *);
void chhtot_(char *, char *, int *);
void iounit_(char *, int *, int);
int  mmrstat_(int *, int *);
int  mmwstat_(int *, int *);


void DbOpen(int lun, char *Name, int rwflag, int *ierr) 
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   * 12.08.98: Modified by A.Semenov (JINR, Dubna)
   *
   * Description: Open a MIZZI file/server and connect a symbolic name to it.
   *
   * Called from: anywhere.
   *
   * External references: OpenTables
   *
   * Argument in/out description
   *  lun     in     IO stream ID
   *  Name    in     file/server name
   *  rwflag  in   0 - open database read-only
   *               1 - open database for reading and writing
   *  ierr    out    Operation status:
   *               0  OK
   *               1  LUN already connected to file/server
   *               2  Can not allocate memory for FileDesc cell
   *               3  Can not open database file
   *
   *
   ************************************************************************/
 
  char mess[80]="DBOPEN: ", warn[80]="DBOPEN: ";
  char Csym[DB_MAX_NAME_SIZE+1],NameC[256];
  DbFileDesc *FD;
  
  int var;

  
  /* Get symbolic name */
  
  strncpy(Csym,MEIO(lun).stream,DB_MAX_NAME_SIZE);
  for(var=0;var<DB_MAX_NAME_SIZE && Csym[var] && Csym[var]!=' ';var++);
  Csym[var]=0;
  
  strcpy(NameC,Name);
  
	
  for(FD=DbFileDescList;FD;FD=FD->next)
  {
	  if(FD->lun==lun)
	  {
	    static int errPtr;
	    
	    /* 
	     * LUN already connected to a file.
	     * Use dbclose before reconnecting to a new file.
	     */

    sprintf(mess+strlen(mess),
       "symbolic name %s already connected to a file/server",Csym);
    mecErrorCount( &errPtr, mess );
    *ierr = 1;
	return;
      }
	  
  }


  sprintf(warn+strlen(warn),
     "opening file/server %s on %s",NameC,Csym );
  if (arteflag_.ibatch == 0)
     mecAllPrint( warn );
  else
     {
     static int errPtr;
     mecWarningCount( &errPtr, warn );
     }
	 
  /* Allocate list cell for file descriptor */
	 
  for(FD=DbFileDescList;FD;FD=FD->next)
	  if(FD->active==0)
		  break;
  
  if(FD==NULL)
  {
	  FD=(DbFileDesc *)malloc(sizeof(DbFileDesc));
	  if(FD==NULL)
	  {
		  static int errPtr;
          sprintf(mess+strlen(mess),"cannot allocate memory");
          mecErrorCount( &errPtr, mess );
          *ierr = 2;
		  return;
      }
	  FD->next=DbFileDescList;
	  DbFileDescList=FD;
  }
  
  
  /* Open database */
  
  if( ( FD->Id = OpenTables( NameC , rwflag?0:TABLEreadonly ) ) == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "cannot open file/server '%s'",NameC);
    mecErrorCount( &errPtr, mess );
    *ierr = 3;
	return;
  }
  
  
  FD->active=1;
  FD->lun=lun;
  FD->rwflag=rwflag;
  strcpy(FD->Name,NameC);
  strcpy(FD->Csym,Csym);
  
  *ierr=0;
  return;
}
