************************************************************************
*
* DGMUOI
* written by A. Snijko, 21/02/95 11.05.24
* corrected by B.Fominykh 04/12/95 - debugging
* corrected by B.Fominykh 12/02/96 - efficiency of chambers
* corrected by B.Fominykh 17/03/96 - fine structure of chambers +
*                                         digi for Ratnikov
* corrected by B.Fominykh 05/06/96 - temporary ARTE tables
*
* This routines translates GEANT output hits into ZEBRA bank structures
* JHITC and JMHITC for inner muon pixel chamber*
*
*       called from DGMAIN
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
*  IPL        in          Detector number in BANK JGEDE
*
*
************************************************************************

      SUBROUTINE DGMUOI(IPL)
C-----------------------------

      IMPLICIT NONE
#include "arte/Arte.fi"
#include "auto/GEO.fi"
#include "auto/MC.fi"
#include "auto/HITB.fi"
#include "auto/DIGB.fi"

      INTEGER NHITDE,NHITOT,NHITW,NHITS,I,J,IH,L
      INTEGER IPL,IFHIT,IFLAG,NHITOD,NRX,NRY
      INTEGER N_PAD,iSup,iMuSup,MAX_PAD_X,MAX_PAD_Y,NPIXELX,NPIXELY
      INTEGER NF_X,NL_X,NF_Y,NL_Y,ISN_X,ISN_Y,N_X,N_Y,I_PAD_NUM
      REAL XLEFT,XRIGHT,YLOW,YHIGH,XT1,XTV,XTH,YTV,YTH,ZWIR
      REAL TGTRX,TGTRY,CSTRX,RNDM,EFF,WWX,WWY,RH,RHOL
      REAL PAD_X,PAD_Y,X_PAD,Y_PAD,XREF,YREF,XL,XR,YL,YU,YBUF
      REAL XPIXEL,YPIXEL,PD_Y
      REAL DIPX,DIPY,XTVC,YTVC,XTHC,YTHC,XCR_L,XCR_R,YCR_L,YCR_U
      INTEGER IPAD,IPDD,IHITC,INDEX0,IDIGC,JT3,JT5,JT7
      REAL HITC,HITSOR,DIGSOR,DIGC

      COMMON/WRKSP1/IPAD(90,30),IPDD(90,30)

#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/MC.ff"
#include "auto/DIGB.ff"
#include "auto/HITB.ff"
      HITC(L,J) = Q(JTMP3+IQ(JTMP3+1)*(J-1)+2+L)
      IHITC(L,J)=IQ(JTMP3+IQ(JTMP3+1)*(J-1)+2+L)
      HITSOR(J) = Q(JTMP4+IQ(JTMP4+1)*(J-1)+3)
      DIGSOR(J) = Q(JTMP5+IQ(JTMP5+1)*(J-1)+3)
      INDEX0(J) =IQ(JTMP6+IQ(JTMP6+1)*(J-1)+3)
      DIGC(L,J) = Q(JTMP7+IQ(JTMP7+1)*(J-1)+2+L)
      IDIGC(L,J)=IQ(JTMP7+IQ(JTMP7+1)*(J-1)+2+L)
* ---------------------------------

C*    get all HITS in detector IPL from GEANT data structure JHITS

      IFHIT  = IGEDE_MIM1(IPL)
      NHITS  = IGEDE_MIML(IPL) - IFHIT + 1

      NHITW=0
      IF(NHITS.LE.0.OR.IFHIT.EQ.0) GOTO 998

* book temporary ARTE tables
      CALL MMBWRK(JTMP3,0,6,300)
      CALL MMBOOK(JTMP3,NHITS)

      CALL MMBWRK(JTMP4,0,1,300)
      CALL MMBOOK(JTMP4,NHITS)

      CALL MMBWRK(JTMP5,0,1,300)

      CALL MMBWRK(JTMP6,0,1,300)
      CALL MMBOOK(JTMP6,NHITS)

      CALL MMBWRK(JTMP7,0,7,300)

C*      YHIGH   -  upper edge of the plane
C*      YLOW    -  lower edge of the plane
C*      XLEFT   -  low left  corner of the plane
C*      XRIGHT  -  low right corner of the plane

      MAX_PAD_X = IGEDE_NWI(1,IPL) ! number of pixels in x-dir
      MAX_PAD_Y = IGEDE_NWI(2,IPL) ! number of pixels in y-dir
      NPIXELX   = IGEDE_NWI(3,IPL) ! number of connected pixels in x direction
      NPIXELY   = IGEDE_NWI(4,IPL) ! number of connected pixels in y direction

      WWX    = FGEDE_XFW(1,IPL) ! x-dist from edge to pixels
      WWY    = FGEDE_XFW(2,IPL) ! y-dist from edge to pixels
      XPIXEL = FGEDE_XFW(3,IPL) ! x size of elemetary pixel
      YPIXEL = FGEDE_XFW(4,IPL) ! y size of elemetary pixel

      XLEFT  = FGEDE_SXMN(IPL)
      XRIGHT = FGEDE_SXMX(IPL)
      YLOW   = FGEDE_SYMN(IPL)
      YHIGH  = FGEDE_SYMX(IPL)

      PAD_X=XPIXEL*FLOAT(NPIXELX)
      PAD_Y=YPIXEL*FLOAT(NPIXELY)
      ZWIR  = FGEDE_ZFW(1,IPL)
      RHOL  = FGEDE_ZFW(2,IPL)

      ISUP=OGEDE_GESL(IPL)     ! superlayer
      IMUSUP=IGESL_NSL(ISUP)   ! muon superlayer
      MAX_PAD_Y=MAX_PAD_Y/NPIXELY
      IF(IMUSUP.EQ.3) MAX_PAD_Y=MAX_PAD_Y + 1

* take the efficiency into account
      EFF = FGEDE_EFF(IPL)  ! efficiency
      CALL GRNDM(RNDM,1)
      IF(RNDM.GT.EFF) GOTO 998
C*    the first loop over the hits -  transform coordinates in front
C     and backside of the sensitive volumen and prepare sorting

c     Determination of the starting position
c     We start pad count from the corner in the whole.
C
c         __________X <--- This is starting (reference) point
c         oooooooooo|
c         -ooooooooo|
c          \oooooooo| <--- pixel chamber
c          |oooooooo|
c

      IF(ABS(XLEFT).LT.ABS(XRIGHT)) THEN
         XREF=XRIGHT
         PAD_X=-PAD_X
      ELSE
         XREF=XLEFT
      ENDIF

      IF(ABS(YLOW).LT.ABS(YHIGH)) THEN
         YREF=YHIGH
         PAD_Y=-PAD_Y
      ELSE
         YREF=YLOW
      ENDIF
      DO 10 I=1,NHITS
      IH = I+IFHIT-1
      JT3=JTMP3+IQ(JTMP3+1)*(I-1)+2
      XT1 = F_V(JMIMP_X,IH,JMIMP) - XREF  ! x coordinate in chamber frame
      Q(JT3+1)=XT1
      Q(JT3+2)=F_V(JMIMP_Y,IH,JMIMP) - YREF ! y coordinate
C* X COMPONENT OF THE MOMENTUM IN ROTATED SYSTEM
      Q(JT3+3)=F_V(JMIMP_TX,IH,JMIMP)
C* Y COMPONENT OF THE MOMENTUM IN ROTATED SYSTEM
      Q(JT3+6)=F_V(JMIMP_TY,IH,JMIMP)
      Q(JT3+4)=F_V(JMIMP_Z,IH,JMIMP)     ! z cordinate of the impact point
      IQ(JT3+5)=I_V(JMIMP_MTRA,IH,JMIMP) ! track number
      Q(JTMP4+IQ(JTMP4+1)*(I-1)+3)=XT1   ! tool for sorting
   10 CONTINUE

C--   SORT THE HITS MATRIX DUE TO INCREASING X

      IF(ABS(XRIGHT).LE.ABS(XLEFT)) THEN
         CALL SORTZV(Q(JTMP4+3),IQ(JTMP6+3),NHITS,1,0,0)
      ELSE
         CALL SORTZV(Q(JTMP4+3),IQ(JTMP6+3),NHITS,1,1,0)
      ENDIF

C*  the second loop over the hits fills the data structures JHIT and JMHIT

      CALL VZERO(IPAD,2700)
      CALL VZERO(IPDD,2700)

      NHITDE = 0

      DO 20 IH=1,NHITS
      I = INDEX0(IH)
      IF(IHITC(5,I).LT.0) GO TO 20

      TGTRX = HITC(3,I)  ! tangent of the track in x direction
      TGTRY = HITC(6,I)  ! tangent of the track in y direction
      XTV = HITC(1,I) + TGTRX *               ! x-coordinate in the input
     +  (F_V(JGEDE_ZMIN,IPL,JGEDE)-HITC(4,I))
      XTH = HITC(1,I) + TGTRX *
     +  (F_V(JGEDE_ZMAX,IPL,JGEDE)-HITC(4,I)) ! on the output
      YTV = HITC(2,I) + TGTRY *               ! y coordinate of input
     +  (F_V(JGEDE_ZMIN,IPL,JGEDE)-HITC(4,I))
      YTH = HITC(2,I) + TGTRY *
     +  (F_V(JGEDE_ZMAX,IPL,JGEDE)-HITC(4,I)) ! on the output
      CSTRX = 0.
      IF(TGTRX.LT.1.E+10) CSTRX = 1./SQRT(1.+ TGTRX**2)

C* SCAN OVER ALL PADS WHICH COULD BE HITTED

      NF_X = INT(XTV/PAD_X) + 1
      NL_X = INT(XTH/PAD_X) + 1

      IF(NF_X.GT.NL_X) THEN
         ISN_X =-1
         NF_X = MIN0(NF_X,MAX_PAD_X)
         NL_X = MAX0(1,NL_X)
      ELSE

        ISN_X = 1
         NF_X = MAX0(1,NF_X)
         NL_X = MIN0(NL_X,MAX_PAD_X)
      ENDIF

C*************************

c      IF(IMUSUP.NE.3) THEN
c        NF_Y = INT(YTV/PAD_Y) + 1
c        NL_Y = INT(YTH/PAD_Y) + 1
c      ELSE
c        IF(ABS(YTV).LT.ABS(PAD_Y)/2.) THEN
c          NF_Y = 1
c        ELSE
c          NF_Y = INT(YTV/PAD_Y+0.5) + 1
c        ENDIF
c        IF(ABS(YTH).LT.ABS(PAD_Y)/2.) THEN
c          NL_Y = 1
c        ELSE
c          NL_Y = INT(YTH/PAD_Y+0.5) + 1
c        ENDIF

*-------------FR-------------------
      NF_Y = INT(YTV/PAD_Y) + 1
      NL_Y = INT(YTH/PAD_Y) + 1

      IF(NF_Y.GT.NL_Y) THEN
         ISN_Y =-1
         NF_Y = MIN0(NF_Y,MAX_PAD_Y)
         NL_Y = MAX0(1,NL_Y)
      ELSE
         ISN_Y = 1
         NF_Y = MAX0(1,NF_Y)
         NL_Y = MIN0(NL_Y,MAX_PAD_Y)
      ENDIF
*-----------------------------------
*
* the control of pixel hitting was changed by B.Fominykh 4.12.95
* distance between input and output points
      DIPX=XTV-XTH
      DIPY=YTV-YTH
      XTVC=AMIN1(XTV,XTH)
      XTHC=AMAX1(XTV,XTH)
      YTVC=AMIN1(YTV,YTH)
      YTHC=AMAX1(YTV,YTH)

      DO 35 N_X=NF_X,NL_X,ISN_X   ! scan over pads in x direction
      DO 36 N_Y=NF_Y,NL_Y,ISN_Y  ! scan over pads in y direction
* check the cross of the pixel by particle
* boundaries of the pixel
      IF(PAD_X.GT.0.) THEN
        XL=REAL(N_X-1)*PAD_X
        XR=REAL(N_X)*PAD_X
      ELSE
        XR=REAL(N_X-1)*PAD_X
        XL=REAL(N_X)*PAD_X
      ENDIF
      IF(XL.GT.XTHC) GOTO 36
      IF(XR.LT.XTVC) GOTO 36

*--------------FR--------------------------
      YL=REAL(N_Y-1)*PAD_Y
      YU=REAL(N_Y)*PAD_Y
      if (IMUSUP.eq.3.and.N_Y.ge.MAX_PAD_Y)  
     &     YU = (REAL(N_Y)-0.5)*PAD_Y
      
      if (YL.gt.YU)  then
       YBUF = YL
       YL = YU
       YU = YBUF
      endif
*------------------------------------------
c      IF(IMUSUP.NE.3) THEN
c        IF(PAD_Y.GT.0.) THEN
c          YL=REAL(N_Y-1)*PAD_Y
c          YU=REAL(N_Y)*PAD_Y
c        ELSE
c          YU=REAL(N_Y-1)*PAD_Y
c          YL=REAL(N_Y)*PAD_Y
c        ENDIF
c      ELSE
c        IF(PAD_Y.GT.0.) THEN
c          IF(N_Y.LE.2) THEN
c            YL=REAL(N_Y-1)*PAD_Y/2.
c          ELSE
c            YL=(REAL(N_Y-2)+0.5)*PAD_Y
c          ENDIF
c        YU=YL+PAD_Y
c        IF(YU.GT.YHIGH-YLOW) YU=YHIGH-YLOW
c      ELSE
c        IF(N_Y.LE.2) THEN
c          YU=REAL(N_Y-1)*PAD_Y/2.
c        ELSE
c          YU=(REAL(N_Y-2)+0.5)*PAD_Y
c        ENDIF
c        YL=YU+PAD_Y
c        IF(YL.LT.YLOW-YHIGH) YL=YLOW-YHIGH
c      ENDIF
c      ENDIF

      IF(YL.GT.YTHC) GOTO 36
      IF(YU.LT.YTVC) GOTO 36
* its O.K. if input or output point inside pixel
      IF(XTV.GE.XL.AND.XTV.LE.XR.AND.
     +   YTV.GE.YL.AND.YTV.LE.YU) GOTO 31
      IF(XTH.GE.XL.AND.XTH.LE.XR.AND.
     +   YTH.GE.YL.AND.YTH.LE.YU) GOTO 31
* check the crosses of pixel boundaries by the particle
      IF(ABS(DIPX).GT.0.01) THEN
        YCR_L=YTH+(XL-XTH)*DIPY/DIPX
        IF(YCR_L.GE.YL.AND.YCR_L.LE.YU) GOTO 31
        YCR_U=YTH+(XR-XTH)*DIPY/DIPX
        IF(YCR_U.GE.YL.AND.YCR_U.LE.YU) GOTO 31
        IF(YCR_L.LE.YL.AND.YCR_U.GE.YU) GOTO 31
        IF(YCR_L.GE.YU.AND.YCR_U.LE.YL) GOTO 31
        GOTO 36  ! no hit
      ELSE
      IF(ABS(DIPY).GT.0.01) THEN
        XCR_L=XTH+(YL-YTH)*DIPX/DIPY
        IF(XCR_L.GE.XL.AND.XCR_L.LE.XR) GOTO 31
        XCR_R=XTH+(YU-YTH)*DIPX/DIPY
        IF(XCR_R.GE.XL.AND.XCR_R.LE.XR) GOTO 31
        IF(XCR_L.LE.XL.AND.XCR_R.GE.XR) GOTO 31
        IF(XCR_L.GE.XR.AND.XCR_R.LE.XL) GOTO 31
        GOTO 36  ! no hit
      ELSE
        GOTO 36  ! no hit
      ENDIF
      ENDIF
      GOTO 36    ! no hit
* pixel is hitted
  31  CONTINUE

      CALL MMBOOK(JTMP5,1)
      CALL MMBOOK(JTMP7,1)
      NHITDE = IQ(JTMP5+2)
      I_PAD_NUM=N_X+(N_Y-1)*MAX_PAD_X
*
C    PAD NUMERATION IS THE FOLLOWING
C    .......
C    .......
C    123....

      JT7=JTMP7+IQ(JTMP7+1)*(IQ(JTMP7+2)-1)+2
      IQ(JT7+1) = N_X               ! pad x-number
      IQ(JT7+2) = N_Y               ! pad y-number
       Q(JT7+3) = HITC(1,I)+XREF    ! x coordinate
      IQ(JT7+4) = I_PAD_NUM         ! pad number
       Q(JT7+5) = ZWIR              ! z position
      IQ(JT7+6) = I+IFHIT-1         ! for impact - digit - hit connection
      IQ(JT7+7) = IHITC(5,I)        ! track number
      JT5=JTMP5+IQ(JTMP5+1)*(IQ(JTMP5+2)-1)+2
      Q(JT5+1)  = HITC(1,I)         ! we store it for sorting purpose

  36  CONTINUE
  35  CONTINUE


 20   CONTINUE

 33   CONTINUE
c
C--   SORT THE HITS MATRIX DUE TO INCREASING X

      IF(IQ(JTMP6+2).LT.NHITDE)
     +     CALL MMBOOK(JTMP6,NHITDE-IQ(JTMP6+2))
      IF(ABS(XRIGHT).LE.ABS(XLEFT)) THEN
         CALL SORTZV(Q(JTMP5+3),IQ(JTMP6+3),NHITDE,1,0,0)
      ELSE
         CALL SORTZV(Q(JTMP5+3),IQ(JTMP6+3),NHITDE,1,1,0)
      ENDIF

C*       set digit and hit bank

      IFLAG = 0
      DO 40 IH = 1, NHITDE
      I = INDEX0(IH)
      N_X=IDIGC(1,I)
      N_Y=IDIGC(2,I)
      N_PAD=IDIGC(4,I)
      X_PAD=PAD_X*(FLOAT(N_X)-0.5)
      PD_Y=PAD_Y
      IF(IMUSUP.NE.3)
     +          Y_PAD=PAD_Y*(FLOAT(N_Y)-0.5)
      IF(IMUSUP.EQ.3) THEN
        IF(N_Y.EQ.MAX_PAD_Y) THEN
          Y_PAD=PAD_Y*(FLOAT(N_Y)-0.75)
          PD_Y=PAD_Y/2.
        ELSE
          Y_PAD=PAD_Y*(FLOAT(N_Y)-0.5)
        ENDIF
      ENDIF
*     is hit in the hole?
      RH=(ABS(X_PAD+XREF))**2+
     +   (ABS(Y_PAD+YREF))**2
      RH=SQRT(RH)


      IF(RH.GT.RHOL.AND.ABS(PAD_Y).GE.YPIXEL) THEN
      IF(IPAD(N_X,N_Y).EQ.0) THEN
         NHITW=NHITW+1
         CALL MMBOOK(JDIGB,1)
         CALL MMBOOK(JHITB,1)
         NHITOT = IQ(JHITB+2) ! increase the number of registered hits
         NHITOD = IQ(JDIGB+2) ! increase the number of digits
         IF(IFLAG.EQ.0) THEN
            IQ(IQGEDE_HIT1(IPL)) = NHITOT
            IQ(IQGEDE_DIG1(IPL)) = NHITOD
            IFLAG = 1
         ENDIF
* change numeration for Ratnikov
         IF(XREF.GT.0..AND.YREF.GT.0.) THEN
           NRX=MAX_PAD_X-N_X+1
           NRY=MAX_PAD_Y-N_Y+1
         ENDIF
         IF(XREF.GT.0..AND.YREF.LT.0.) THEN
           NRX=MAX_PAD_X-N_X+1
           NRY=N_Y
         ENDIF
         IF(XREF.LT.0..AND.YREF.GT.0.) THEN
           NRX=N_X
           NRY=MAX_PAD_Y-N_Y+1
         ENDIF
         IF(XREF.LT.0..AND.YREF.LT.0.) THEN
           NRX=N_X
           NRY=N_Y
         ENDIF


         IQ(IQDIGB_ID_L()) = 1000000*NRX+10000*NRY + IPL !
*old*         IQ(IQDIGB_ID_L()) = 1000000*N_X+10000*N_Y + IPL !

         Q(QHITB_X_L()) = X_PAD+XREF ! x-coordinate of the pad
         Q(QHITB_Y_L()) = Y_PAD+YREF ! y-coordinate of the pad
         Q(QHITB_Z_L()) = DIGC(5,I)  ! z-coordinate
         Q(QHITB_Q_L()) = ABS(PAD_X) ! x-size of hit
         Q(QHITB_R_L()) = ABS(PD_Y) ! y-size of hit
         IQ(IQHITB_GEDE_L())= IPL    ! MC-detector number

         CALL MMBREL(JMIDI,IDIGC(6,I),NHITOD) ! creation of relations
         CALL MMBREL(JMIHI,IDIGC(6,I),NHITOT) ! creation of relations
         CALL MMBREL(JHIDI,NHITOT,NHITOD)     ! creation of relations

         IPAD(N_X,N_Y) = NHITOT
         IPDD(N_X,N_Y) = NHITOD
      ELSE
         CALL MMBREL(JMIDI,IDIGC(6,I),IPDD(N_X,N_Y))
         CALL MMBREL(JMIHI,IDIGC(6,I),IPAD(N_X,N_Y))
      ENDIF
      ENDIF

   40 CONTINUE
      IQ(IQGEDE_DIGL(IPL)) = NHITOD
      IQ(IQGEDE_HITL(IPL)) = NHITOT
      IF(NHITW.GT.0) GOTO 999
*
 998  CONTINUE
         IQ(IQGEDE_DIG1(IPL)) = 0
         IQ(IQGEDE_HIT1(IPL)) = 0
         IQ(IQGEDE_DIGL(IPL)) = -1
         IQ(IQGEDE_HITL(IPL)) = -1
 999  RETURN
      END
