
       SUBROUTINE CGGESL(CSL,MUTT,IR)

************************************************************************
*
* CGGESL 
* written by Siegmund Nowak 01/97
*
* This routines creates GEANT superlayer banks from ARTE NESL bank
*
*       called from every geometry routine
*
* Arguments  kind         description
*-----------------------------------------------------------------------
*   CSL      in           name of the superlayer
*   MUTT     in           mother name into which it should be positioned
*                         for the moment are foreseen: MOTH,MAGN,RPOT,MUON
*                         RPOT is equivalent to MOTH except that the medium
*                         vaccuum is used instead of air
*   IR       in           overall rotation matrix number (gean)
************************************************************************

      IMPLICIT NONE
      INTEGER I,ISL,IR,HSL,IMED,SHP,IVOL,IUCOMP,MEDNUM,NPAR 
      REAL PAR(8),XMEAN,YMEAN,ZMEAN,X00,Y00,Z00
      CHARACTER*4 CSL,MUTT,MUTTC,CMUTT(6),SHAPE
#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/gcflag.fi"
#include "gean/ZGMOTH.fi"
#include "auto/GEO.fi"
#include "gean/ZGESL.fi"
      DATA CMUTT /'MOTH','MAGN','RPOT','MUON','TRSL','    '/ 
#include "arte/Arte.ff"
#include "auto/GEO.ff"

      CALL UCTOH(CSL,HSL,4,4)
      DO  ISL=1,NNESL()
       IF(HSL.EQ.HNESL_NAME(ISL)) GO TO 1 
      ENDDO
      CALL MEERRO('CGGESL- Superlayer not found: '//CSL)

 1    SHP = INESL_SHP(ISL)
      IF(SHP.EQ.1) THEN
         PAR(1) = .5*(FNESL_X2(ISL)-FNESL_X1(ISL))
         PAR(2) = .5*(FNESL_Y2(ISL)-FNESL_Y1(ISL))
         PAR(3) = .5*(FNESL_Z2(ISL)-FNESL_Z1(ISL))
         XMEAN  = .5*(FNESL_X2(ISL)+FNESL_X1(ISL))
         YMEAN  = .5*(FNESL_Y2(ISL)+FNESL_Y1(ISL))
         ZMEAN  = .5*(FNESL_Z2(ISL)+FNESL_Z1(ISL))
         SHAPE  = 'BOX '
         NPAR   = 3
      ELSEIF(SHP.EQ.3) THEN
         PAR(1) = .5*(FNESL_X2(ISL)-FNESL_X1(ISL))
         PAR(2) =  PAR(1) + FNESL_DX(ISL)
         PAR(3) = .5*(FNESL_Y2(ISL)-FNESL_Y1(ISL))
         PAR(4) =  PAR(3) + FNESL_DY(ISL)
         PAR(5) = .5*(FNESL_Z2(ISL)-FNESL_Z1(ISL))
         XMEAN  = .5*(FNESL_X2(ISL)+FNESL_X1(ISL))
         YMEAN  = .5*(FNESL_Y2(ISL)+FNESL_Y1(ISL))
         ZMEAN  = .5*(FNESL_Z2(ISL)+FNESL_Z1(ISL))
         SHAPE  = 'TRD2'
         NPAR   = 5
      ELSEIF(SHP.EQ.5) THEN
         PAR(1) = FNESL_X1(ISL)
         PAR(2) = FNESL_X2(ISL)
         PAR(3) = .5*(FNESL_Z2(ISL)-FNESL_Z1(ISL))
         XMEAN  = FNESL_Y1(ISL)
         YMEAN  = FNESL_Y2(ISL)
         ZMEAN  = .5*(FNESL_Z2(ISL)+FNESL_Z1(ISL))
         SHAPE  = 'TUBE'
         NPAR   = 3
      ELSE 
         CALL MEERRO('CGGESL:other shapes then BOX/TRD2 not foreseen') 
      ENDIF

C--   check for the mother ( MOTH, MAGN, RPOT, MUON, TRD, BLANK )
      I = IUCOMP(MUTT,CMUTT,6)
      IF(I.LE.0) 
     + CALL MEERRO('CGGESL- a wrong mother has been installed: '//MUTT)

      MUTTC = MUTT
      IF(I.EQ.1.OR.I.EQ.3) THEN
         X00 = XMEAN
         Y00 = YMEAN
         Z00 = ZZERO + ZMEAN
         IF(I.EQ.1.AND.CSL.EQ.'MAGN')  THEN  
            IMED = MEDNUM(2,'AIR')
         ELSE IF(I.EQ.1.AND.CSL.NE.'MAGN') THEN 
            IMED = MEDNUM(1,'AIR')
         ELSE
            IMED = MEDNUM(1,'VACUUM')
            MUTTC = 'MOTH'
         ENDIF
      ELSEIF(I.EQ.2) THEN
         X00 = -.5*(FNESL_X1(MAGNSL)+FNESL_X2(MAGNSL)) + XMEAN
         Y00 = -.5*(FNESL_Y1(MAGNSL)+FNESL_Y2(MAGNSL)) + YMEAN
         Z00 = -ZMAGM + ZMEAN
         IMED = MEDNUM(2,'AIR')
      ELSEIF(I.EQ.4) THEN
         X00 = -.5*(FNESL_X1(MUONSL)+FNESL_X2(MUONSL)) + XMEAN
         Y00 = -.5*(FNESL_Y1(MUONSL)+FNESL_Y2(MUONSL)) + YMEAN
         Z00 = -.5*(FNESL_Z2(MUONSL)+FNESL_Z1(MUONSL)) + ZMEAN
         IMED = MEDNUM(1,'AIR')
      ELSEIF(I.EQ.5) THEN
         X00 = -.5*(FNESL_X1(TRDSL)+FNESL_X2(TRDSL)) + XMEAN
         Y00 = -.5*(FNESL_Y1(TRDSL)+FNESL_Y2(TRDSL)) + YMEAN
         Z00 = -.5*(FNESL_Z1(TRDSL)+FNESL_Z2(TRDSL)) + ZMEAN
         IMED = MEDNUM(1,'AIR')
      ELSEIF(I.EQ.6) THEN      
         IMED = MEDNUM(1,'AIR')
      ENDIF
      CALL GSVOLU(CSL,SHAPE,IMED,PAR,NPAR,IVOL)
      CALL GSATT(CSL,'SEEN',0) 

C--   posit the superlayer
      IF(I.NE.6) CALL GSPOS(CSL,ISL,MUTTC,X00,Y00,Z00,IR,'MANY')
 99   RETURN
      END

