/***********************************************************************
* GEOMAIN
* written by Siegmund Nowak
*
* This routines is the main routine for generating the detector geometry
*
*       called from HGINIT
***********************************************************************/
#include "stdio.h"
#include "string.h"
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteMessage.hh"
#include "gean/gclink.h"
#include "gean/gcflag.h"
#include "gean/gcnum.h"
#include "gean/ZGMOTH.h"
#include "gean/ZGDETV.h"
#include "gean/ZGESL.h"

extern "C" {
   void geomain_();
   void mmbook_(int &, const int &);
   void mmdrop_(int &);
   void gesldat_();
   void datime_(int *, int *);
   void mzbook_(int*, int*, int*, int*, char*, int*, int*, int*, int*, int*, int);
   void cggesl_(char*, char*, int*);
   int  mednum_(int *, char*, int);
   void sortzv_(float[],int[],int*,int*,int*,int*);
   int  utcursl_(int); 
   void targeo_();
   void detgeo_();
   void svdgeo(int); void silset_();
   void sz0g96_();    void sz2geo_();
   void pipgeo_();    void ebmgeo_(char*,int);
   void itrgeo_(int*); void mtugeo_(int*);  void mtset_();
   void otrgeo_();     void wtlgeo_();      void otugeo_();     void wtlset_();
   void ptlgeo_(int*); void ptugeo_(int*);
   void rilgeo_();    void riugeo_();    void rgnini_(); 
   void trdgeo_();    void trdg98_();    void trugeo_();  void trinit_(); 
   void ecageo_();
   void mugeo_(char *);     void muugeo_();    void muoset_();
   void killgeo_();
   void ggclos_();
}

   void gvers();  
   void svdgeo();
   void fldgeo();
   void medmain();
   void geofin();
   int first_geo[32]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
   FILE *filptr[32]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
void geomain_() {
     int i0=0; int i1=1; int i3=3;
//   select geometry version
     if(arteflag_.geomvers == 0.) {  
         arteMsg << "geomain: no geomvers given";    arteMsg.error();
     }

//   create hb-geometry header bank nghd
     mmdrop_(ArtePointer<NGHD>::getLink().getRef());
     mmbook_(ArtePointer<NGHD>::getLink().getRef(), 1);
     ArtePointer<NGHD> ghd(1);
     datime_(&ghd->date, &ghd->time);
     if(gcflag_.idebug) {
       arteMsg << "start gvers"; arteMsg.warn();
     }   
     gvers(); 

     mmbook_(ArtePointer<NESL>::getLink().getRef(), 0);
     mmbook_(ArtePointer<NEDE>::getLink().getRef(), 0);
     mmbook_(ArtePointer<NCAL>::getLink().getRef(), 0);
     mmbook_(ArtePointer<NWAL>::getLink().getRef(), 0);

//   read in superlayer file
     if(gcflag_.idebug) {
       arteMsg << "start gesldat"; arteMsg.warn();
     }     
     gesldat_();

//   define materials 
     if(gcflag_.idebug) {
       arteMsg << "start medmain"; arteMsg.warn();
     }
     medmain();

//   initialize rotation matrix
     if(gcflag_.idebug) {
       arteMsg << "start rotation"; arteMsg.warn();
     }
     if(gclink_.jrotm <= 0) {
       char s1[4];                strncpy(s1,"rotm",4);
     //         arteMsg << "initialisation of rot matrices";    arteMsg.warn();
       mzbook_(&gcbank_.ixcons,&gclink_.jrotm,&gclink_.jrotm,
                &i1,s1,&gcnum_.nrotm,&gcnum_.nrotm,&i0,&i3,&i0,4);
       for (int i=0;i<gcnum_.nrotm;i++)
         *(gcbank_.lq + gclink_.jrotm -2 - i) = 0;}

//   mother and magnet volume, magnetic field, filling of zgmoth_ structure
     if(gcflag_.idebug) {
       arteMsg << "start MOTH/MAGN/FELD"; arteMsg.warn();
     }
     char moth[]="MOTH", magn[]="MAGN", blank[]="    ";
     int ind[100];  float znxt[100];
     ArteTableIterator<NESL> gs;
     for(; gs.valid(); gs.next()){
      int i =gs.index();
      if(gs->cmp == LGESL_MOTH) {      zgmoth_.MOTHSL = i;
         zgmoth_.ZZERO = -.5*(gs->z2+gs->z1);    cggesl_(moth,blank,&i0);  }
      else if(gs->cmp  ==  LGESL_MAG) {zgmoth_.MAGNSL = i;
         zgmoth_.ZMAGM =  .5*(gs->z2+gs->z1);    cggesl_(magn,moth,&i0);}
      else if(gs->cmp  ==  LGESL_RIC)  zgmoth_.RICHSL = i;
      else if(gs->cmp  ==  LGESL_TRSL) zgmoth_.TRDSL  = i;
      else if(gs->cmp  ==  LGESL_ECA ) zgmoth_.CALOSL = i;
      else if(gs->cmp  ==  LGESL_MUON) zgmoth_.MUONSL = i;
      else if(gs->cmp  ==  LGESL_FLD ) {zgmoth_.FELDSL = i;  fldgeo();}
      znxt[i-1] = gs->z1;  }
     int nnesl = ArtePointer<NESL>::nrow();
     sortzv_(znxt,ind,&nnesl,&i1,&i0,&i0);      

//   start of subdetector steering
//  target
     if(gcflag_.idebug) {
       arteMsg << "start target"; arteMsg.warn();
     }
    zgmoth_.CURRSL = utcursl_(LGESL_TAR); 
     if(zgdetv_.DETV[LGESL_TAR-1] == 'S')       detgeo_();
     else                                        targeo_();

//  silicon-vertex-detector
    for(int i=0;i<nnesl;i++) {
       int isl = ind[i];
       ArtePointer<NESL> sl(isl);
       zgmoth_.CURRSL = isl;
       if(sl->cmp == LGESL_SIL) {
          if(gcflag_.idebug) {
             arteMsg << "start SVD Superlayer " << isl;    arteMsg.warn();
          } 
          if(sl->mfl == 1) break;
          svdgeo(0);
       }
    }      

//  roman pot, szh0,  magnet components and beam pipes
    zgmoth_.CURRSL = utcursl_(LGESL_RPOT); 
    if(gcflag_.idebug) {
      arteMsg << "start magnet components and beam pipes ";    arteMsg.warn();
    }
    detgeo_();                   // outside magnet
    detgeo_();                   // inside  magnet 

    zgmoth_.CURRSL = utcursl_(LGESL_HOD1);
    if(zgmoth_.CURRSL != 0)   sz0g96_();

    zgmoth_.CURRSL = utcursl_(LGESL_MAG);       detgeo_();
    zgmoth_.CURRSL = utcursl_(LGESL_PPIP);
    detgeo_();                   // inside magnet
    detgeo_();                   // outside magnet

    zgmoth_.CURRSL = utcursl_(LGESL_EPIP);     ebmgeo_("MAGN",4);

//  tracker
    for(int i=0;i<nnesl;i++) {
       int isl = ind[i] ;
       ArtePointer<NESL> sl(isl);        
       zgmoth_.CURRSL = isl;

//  silicon in magnet
       if(sl->cmp == LGESL_SIL && sl->mfl == 1) svdgeo(1);

//  inner tracker
       else if(sl->cmp == LGESL_MSG) {
         if(gcflag_.idebug) {
            arteMsg << "start ITR SL " << isl;    arteMsg.warn();
         }
         if(zgdetv_.DETV[LGESL_MSG-1] == 'S')      detgeo_();
         else if(zgdetv_.DETV[LGESL_MSG-1] == 'L') itrgeo_(&isl);
         else if(zgdetv_.DETV[LGESL_MSG-1] == 'U') mtugeo_(&isl);
       }

//  outer tracker
       else if(sl->cmp == LGESL_WCH) {
         if(gcflag_.idebug) {
            arteMsg << "start OTR SL " << isl;    arteMsg.warn();
         }
         if(zgdetv_.DETV[LGESL_WCH-1] == 'L')      otrgeo_();
         else if(zgdetv_.DETV[LGESL_WCH-1] == 'S') wtlgeo_();
         else if(zgdetv_.DETV[LGESL_WCH-1] == 'U') otugeo_();
       }

//  pt chambers

       else if(sl->cmp == LGESL_PTC) { 
         if(gcflag_.idebug) {
            arteMsg << "start PTL SL " << isl;    arteMsg.warn();
         }
         if(zgdetv_.DETV[LGESL_PTC-1] == 'U') ptugeo_(&isl);
         else ptlgeo_(&isl);
       }

//  rich counter 
       else if(sl->cmp == LGESL_RIC) {
         if(gcflag_.idebug) {
            arteMsg << "start RICH,  SL " << isl;   arteMsg.warn();
         }         if(zgdetv_.DETV[LGESL_RIC-1] == 'U') riugeo_();
         else {    detgeo_();   rgnini_(); 
         }
       }

//  trd
       else if(sl->cmp == LGESL_TRSL) {
         if(gcflag_.idebug) {
            arteMsg << "start TRD - SL " << isl;    arteMsg.warn();
         }
         if(zgdetv_.DETV[LGESL_TRSL-1] == 'U') trugeo_();
         else  { 
            trdgeo_(); 
            if(zgdetv_.DETV[LGESL_TRSL-1] == 'L')    trinit_();
	  }
       }

//  scintillation counters 
       else if(sl->cmp == LGESL_HOD3)  detgeo_();

//  dosimeters 
       else if(sl->cmp == LGESL_HOD2)  sz2geo_();


//  ecal - electromagnetic calorimeter
       else if(sl->cmp == LGESL_ECA) {
          if(gcflag_.idebug) {
             arteMsg << "start ECAL - SL " << isl;    arteMsg.warn();
          }
          ecageo_();
//        electron beam pipe inside cal
          zgmoth_.CURRSL = utcursl_(LGESL_EPIP);    ebmgeo_("CALO",4);
       }

//  muon - muon spectrometer
       else if(sl->cmp == LGESL_MUON) {
       if(gcflag_.idebug) {
          arteMsg << "start MUON - SL " << isl;    arteMsg.warn();
       }
          if(zgdetv_.DETV[LGESL_MUON-1] == 'U') muugeo_();
          else  {  mugeo_("SHORT");  muoset_();
          }
//        electron beam pipe inside muon
          zgmoth_.CURRSL = utcursl_(LGESL_EPIP);     ebmgeo_("MUON",4);
       }
    }

    silset_();
    mtset_();
    if(zgdetv_.DETV[LGESL_WCH-1] == 'S')    wtlset_();

    killgeo_();

//  close geometry banks. (obligatory system routine)
    ggclos_();
    geofin();

}

