*
*  B.Fominykh  10.02.98
*  acceleration of MC calculations in muon detector
*  MUTURB has to be called from GUSTEP
*
*  Algorithm:
*  for electrons and positrons the stopping range is calculated
*  taking into account dE/dX and soft bremstrahlung.
*  It stops the tracking of electron if a distance befor boundary more
*  the stopping range. 
*
*!!  routine has to be changed if the geometry of proton beam pipe
*!!  or the geometry of absorbers will be changed
*!!  because the geometry is defined inside the program
 
*  NO input parameters
*
      SUBROUTINE MUTURB
C     =================
*
      IMPLICIT NONE
      INTEGER I,J,ITB
      REAL R,SWW,STW,ZWW
* ZWAL - start and final z-positions of hadron absorbers
      REAL ZWAL(6)
* NSPBP - numer of steps in the proton beam pipe
* ZSPBP,RSPBP,LSPBP - Zstart,Rout,Length of proton beam pipe
      INTEGER NSPBP
      PARAMETER (NSPBP=13)
      REAL ZSPBP(NSPBP),RSPBP(NSPBP),LSPBP(NSPBP)
*
      REAL ASAFE,ZFER,PECUT
* ASAFE - safety factor for ranges
      PARAMETER (ASAFE=0.8)
* Z of iron
      PARAMETER (ZFER=26.)
* momentum cut for electrons (GeV/c)
      PARAMETER (PECUT=0.5)
*
      REAL ET(31),RAT(31)
*
#include "arte/Arte.fi"
#include "gean/gcmate.fi"
#include "gean/gctrak.fi"
#include "gean/gctmed.fi"
#include "gean/gckine.fi"
#include "gean/ZGMOTH.fi"
*
* table of stopping range (g/cm**2) in iron 
* as function of electron energy (GeV)
      DATA RAT /0., 4.259E-4,2.110E-2,2.475E-1,5.997E-1,9.568E-1,
     +          1.307, 1.715, 1.908, 2.303, 2.618, 3.227, 3.810,
     +          4.369, 4.907, 5.426, 5.926, 10.17, 13.47, 16.15,
     +          18.42, 20.37, 23.63, 26.28, 35.00, 40.34, 44.20,
     +          47.22, 49.70, 53.64, 56.71 /
      DATA ET  /0., 0.00001, 0.0001, 0.0005, 0.001, 0.0015, 0.002,
     +              0.0026 , 0.003 , 0.0035, 0.004, 0.005 , 0.006,
     +              0.007  , 0.008 , 0.009 , 0.01 , 0.02  , 0.03 ,
     +              0.04   , 0.05  , 0.06  , 0.08 , 0.1   , 0.2  ,
     +              0.3    , 0.4   , 0.5   , 0.6  , 0.8   , 1. /
*
* geometry of the step proton beam pipe
      DATA ZSPBP /1485.15, 1519.9 , 1544.8 , 1571.95, 1599.9 , 
     +            1649.95, 1681.45, 1719.35, 1767.95, 1818.25,
     +            1854.55, 1914.05, 1978.85 /
      DATA LSPBP /  34.75,   24.9 ,   27.15,   27.95,   50.05,
     +              31.5 ,   37.9 ,   48.6 ,   50.3 ,   36.3 ,
     +              59.5 ,   64.8 ,   18.3 /
      DATA RSPBP /   3.2 ,    3.45,    3.8 ,    4.  ,    4.25,
     +               4.45,    4.65,    5.  ,    5.2 ,    5.4 ,
     +               5.72,    6.45,    6.65/
* start and end Z of hadron absorbers
      DATA ZWAL / 1490., 1611.5, 1650., 1738.5, 1770., 1869. /
*
#include "arte/Arte.ff"
C----+===1=========2=========3=========4=========5=========6=========7==*******
*
* input check-point: only electrons in nonsensitive volume, p < PECUT
      IF(ITRTYP.NE.2)    GOTO 200
      IF(ISVOL.NE.0)     GOTO 200
      IF(VECT(7).GE.PECUT)  GOTO 200
* stopping range for the electron
      DO 10 I=2,31
        IF(VECT(7).GT.ET(I)) GOTO 10
        R=RAT(I-1)+(VECT(7)-ET(I-1))/(ET(I)-ET(I-1))*(RAT(I)-RAT(I-1))
* correction if non-iron (beton)
        IF(Z.NE.ZFER) R=R*SQRT(ZFER/Z)
        R=R/DENS
        GOTO 20
  10  CONTINUE
      GOTO 200
* 
  20  SWW=0.
      ZWW=VECT(3)-ZZERO
* loops over 3 hadron absorbers
      DO 30 I=1,3
        IF(ZWW.GT.ZWAL(2*I))   GOTO 30
        IF(ZWW.LT.ZWAL(2*I-1)) GOTO 200
* distance to edge
        IF(VECT(6).GT.0.) THEN
          SWW=ZWAL(2*I)-ZWW
        ELSE
          SWW=ZWW-ZWAL(2*I-1)
        ENDIF  
* distance to proton beam-pipe
        ITB=0
        DO 22 J=1,NSPBP
          IF(ZWW.GT.ZSPBP(J)+LSPBP(J)) GOTO 22
          IF(ZWW.LT.ZSPBP(J))          GOTO 200
          ITB=J
          GOTO 25
  22    CONTINUE
        GOTO 30
  25    STW=(SQRT(VECT(1)**2+VECT(2)**2)-RSPBP(ITB))
* min distance
        IF(STW.LT.SWW) SWW=STW
*
        IF(ASAFE*SWW.GT.R) THEN
           ISTOP=1
***           PRINT*, 'MUTURB: track stopped'
           GOTO 200
        ENDIF
  30  CONTINUE
*
 200  RETURN
      END
