        subroutine otrcov(material,zpos1,zpos2)
******************************************************************
*
* Defines cover plates for a wall with two holes:
* for proton and electron beam pipes.
*
* +-----------------------------+ y2
* |           |     |  |  |     |
* |           |     |  |  |     |
* |           |  2  |4 |5 |     |
* |           |     |  |  |     |
* |           |     |  |  |     |
* |           +-----+  |  |     | yp2
* |    1      |XXXXX|  |  |  7  |
* |           |XXXXX|  |  |     |
* |           +-----+  |  |     | yp1
* |           |     |  |  |     |
* |           |     |  +--+     | ye2
* |           |  3  |  |XX|     |
* |           |     |  +--+     | ye1
* |           |     |  |  |     |
* |           |     |  |6 |     |
* |           |     |  |  |     |
* +-----------------------------+ y1
* x1         xp1  xp2 xe1 xe2   x2
*
* Called from: otwall.F
*
* Arguments    in/out       Description
*
* material     in           material of wall
* zpos1,zpos2  in           Borders on Z coordinate (may be in any order)
*
* Alexander Lanyov     Mar 1999
*
************************************************************************
        implicit none

#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGBPIP.fi"
#include "gean/ZGOTR.fi"
#include "auto/GEO.fi"
#include "gean/ZGESL.fi"

#include "gean/otrgeo.fi"

        real zpos1,zpos2,z1,z2
        real x1,x2,y1,y2,xp1,xp2,yp1,yp2,xe1,xe2,ye1,ye2
        character*4 material,csl

#include "arte/Arte.ff"
#include "auto/GEO.ff"

        call uhtoc(hnesl_name(currsl),4,csl,4)
        if (zpos1 .lt. zpos2) then
           z1 = zpos1
           z2 = zpos2
        else
           z1 = zpos2
           z2 = zpos1
        end if

        x1 = xmin_all
        x2 = xmax_all
        y1 = ymin_all - 7.
        y2 = ymax_all + 7.

c All coordinates below are approximate.
        if (csl(1:2) .eq. 'MC') then
           xp1 = -12.
           xp2 =  10.
        else         
           xp1 = -13.
           xp2 =  12.
        end if
        yp1 = -13.5   ! was 15.
        yp2 =  13.5   ! was 15.

        if (x2 .lt. xp1) then
c For 99 run we have superlayer totally on the left side
           call crbox(material,csl,x1,x2,y1,y2,z1,z2)
           goto 999
        end if


c MC05-MC08 have special e-beam cutout (not hole)
        if (csl .eq. 'MC05') then
           xe1 =  20.
           xe2 =  70.
           ye2 = -70.
           ye1 =   0.
        else if (csl .eq. 'MC06') then
           xe1 =  20.
           xe2 =  70.
           ye2 = -80.
           ye1 =   0.
        else if (csl .eq. 'MC08') then
           xe1 =  22.
           xe2 =  70.
           ye2 = -90.
           ye1 =   0.
        else if (csl(1:2) .eq. 'MC') then
c MC01-MC04 have no e-beam cutout
           xe1 =   0.
        else
c PC,TC has a hole for e-beam pipe
           xe1 =  40.
           xe2 =  52.
           ye1 = -93.
           ye2 = -68.
        end if

c------------------------------
        call crbox(material,csl,x1 ,xp1,y1 ,y2 ,z1,z2) ! 1. left
        call crbox(material,csl,xp1,xp2,yp2,y2 ,z1,z2) ! 2. right
        call crbox(material,csl,xp1,xp2,y1 ,yp1,z1,z2) ! 3. lower

        if (xe1.eq.0.              ! MC01-MC04 have no e-beam cutout
     @ .or. xe2.gt.x2) then        ! Left half also has no e-beam cutout
           call crbox(material,csl,xp2,x2 ,y1 ,y2 ,z1,z2) ! 4-7. right
        else
           call crbox(material,csl,xp2,xe1,y1 ,y2 ,z1,z2) ! 4. left to the e-beam
           call crbox(material,csl,xe1,xe2,ye2,y2 ,z1,z2) ! 5. up to the e-beam
           if (ye1.ne.0.) call crbox(material,csl,xe1,xe2,y1,ye1,z1,z2) ! 6. down to the e-beam
           call crbox(material,csl,xe2,x2 ,y1 ,y2 ,z1,z2) ! 7. right to the e-beam
        end if

 999    return
        end
