#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "gean/zgeom.h"
#include "gean/gclink.h"
#include "gean/ZGESL.h"


extern "C" {
   void gsposp_(char*,unsigned int*,char*,float*,float*,float*,
                int*,char*,float[],unsigned int*);
   }

int    zgpos::maxpos[33];
int    zgpos::nrpos[33] ;
hgpos *zgpos::ptpos[33];

/****************************************************************************/
void zgpos::ini(int n){ 
     ptpos[zgvol::ind_pt] = new hgpos[n];
     if(ptpos[zgvol::ind_pt]==NULL)
        {arteMsg << "no hgpos[n] allocated"; arteMsg.error();}
     maxpos[zgvol::ind_pt] = n;
     nrpos[zgvol::ind_pt] =0;  
}
/****************************************************************************/
void zgpos::fin() {   
     delete [] ptpos[zgvol::ind_pt];
     nrpos[zgvol::ind_pt] = 0; 
}   
/****************************************************************************/
void zgpos::increase() {
     if(nrpos[zgvol::ind_pt] < maxpos[zgvol::ind_pt]-1) nrpos[zgvol::ind_pt]++;
     else {
       hgpos *pos, *temp, *tmp; 
       int n =2*maxpos[zgvol::ind_pt];
       temp = new hgpos[n];    
       if(temp == NULL) {
          arteMsg << "no hgpos[2*n] allocated";   arteMsg.error();
       } 
       for(int i=0; i <= nrpos[zgvol::ind_pt]; i++) {
           pos = ptpos[zgvol::ind_pt] + i ;    tmp = temp + i ;   *tmp = *pos; 
       }
       delete [] ptpos[zgvol::ind_pt];        
       ptpos[zgvol::ind_pt] = temp;
       maxpos[zgvol::ind_pt] = 2* maxpos[zgvol::ind_pt]; 
       nrpos[zgvol::ind_pt]++;
     }
}
/****************************************************************************/  
void zgpos::Set_default() {
     hgpos *pos = pospointer(size()-1);
     memset(pos,0,sizeof(hgpos));
     pos->index_hgvol = -1;
}
/****************************************************************************/
void zgpos::Set_vol_pos_Rel() {
     zgvol ZGVOL;
     hgvol *vol = ZGVOL.volpointer(ZGVOL.size()-1) ;
     vol->index_hgpos = size()-1 - vol->ncomp;
}
/****************************************************************************/
void zgpos::Read_comp(char *card) {
     char taga[16],tagb[16];
     zgvol ZGVOL;
     hgvol *vol = ZGVOL.volpointer(ZGVOL.size()-1) ;
     hgpos *pos = pospointer(size()-1);
     int nitem=sscanf(card,"%s %d %f %f %f %s", taga, &pos->ctyp, 
                      &pos->xpos, &pos->ypos,&pos->zpos, tagb );
     if (nitem!=6) {
           arteMsg <<"check standard data on comp cards"; arteMsg.error();
     }
     fillstr(taga,4);
     fillstr(tagb,4);
     int j = ZGVOL.Get_volnr(taga,pos->ctyp);
     if(j<0) { 
        arteMsg <<"comp card: volume " <<taga << "not defined before";
        arteMsg.error();
     } 
     else            {
        hgvol *dau = ZGVOL.volpointer(j);
        strncpy(pos->comp,taga,4);
        strncpy(pos->only,tagb,4);
        pos->xpos = pos->xpos + vol->rel_px - dau->rel_px;
        pos->ypos = pos->ypos + vol->rel_py - dau->rel_py; 
        pos->zpos = pos->zpos + vol->rel_pz - dau->rel_pz; 
        // rotation matrix, sector and layer 
        card+=itemlength(card,nitem);
        nitem = sscanf(card," %s ",taga);
        if(nitem== -1 || !strncmp(taga,"!",1) || !strncmp(taga,"//",2)) {
           pos->rotm = pos->sector = pos->layer = 0;
           pos->rot_px = pos->rot_py = pos->rot_pz =0.;
           vol->ncomp++;
           pos->index_hgvol = ZGVOL.size()-1;
           increase(); 
           return; 
        }
        nitem=sscanf(card," %d %d %d ",&pos->rotm,&pos->sector,
                     &pos->layer);
        if (nitem!=3) {
           arteMsg <<"check rot/lay/sec on comp cards";  arteMsg.error();
        }
        if(pos->layer > 15) {
           arteMsg <<"zgpos: more than 15 layers - contact S.Nowak";
           arteMsg.error();
        } 
        if(pos->sector > 63) {
           arteMsg <<"zgpos: more than 63 sector - contact S.Nowak";
           arteMsg.error();
        }
        // rotation point
        card+=itemlength(card,nitem);
        nitem = sscanf(card," %s ",taga);
        if(nitem== -1 || !strncmp(taga,"!",1) || !strncmp(taga,"//",2)) {
           pos->rot_px = pos->rot_py = pos->rot_pz =0.;
           vol->ncomp++;
           pos->index_hgvol = ZGVOL.size()-1;
           increase(); 
           return; 
        }
        nitem=sscanf(card," %f %f %f ",&pos->rot_px,&pos->rot_py,
                     &pos->rot_pz);
        if (nitem!=3) {
           arteMsg <<"check rotation point data on comp cards";
           arteMsg.error();
        }
        pos->rot_px = pos->rot_px + vol->rel_px ;
        pos->rot_py = pos->rot_py + vol->rel_py ;
        pos->rot_pz = pos->rot_pz + vol->rel_pz ;
        vol->ncomp++; 
        pos->index_hgvol = ZGVOL.size()-1;
        increase(); 
        return; 
     }
}
/****************************************************************************/ 
void zgpos::Read_resolve(char *card) {
     char taga[16];
     zgvol ZGVOL;
     hgvol *vol= ZGVOL.volpointer(ZGVOL.size()-1);
     int type;
     int nitem=sscanf(card,"%s %d",taga, &type);
     taga[5] = '\0';
     if (nitem!=2) { 
         arteMsg << "check card resolve for " << taga;  arteMsg.error();
     } 
     for (int m=0;m<3;m++) taga[m]=toupper(taga[m]);
     float xp=0, yp=0, zp =0 ; 
     card+=itemlength(card,nitem);
     nitem=sscanf(card,"%f %f %f",&xp, &yp, &zp ); 
     if (nitem!=3) {
         arteMsg << "check card resolve"; arteMsg.error();
     } 
     int n = ZGVOL.Get_volnr(taga,type);
     if(n<0) { 
         arteMsg <<"volume " <<taga << " to resolve does not exist";
             arteMsg.error();
     }
     else  {
        hgvol *dau = ZGVOL.volpointer(n);
        if(dau->index_hgpos < 0) {
           arteMsg <<"volume " <<taga << " nothing to resolve "; arteMsg.warn();
        }
        else { 

          for(int s=0; s< dau->ncomp;s++) { 
             hgpos *pos    = pospointer(size()-1);
             *pos = *pospointer(dau->index_hgpos + s);
             pos->xpos += xp + vol->rel_px - dau->rel_px;   
             pos->ypos += yp + vol->rel_py - dau->rel_py;
             pos->zpos += zp + vol->rel_pz - dau->rel_pz;   
             pos->index_hgvol = ZGVOL.size()-1;
             increase();
          }
          vol->ncomp = vol->ncomp + dau->ncomp;
        }
        return ; 
     }
}
/****************************************************************************/
void zgpos::Read_Mirror_x() {
     if (nrpos[zgvol::ind_pt] >0) {
       zgvol ZGVOL;
        hgvol *vol=ZGVOL.volpointer(ZGVOL.size()-1); 
        hgvol *oldv = vol-1;
	if(oldv->index_hgpos < 0) {
	   arteMsg <<"mirror_x nonexecutable - previous without components";
	   arteMsg.error();
        }
        for(int s=0; s< oldv->ncomp;s++) {  
           *pospointer(size()-1) = *pospointer(oldv->index_hgpos + s);
           pospointer(size()-1)->xpos = - pospointer(size()-1)->xpos;
           pospointer(size()-1)->index_hgvol = ZGVOL.size()-1;
           increase();
        }
        vol->ncomp = oldv->ncomp;       
        vol->rel_px = -oldv->rel_px;
     }
}
/****************************************************************************/ 
void zgpos::Read_Mirror_y() {
     if (size() >1) {
        zgvol ZGVOL; 
        hgvol *vol=ZGVOL.volpointer(ZGVOL.size()-1); 
        hgvol *oldv = vol-1;
	if(oldv->index_hgpos < 0) {
	   arteMsg <<"mirror_y noexecutable - previous without components";
	   arteMsg.error();
        }
        for(int s=0; s< oldv->ncomp;s++) {  
           *pospointer(size()-1) = *pospointer(oldv->index_hgpos + s);
           pospointer(size()-1)->ypos = - pospointer(size()-1)->ypos;
           pospointer(size()-1)->index_hgvol = ZGVOL.size()-1;
           increase();
        }
        vol->ncomp = oldv->ncomp;       
        vol->rel_py = -oldv->rel_py;
     }
}
/****************************************************************************/
void zgpos::Read_Mirror_z() {
     if (size() > 1) {
        zgvol ZGVOL; 
        hgvol *vol = ZGVOL.volpointer(ZGVOL.size()-1); 
        hgvol *oldv = vol-1;
        if(oldv->index_hgpos < 0) {
	   arteMsg <<"mirror_z noexecutable - previous without components";
	   arteMsg.error();
        }
        for(int s=0; s< oldv->ncomp;s++) {  
           *pospointer(size()-1) = *pospointer(oldv->index_hgpos + s);
           pospointer(size()-1)->zpos = - pospointer(size()-1)->zpos;
           pospointer(size()-1)->index_hgvol = ZGVOL.size()-1;
           increase();
        }
        vol->ncomp = oldv->ncomp;       
        vol->rel_pz = -oldv->rel_pz;
     }
}
/****************************************************************************/
void zgpos::Gposit_all(int &nr_vol,int &icopy) {
     unsigned int npar; int gnr_rot;  float par[11],xp,yp,zp;  zgvol ZGVOL;
     char volume[5], mot[5];    volume[4] = mot[4] = '\0';
     hgvol *moth = ZGVOL.volpointer(nr_vol);
     strncpy(mot,moth->object,4);
     for(int i=0;i<moth->ncomp;i++) {
        hgpos *pos = pospointer(moth->index_hgpos + i);
        strncpy(volume,pos->comp,4);
        int j = ZGVOL.Get_volnr(volume,pos->ctyp);
        if(j>=0) { 
          npar = ZGVOL.Get_par(j,par);
          if(Get_RotPos(nr_vol,i,xp,yp,zp,gnr_rot)) {
             unsigned int inum = pos->ctyp<<16;
             icopy++; 
             if(icopy > 4095) {
                arteMsg <<"zgpos: > 4095 copies - contact S.Nowak";
                arteMsg.error();
             }
             inum = inum + icopy;
             gsposp_(volume,&inum,mot,&xp,&yp,&zp,&gnr_rot,
                        pos->only,par,&npar); 
	   }
          else {
             arteMsg <<"rotation matrix for comp " << volume <<" not found";
             arteMsg.error();
          } 
        }
        else {
           arteMsg <<"volume "<< volume <<" not found";arteMsg.error();
        }
     }
}
/****************************************************************************/
void zgpos::Gposit_superlayer(char * cname,int & nr_vol, int & icopy){
     unsigned int npar; int gnr_rot;  float par[11],xp,yp,zp;  zgvol ZGVOL;
     char volume[5], mot[5];    volume[4] = mot[4] = '\0';
     hgvol *moth = ZGVOL.volpointer(nr_vol);
     strncpy(mot,moth->object,4);
     for(int i=0;i<moth->ncomp;i++) {
        hgpos *pos = pospointer(moth->index_hgpos + i);
        strncpy(volume,pos->comp,4);
        strncpy(cname,"    ",4);
        Get_RotPos(nr_vol,i,xp,yp,zp,gnr_rot);
        if(!Check_superlayer(cname,xp,yp,zp)) {
           arteMsg << "volume " << volume <<"  "<< pos->ctyp <<"outside det";
           arteMsg.warn();   continue; 
        }
        int j = ZGVOL.Get_volnr(volume,pos->ctyp);
        if(j>=0) { 
           npar = ZGVOL.Get_par(j,par);
           unsigned int inum = pos->ctyp<<16;
           icopy++; 
           if(icopy > 4095) {
              arteMsg <<"zgpos: > 4095 copies - contact S.Nowak";
              arteMsg.error();
           }
           inum = inum + icopy;
           gsposp_(volume,&inum,cname,&xp,&yp,&zp,&gnr_rot,
                        pos->only,par,&npar); 
        } 
        else {
           arteMsg <<"volume "<< volume <<" not found";     arteMsg.error();
        }
     }
}
/****************************************************************************/
int  zgpos::Check_superlayer(char *cname,float &xp,float &yp,float &zp){
     int found =0; float xp1=0, yp1=0, zp1=0;
     if (!strncmp(cname,"    ",4)) {
        ArteTableIterator<NESL> gs;
        zgvol ZGVOL; 
        for(; gs.valid(); gs.next()) {
          if(!ZGVOL.Check_Gvol(gs->name)) continue; 
          if(zp < gs->z1 || zp > gs->z2) continue;
          if(gs->shp == 1) { 
             if(xp < gs->x1 || xp > gs->x2) continue;   
             if(yp < gs->y1 || yp > gs->y2) continue; 
          }
          else if(gs->shp == 3) { 
             float ddx = gs->dx/(gs->z2-gs->z1)*(zp-gs->z1);
             float ddy = gs->dy/(gs->z2-gs->z1)*(zp-gs->z1);
             if(xp < (gs->x1-ddx) || xp > (gs->x2+ddx)) continue;
             if(yp < (gs->y1-ddy) || yp > (gs->y2+ddy)) continue;
          }
          else if(gs->shp == 5) { float r = sqrt(xp*xp + yp*yp);
            if(r < gs->x1 || r > gs->x2)  continue; 
          }
          strncpy(cname,gs->name,4); cname[4] ='\0';   
          xp1 = xp - .5*(gs->x1+gs->x2);  
          yp1 = yp - .5*(gs->y1+gs->y2);  zp1 = zp - .5*(gs->z1+gs->z2);
          found = 1;  
        }
     }
     xp = xp1;     yp = yp1;   zp =zp1;    
     return found; 
}
/****************************************************************************/
int  zgpos::Get_RotPos(const int nr_vol, const int index,
                       float &xp,float &yp,float &zp,int &nr_rot)   {
     zgvol ZGVOL;   zgrot ZGROT;
     hgvol *moth = ZGVOL.volpointer(nr_vol); 
     hgpos *pos = pospointer(moth->index_hgpos + index);
     int found = 0;
     for(unsigned int k=0; k< ZGROT.size(); k++) { 
         hgrot *rot = ZGROT.rotpointer(k);
         if(abs(pos->rotm) == rot->rotnum) {
            xp = pos->xpos,yp = pos->ypos,zp = pos->zpos;
            if(pos->rotm !=0) {
               float xp1=xp-pos->rot_px;       
               float yp1=yp-pos->rot_py;       
               float zp1=zp-pos->rot_pz;       
               xp = rot->grot[0]*xp1 + rot->grot[3]*yp1 + 
                    rot->grot[6]*zp1 + pos->rot_px ; 
               yp = rot->grot[1]*xp1 + rot->grot[4]*yp1 + 
                    rot->grot[7]*zp1 + pos->rot_py ; 
               zp = rot->grot[2]*xp1 + rot->grot[5]*yp1 + 
                    rot->grot[8]*zp1 + pos->rot_pz ;
            }
            found = 1;    nr_rot = rot->irotm;   break; 
         }
     }
     return found; 
}
/****************************************************************************/
char *zgpos::Get_posname(int nr_vol,int index,int &type) {
     zgvol ZGVOL;
     hgvol *vol = ZGVOL.volpointer(nr_vol);
     hgpos *pos = pospointer(vol->index_hgpos + index - 1);
     type = pos->ctyp;
     return pos->comp;
}
/****************************************************************************/ 
int  zgpos::Get_layer(int nr_vol,int index) {
     zgvol ZGVOL;
     hgvol *vol = ZGVOL.volpointer(nr_vol);
     hgpos *pos = pospointer(vol->index_hgpos + index - 1);
     return pos->layer;
}
/****************************************************************************/
int  zgpos::Get_sector(int nr_vol,int index) {
     zgvol ZGVOL;
     hgvol *vol = ZGVOL.volpointer(nr_vol);
     hgpos *pos = pospointer(vol->index_hgpos + index - 1);
     return pos->sector;
}
/****************************************************************************/

