#ifndef _RCMATCHPOINT_HH_
#define _RCMATCHPOINT_HH_
//---------------------------------------------------------------------
// Created: 13.08.97,   Olya Igonkina

// MARPLE class of RcPoints of matching  

//---------------------------------------------------------------------


#include <iostream.h>

#include "arte/ArteMessage.hh"

#include "marple/RcPoint.hh"
#include "RcMatchFlag.hh"

/** class for matching of two RcPoints.
  It creates a weighted mean point using information from origin RcPoints.
  $ \chi^2 $ of matching is calculated.
  Class is inherited from RcPoint.
  */
class RcMatchPoint : public RcPoint
{

public:
  ///                 default constructor
  inline RcMatchPoint(); 

  /// create a mean point and calculate $ \chi^2 $, {\tt lineswim} manages prolongation of rp1 to rp2
  RcMatchPoint( RcPoint &rp1, 
		RcPoint &rp2,
		RcMatchFlag matchFlag,
		int lineswim = 0);

  ///                 copy constructor
  inline RcMatchPoint( const RcMatchPoint &rp ); 
  ///                 copy constructor
  inline RcMatchPoint(       RcPoint &rp ); 


  ///                 destructor
  inline ~RcMatchPoint(); 
  
  ///                   assignment operator
  inline RcMatchPoint & operator =  (const RcMatchPoint &rp);
  ///                   assignment operator
  inline RcMatchPoint & operator =  (const RcPoint &rp); 
  ///                 true if chi2() < rp1.chi2()
  inline int operator < ( const RcMatchPoint &rp1 ) const; 

  ///                  returns \Ref{ RcMatchFlag } value 
  inline RcMatchFlag matchFlag() const;

  ///                  return $ \chi^2 $ of matching
  inline HepDouble chi2() const;  
  ///       number of variables to be matched
  inline int       Nvariables() const;
  ///                 set $ \chi^2 $ 
  inline void      setchi2( HepDouble chi2) ;

  //                  return quality of matching
  HepDouble        quality() const;  

  ///                  return vector of pointers to original RcPoints
  inline vector<RcPoint *> origin() const;
  ///  add one more point to vector of origins, DO NOT change point itself 
  inline void addOrigin( RcPoint & rp) ; 
  ///                 erase a list of Origin
  inline void eraseOrigin() ; 


  ///                  return relation with HITB 
  inline vector<ArtePointer<HITB> > hitbList() const; 
  ///      add HITB entry to list of connected HITB, DO NOT change point itself
  inline void connectHitb( const ArtePointer<HITB> hitb );
  ///                  correct RcMatchPoint by measured HITB
  void addHitb( const ArtePointer<HITB> hitb, int & iErr );
  ///                  erase HITB entry from the list of connected HITB
  inline void eraseHitb( const ArtePointer<HITB> hitb );

  ///                 return relation with MTRA 
  vector<ArtePointer<MTRA> > mtraList() const; 

  ///                 return relation with HITC
  vector<ArtePointer<HITC> > hitcList() const; 

  ///  Kalman refit of the one of original RcPoint's using RcMatchPoint parameters, refit all original RcPoint if iOrig < 0
  HepDouble refitOrig (int & iErr, int iOrig = -1);


private:
  ///        $ \chi^2 $ of matching
  HepDouble fchi2;
  //        quality of matching
  //HepDouble fquality;

  ///       number of variables to be matched
  int      fNvariables;

  RcMatchFlag fMatchFlag;

  ///        vector of origin RcPoints
  vector<RcPoint *> foriginPoints;
  //        vector of adition HITB entries
  vector<ArtePointer<HITB> > fHitbList;

};


ostream & operator << (ostream &, const RcMatchPoint &);
///                   true if( prm1->chi2() < prm2->chi2() )
bool comp_chi2( const RcMatchPoint * prm1, const RcMatchPoint * prm2);

#include "marple/RcMatchPoint.icc"

#endif

