#ifndef _MARPLE_HH_
#define _MARPLE_HH_
//---------------------------------------------------------------------
// Created: 15.10.97  by   Olya Igonkina for MARPLE

// MARPLE prototypes for reconstruction functions 

//---------------------------------------------------------------------


#include <vector>

#include <CLHEP/Utilities/fortran.h>
#include <CLHEP/Vector/ThreeVector.h>
#include <CLHEP/Matrix/DiagMatrix.h>
#include <CLHEP/Matrix/SymMatrix.h>
#include <CLHEP/Matrix/Matrix.h>


extern "C" 
{ 
  extern void FORTRAN_ROUTINE( kugetr   )(float *); 
  extern void FORTRAN_ROUTINE( kugeti   )(int *  ); 
}

/// constants used for pack and unpack covariant matrix ( C/C++ style )
extern int const  c_matrix_pack[5][5];
/// constants used for pack and unpack covariant matrix ( FORTRAN style )
extern int const  fortran_matrix_pack[5][5];

/// constants used for pack and unpack GTAR rotation matrix ( C/C++ style )
extern int const  c_rotation_matrix[3][3];
/// constants used for pack and unpack GTAR rotation matrix ( FORTRAN style )
extern int const  fortran_rotation_matrix[3][3];

/// Infinity constant for covariant matrix elements
extern HepDouble const MARPLE_INFINITY;

/// value is supposed to be equal 0 if it less that MARPLE\_NEGLIGIBLE
extern HepDouble const MARPLE_NEGLIGIBLE;

/// level of MARPLE printout, -1 means no printout at all
extern short MARPLE_DEBUG;

/// if equals 1 MARPLE save information to RSEG bank
extern short MARPLE_MAKERSEG;

/// if it set to 1 the softest cuts are choosen to see corelation between parts
extern short MARPLE_SOFT;

/// if it set to 1 marple takes combinations from different events.
extern short MARPLE_BKGR;


#include "marple/RcMsOfGesl.hh"
//#include "marple/RcMatchPoint.hh"
#include "marple/RcMatchConst.h"
#include "marple/RcLimit.h"


#include "marple/RcPoint.hh"
#include "marple/RcSegPoint.hh"
#include "marple/RcTraPoint.hh"
#include "marple/RcClusPoint.hh"
#include "marple/RcMatchPoint.hh"

/// vector of average Multiple Scattering Angle of superlayers (GESL), created by marpleinit\_() 
extern vector <RcMsOfGesl> msGeslList;

// the function should be used to check magnet field in marple, 
// the variable could be rewritten
extern bool maMagnetON();   
extern bool MARPLE_MagnetON; 

extern "C"
{
  /// marple initialization
  extern void FORTRAN_ROUTINE( marpleinit)();

  /// marple function which should be called at the end of program
  extern void FORTRAN_ROUTINE( marplestop)();

  /// marple kuip commands
  extern void FORTRAN_ROUTINE( marplekuip)();

  /// does match between two sets of segments [Rse1,Rse2] #&# [Rsf1,Rsf2]
  extern void FORTRAN_ROUTINE( matchrsersf  )(int *Rse1, int *Rse2,
					      int *Rsf1, int *Rsf2);

  /// does match between SVD and Main Tracker segments magnet ON/OFF
  extern void FORTRAN_ROUTINE( matchsvdmt  )();

  /// object with parameters for matchsvdmt\_()
  extern RcMatchConst rcSvdMtConst;

  /// does match between SVD and Outer Tracker segments magnet ON
  extern void FORTRAN_ROUTINE( matchsvdot  )();

  /// object with parameters for matchsvdot\_()
  extern RcMatchConst rcSvdOtConst;

  /// does match between SVD segments and ECAL clusters, magnet ON/OFF
  extern void FORTRAN_ROUTINE(matchsvdecal)( float * threshold_cut ); 

  /// does match between SVD segments and ECAL clusters, magnet OFF
  extern void FORTRAN_ROUTINE(matchsenomag)( float * threshold_cut ); 

  /// does match between RICH segments and ECAL clusters, magnet ON/OFF
  extern void FORTRAN_ROUTINE(matchrichecal)( float * threshold_cut,
					      float * minCherenkov_angle); 

  /// does match between SVD and RICH segments, magnet ON/OFF
  extern void FORTRAN_ROUTINE(matchsvdrich) ();

}

/// check threshold cuts for RCCL
extern int maTakeRccl(float threshold, ArtePointer<RCCL> rccl );



#endif
