//---------------------------------------------------------------------//
//         matching of SVD segments to rich pseudo-segments            //
// This routine calls Olya's marple package and stores relations       //
//    RSEG_RBEG between the two segments found                         //
//     By D. Samtleben / M. Bruinsma 
//
// optimized for Holmes(svd) - RingSearchArte(rich) 20.04.99    olya  
//                                     (magnet OFF) 16.05.99   olya   
//---------------------------------------------------------------------//

    
#include "marple/marple.hh"
#include "marple/RcMatchBuilder.hh"
#include "arte/RSEG_cmp.hh"
#include "arte/arte.hh"

#ifdef MARPLEPERFORMANCES
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
Chrono * chMatchSvdRich = 0;
#endif


RcMatchConst rcSVDRichConst = 
  {
    10 /* = flag: use only ty info */, 
    200. /*= Z value of plane */ , 0. /*= zmin*/ , 1500. /*zmax */ ,
    0 /*use pair cut */,2 /* =n pair left */ ,1	/*= n pair right */,
    1 /*= use delta cut */, 
    1E+10 /* = delta x */ ,     1E+10 /* = delta y */ ,
    1E+10 /*=delta tx*/,       0.003, // 0.01 /*=delta ty */ ,
    1E+10 /* = delta_pe */,
    150 /* = chi2 */
  };

RcMatchConst rcSVDRichNoMagConst = 
  {
    110 /* = flag: use only ty info */, 
    200. /*= Z value of plane */ , 0. /*= zmin*/ , 1500. /*zmax */ ,
    1 /*use pair cut */,2 /* =n pair left */ ,1	/*= n pair right */,
    1 /*= use delta cut */, 
    1E+10 /* = delta x */ ,     1E+10 /* = delta y */ ,
    0.005 /*=delta tx*/,    0.004, // 0.01 /*=delta ty */ ,
    1E+10 /* = delta_pe */,
    120 /* = chi2 */
  };

// soft cuts to search correlation  
RcMatchConst rcSoftSRConst = 
  {
    10 /* = flag: use only ty info */, 
    200. /*= Z value of plane */ , 0. /*= zmin*/ , 1500. /*zmax */ ,
    0 /*use pair cut */,2 /* =n pair left */ ,1	/*= n pair right */,
    1 /*= use delta cut */, 
    1E+10 /* = delta x */ ,   1E+10 /* = delta y */ ,
    1E+10 /*=delta tx*/,      0.005, // 0.01 /*=delta ty */ ,
    1E+10 /* = delta_pe */,
    100000 /* = chi2 */
  };
  

extern "C"
void FORTRAN_ROUTINE(matchsvdrich) ()
{
# ifdef MARPLEPERFORMANCES
  if(chMatchSvdRich == 0){
    ChronoManager CM;
    chMatchSvdRich = CM.createChrono("match Svd Rich");
  }
  chMatchSvdRich->start();
# endif

  RcMatchBuilder<RcSegPoint, RcSegPoint>  SVDRichMatch;

  //------------------------------------------------------------------
  // put the defined constants in this builder:
  //------------------------------------------------------------------
  
  if( MARPLE_SOFT )
    SVDRichMatch.setConst(rcSoftSRConst);
  else
    if( !maMagnetON() )
      SVDRichMatch.setConst(rcSVDRichNoMagConst);
    else
      SVDRichMatch.setConst(rcSVDRichConst);


  int iErr; 

  //------------------------------------------------------------------
  // create two vectors: one for the SVD (RSEG) segments and
  //  one for the (quasi-)segments in the rich
  // first the SVD segment - vector:
  //------------------------------------------------------------------
  
  for( ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
       irseg != ArteTable<RSEG>::end(); ++irseg )
    {
      //if( (*irseg)->cmp & Rsegc::vxd)
      if( (*irseg)->cmp == Rsegc::vxd)
        {         
          RcSegPoint * MyptToPoint = new RcSegPoint( *irseg , 1); // endPoint
	  MyptToPoint->prolongate( rcSVDRichConst.z_plane);
	  SVDRichMatch.addPointLeft (*MyptToPoint);
	  MyptToPoint = NULL;
	}
    }
  if( SVDRichMatch.vectorLeft().size() == 0 ) return;
  
  //------------------------------------------------------------------
  // secondly the quasi-segments from the rich
  //------------------------------------------------------------------
  
  for( ArteTable<RSEG>::iterator richsegment = ArteTable<RSEG>::begin();
       richsegment != ArteTable<RSEG>::end(); ++richsegment )
    {
      //if( (*richsegment)->cmp & Rsegc::rich  &&
      if( (*richsegment)->cmp == Rsegc::rich  &&
	  (*richsegment)->ze/sqrt((*richsegment)->ye) > 5 )
	{
	  RcSegPoint *MyptToPoint2 = new RcSegPoint (*richsegment ,0 );
	  MyptToPoint2->setZ( rcSVDRichConst.z_plane);
	  SVDRichMatch.addPointRight (*MyptToPoint2);
	  MyptToPoint2 = NULL;
	}
    }

  //------------------------------------------------------------------
  // go for it:
  //------------------------------------------------------------------
  SVDRichMatch.doMatch();
  
  
  // and estimate momentum just for case
  for( unsigned int iMatch = 0; 
       iMatch < SVDRichMatch.matchList().size(); iMatch++ )
    {
      RcPoint * svd  = SVDRichMatch.leftOrigin(iMatch);
      RcPoint * rich = SVDRichMatch.rightOrigin(iMatch);
      RcPoint tmp = *svd ;
     
      if( svd->tx() > rich->tx() ) 
	tmp.setpe( 0.1 );
      else
	tmp.setpe( -0.1 );
      tmp.estimatePEfromTAR( rich->tx(), rich->ty(), 1000, 
			     rich->cov()(3,3), rich->cov()(4,4), 
			     0,
			     3, iErr, 0 );
      SVDRichMatch.matchList()[iMatch]->setpe( tmp.pe() );
      SVDRichMatch.matchList()[iMatch]->setcov(5,5, tmp.cov()(5,5) );
    }


  //--------------------------------------------------------------------
  // print result of matching
  //--------------------------------------------------------------------
  if( MARPLE_DEBUG >= 0 && MARPLE_DEBUG != 10 && MARPLE_DEBUG != 20)
    {
      static int firstSVDMT = 1;
      if( firstSVDMT )
        {
          cout << *SVDRichMatch.ptToMatchConst();
          firstSVDMT = 0;
        }
      SVDRichMatch.stat(MARPLE_DEBUG);
    }
      
  if( MARPLE_DEBUG >= 20)
    {
      SVDRichMatch.fillMatch(8114);
      SVDRichMatch.fillDeltas(8214, 8314);
    }else if ( MARPLE_DEBUG >= 10 ){
      SVDRichMatch.fillDeltas(8214);
    }



  //------------------------------------------------------------------
  // save the relations to the RSEG table:
  //------------------------------------------------------------------
  if( MARPLE_MAKERSEG ) SVDRichMatch.saveRSEG(iErr);

  SVDRichMatch.saveRelation(iErr);
  if(iErr)
    {
      cout << "not all relations were saved. " << endl;
    }
 
  SVDRichMatch.deleteAll();
# ifdef MARPLEPERFORMANCES
    chMatchSvdRich->stop();
# endif
}

