/* Cable<AnyIChamber>
   -----------------------------------------
   Can be used for any Setup.
   Cable<AnyIChamber> = vector<ISection_Cell> + AnyIChamber ichamber;
   Note that there is one ichamber for all cable channels.
   This means that all wires in the cable are connected
   to the same chamber. (There is no cable which is connected to
   2 chambers simultaneously, for example).
   Note also that the Cable<AnyIChamber> class is parameterized by
   AnyIChamber, not, e.g., AnySetup.

   Cable channel can be **not** connected to the cell. In this case
   its ISection_Cell.isection == chamber.end().
   This check can be done via 
     bool is_connected(unsigned int n);
   where n is the channel number inside cable.

   Cables<AnyIChamber>
   --------------------------------
   is a map<string, Cable<AnyIChamber> >

   Usually one should use Cables<AnyIChamber> and probably
   FEDs<AnyIChamber> (defined in ptset/FEDs.hh)

   As an example see 
     ptset/usage_of_cables_and_FEDs.example_readme
   */

// Author V.Balagura (balagura@mail.desy.de)   (21.02.99).

#ifndef _cables_hh
#define _cables_hh 1

#include <vector>
#include <map>
#include <string>

template <class AnyIChamber>
struct Cable 
  : public vector<typename 
iterator_traits<AnyIChamber>::value_type::second_type::ISection_Cell> {
  AnyIChamber ichamber;
  bool is_connected(unsigned int n) {
    return (*this)[n].isection != (*ichamber).second.end();
  }
};

template <class AnyIChamber>
struct Cables : public map<string, Cable<AnyIChamber>, less<string> > {
};

#endif
