// 
//  Author:  Rainer Mankel                     Created:  01-Apr-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// class rangerDomLay

#include <math.h>
#include "rang/DomLay.hh"
#include "rang/DomWin.hh"
#include "rang/LayPl.hh"
#include "rang/SNM.hh"
#include "rang/CompLaySet.hh"
#include "rang/CompLay.hh"
#include "rang/Geom.hh"

extern "C" {
  void tfxrang_(const int& ipl,float& xmin,float& xmax);
}

rangerDomLay::rangerDomLay(rangerDomWin* wi) {
  window = wi;
  for (int m=0; m<=2; ++m) {
    lsn[m] = 0;
    prv[m] = 0;
    nxt[m] = 0;
  }
  zRef = 0;
  Method = 0;
  alphaStereo = 0;
  hL = 0;
}

rangerDomain* rangerDomLay::dom() const { 
  return (window? window->dom(): 0); 
}

int rangerDomLay::accum(rangerDomWin* wi,int gLayer,
			rangerSNM* s,const int method,
			const int subGLay1,const int subGLay2) {
  // method=-1 means: first accumulated layer plane decides method
  int nAcc = 0;
  double zAcc = 0;
  if (s) {
    // loop over SNM
    for (vector<pair<int,int> >::const_iterator n=s->neigh().begin();
	 n != s->neigh().end(); ++n) {
      // loop over SL in dom
      for (vector<ArtePointer<GESL> >::const_iterator 
	     sI=wi->dom()->domSl().begin();
	   sI != wi->dom()->domSl().end(); ++sI) {
	if ((*sI)->cmp != (*n).first) continue;
// 	cout << "...........search for cmp/sec/glayer="
// 	     << (*n).first << "/" << (*n).second << "/" << gLayer << endl;
	for (ArteRelationIterator<GEDE> gI=(*sI)->gede.begin();
	     gI != (*sI)->gede.end(); ++gI) {
	  if ((*gI)->sec != (*n).second) continue;
	  if ((*gI)->lay != gLayer) continue;

	  // check method
	  int gMeth = (fabs(rangerGeom::vol( (*gI) )->Sin()) < 0.010)? 1:2;
	  //cout << "gede=" << (*gI) << " gMeth=" << gmeth << endl;

#ifdef GTEST
	  ArtePointer<GEDE> g1(656);
	  if ((*gI) == g1) {
	    cout << " gI=" << *gI << " dom=" 
		 << wi->dom()->nam() 
		 << " cmp/sec=" << wi->cmp() << "/" << wi->sec()
		 << " gl=" << gLayer
		 << " meth=" << method
		 << "gMeth=" << gMeth 
		 << " Method=" << Method << endl;
	  }
#endif

	  if (method > 0) {
	    if (gMeth != method) continue;
	  }
	  else if (method == -1) {
	    if (Method) {
	      if (gMeth != Method) continue;
	    }
	  }
#ifdef GTEST
	  if ((*gI) == g1) cout << "after method check" << endl;
#endif

	  // check overlap with sublayer range
	  
	  if (subGLay1 > (*gI)->nsl || subGLay2 < 1
	      || subGLay1 > subGLay2) continue;
#ifdef GTEST
	  if ((*gI) == g1) cout << "after subl check" << endl;
#endif
	  // check overlap with domain window
	  float xMin = 0;
	  float xMax = -1;
	  tfxrang_((*gI).index(),xMin,xMax);
	  if (xMin > wi->xmax() || xMax < wi->xmin()) continue;
#ifdef GTEST
	  if ((*gI) == g1) cout << "after txrang check" << endl;
#endif

	  // create new domain layer plane
	  addLayPl(new rangerLayPl(this,*gI,subGLay1,subGLay2));
	  ++nAcc;
	  zAcc += 0.5 * ((*gI)->zmin + (*gI)->zmax);

	  // set method if needed
	  if (method == -1 && ! Method) {
	    Method = gMeth;
	    alphaStereo = asin( rangerGeom::vol( (*gI) )->Sin() );
	  }
	  
	  // check the stereo angle
	  if (Method) {
	    if (fabs(asin(rangerGeom::vol( (*gI) )->Sin()) - alphaStereo)
		> 0.01) {
	      cout << " () rangerDomLay::accum: stereo angles incompatible: "
		   << 57.295*asin(rangerGeom::vol( (*gI) )->Sin())
		   << " and " << 57.295*alphaStereo << endl;
	      cout << "              wi=" << *wi << " gede=" << *gI 
		   << " cmp=" << (*gI)->cmp << " gLayer=" << gLayer << endl;
	    }
	  }
	}
      }
    }
  }
  else {
    cout << "rangerDomLay::accum: SNM* is required" << endl;
  }
  if (nAcc) zRef = zAcc / nAcc;
  return nAcc;
}

// int rangerDomLay::accumVLT(rangerDomWin* wi,int gLayer,
// 			   rangerSNM* s,const int method,
// 			   rangerVLTSet* vlts) {

//   if (vlts) {
//     rangerVLT* vlt = vlts->getVLT(wi->cmp());
//     if (vlt) {
//       rangerSubLayerSet* sls = vlt->getSLS(gLayer);
//       if (sls) {
// 	return accum(wi,sls->gLayer,s,

// }

int rangerDomLay::accumComp(rangerDomWin* wi,int gLayer,rangerSNM* s,
			    rangerCompLaySet& clset,int method=-1) {
  // method=-1 means: first accumulated layer plane decides method
  int nAcc = 0;
  double zAcc = 0;
  if (s) {
    // loop over SNM
    for (vector<pair<int,int> >::const_iterator n=s->neigh().begin();
	 n != s->neigh().end(); ++n) {

      // translate layer number using composite layer set
      rangerCompLay* cl = clset.findCl(s->cmp(),gLayer);
      if (cl) {
	rangerSubLayerSet* slset = cl->slSet((*n).first);

#ifdef FINDWARN
	if (! slset) cout << "rangerDomLay::accumComp: slset=" 
			  << slset << endl;
#endif

	if (slset) {
	  nAcc += accumCmpSec(wi,(*n).first,(*n).second,slset->gLayer,
			      method,slset->subLay1,
			      slset->subLay2,zAcc);
	}
      }
    }
  }
  if (nAcc) zRef = zAcc / nAcc;
  return nAcc;
}


int rangerDomLay::accumCmpSec(rangerDomWin* wi,int cmp,int sec,int gLayer,
			      const int method,
			      const int subGLay1,const int subGLay2,
			      double& zAcc) {

  int nAcc = 0;
  for (vector<ArtePointer<GESL> >::const_iterator 
	 sI=wi->dom()->domSl().begin();
       sI != wi->dom()->domSl().end(); ++sI) {
    if ((*sI)->cmp != cmp) continue;
// 	cout << "...........search for cmp/sec/glayer="
// 	     << (*n).first << "/" << (*n).second << "/" << gLayer << endl;
    for (ArteRelationIterator<GEDE> gI=(*sI)->gede.begin();
	 gI != (*sI)->gede.end(); ++gI) {
      if ((*gI)->sec != sec) continue;
      if ((*gI)->lay != gLayer) continue;

      // check method
      int gMeth = (fabs(rangerGeom::vol( (*gI) )->Sin()) < 0.010)? 1:2;
      //cout << "gede=" << (*gI) << " gMeth=" << gmeth << endl;

#ifdef GTEST
      ArtePointer<GEDE> g1(656);
      if ((*gI) == g1) {
	cout << " gI=" << *gI << " dom=" 
	     << wi->dom()->nam() 
	     << " cmp/sec=" << wi->cmp() << "/" << wi->sec()
	     << " gl=" << gLayer
	     << " meth=" << method
	     << "gMeth=" << gMeth 
	     << " Method=" << Method << endl;
      }
#endif

      if (method > 0) {
	if (gMeth != method) {
	  cout << "rangerDomLay::accumCmpSec: method mismatch (1): "
	     << " gMeth, method=" << gMeth << " " << method << endl;
	}
	if (gMeth != method) continue;
      }
      else if (method == -1) {
	if (Method) {
	  if (gMeth != Method) {
	    cout << "rangerDomLay::accumCmpSec: method mismatch (2): "
		 << " gMeth,Method=" << gMeth << " " << Method << endl;
	  }
	  if (gMeth != Method) continue;
	}
      }
#ifdef GTEST
      if ((*gI) == g1) cout << "after method check" << endl;
#endif

      // check overlap with sublayer range
            if (subGLay1 > (*gI)->nsl || subGLay2 < 1
	  || subGLay1 > subGLay2) continue;
#ifdef GTEST
      if ((*gI) == g1) cout << "after subl check" << endl;
#endif
      // check overlap with domain window
      float xMin = 0;
      float xMax = -1;
      tfxrang_((*gI).index(),xMin,xMax);
//       if (xMin > wi->xmax() || xMax < wi->xmin()) {
// 	cout << "rangerDomLay::accumCmpSec: no overlap, "
// 	     << " ge=" << (*gI).index() << endl;
//       }

      if (xMin > wi->xmax() || xMax < wi->xmin()) continue;
#ifdef GTEST
      if ((*gI) == g1) cout << "after txrang check" << endl;
#endif

      // create new domain layer plane
      addLayPl(new rangerLayPl(this,*gI,subGLay1,subGLay2));
      ++nAcc;
      zAcc += 0.5 * ((*gI)->zmin + (*gI)->zmax);

      // set method if needed
      if (method == -1 && ! Method) {
	Method = gMeth;
	alphaStereo = asin( rangerGeom::vol( (*gI) )->Sin() );
      }
	  
      // check the stereo angle
      if (Method) {
	if (fabs(asin(rangerGeom::vol( (*gI) )->Sin()) - alphaStereo)
	    > 0.01) {
	  cout << " () rangerDomLay::accumCmpSec: stereo angles incompatible: "
	       << 57.295*asin(rangerGeom::vol( (*gI) )->Sin())
	       << " and " << 57.295*alphaStereo << endl;
	  cout << "              wi=" << *wi << " gede=" << *gI 
	       << " cmp=" << (*gI)->cmp << " gLayer=" << gLayer << endl;
	}
      }
    }
  }

#ifdef ACCUMWARN
  if (nAcc == 0) {
    cout << "rangerDomLay::accumCmpSec: failed for "
	 << " win=" << *win()
	 << " cmp="  << cmp 
	 << " sec=" << sec << " gl=" << gLayer << endl;
  }
#endif

  return nAcc;
}



	
	



int rangerDomLay::cntLP(int cm,int sc){

  int n = 0;
  for (vector<rangerLayPl*>::iterator li=lpls.begin();
       li != lpls.end(); ++li) {
    if (cm > 0 && (*li)->ge()->cmp != cm) continue;
    if (sc > 0 && (*li)->ge()->sec != sc) continue;
    ++n;
  }
  return n;
}


void rangerDomLay::updLsn(int* inLsn){
  ++inLsn[0];
  lsn[0] = inLsn[0];
  lsn[1] = 0;
  lsn[2] = 0;
  if (Method >=1 && Method <=2) {
    ++inLsn[Method];
    lsn[Method] = inLsn[Method];
  }
  else {
    cout << "rangerDomLay::updLsn: Oh no!!! Method=" << Method << endl;
  }
}


void rangerDomLay::updPrv(rangerDomLay* prev[]) {
  for (int m=0; m<=2; ++m) {
    prv[m] = prev[m];
  }
}


void rangerDomLay::updNxt(rangerDomLay* next[]) {
  for (int m=0; m<=2; ++m) {
    nxt[m] = next[m];
  }
}


ostream& operator<<(ostream &os,const rangerDomLay &dl){
  os << "(" << dl.lsn[0] << ")";
  return os;
}



void rangerDomLay::dump(const int level){
  if (level > 0) {
    cout << "   DomLay zRef=" << zRef 
	 << " method " << Method
	 << " lsn " << lsn[0] << " " << lsn[1] << " " << lsn[2] << endl;
    
    cout << "                                next ";
    if (nxt[0]) cout << *nxt[0];
    cout << "/";
    if (nxt[1]) cout << *nxt[1];
    cout << "/";
    if (nxt[2]) cout << *nxt[2];
    cout << " prev ";
    if (prv[0]) cout << *prv[0];
    cout << "/";
    if (prv[1]) cout << *prv[1];
    cout << "/";
    if (prv[2]) cout << *prv[2];
    cout << endl;

    for (vector<rangerLayPl*>::iterator li=lpls.begin();
	 li != lpls.end(); ++li) {
      (*li)->dump(level-1);
    }
  }
}
