// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include <math.h>
#include "arte/arte.hh"
#include "rang/Node.hh"
#include "rang/Track.hh"
#include "rang/Fit.hh"
#include "rang/zfpar.hh"
#include "rang/hfill.hh"
#include "rang/FitMode.hh"
#include "rang/RMap.hh"
#include "rang/Geom.hh"
#include "rang/FLAGHist.hh"
#include "rang/rginit.hh"

int rangerFit(ArtePointer<RSEG> rseg,
	      rangerParticleType pType) {

  rginit_();
  rangerGeom::condUpdate();    // initialization

  rangerRMap rmap(rseg);                  // create base of R table mapping
  rangerTrack t(rseg,ModeP,pType,&rmap);  // create track from RSEG

  int stat = t.fit();

  if (stat < 0) {
    rseg->fit = 10;
    return -10;
  }

#ifdef rangerHIST
  int id=6;
  float rad = t.totalRadThickness();
  float y = 0;
  float w=1;
  hfill_(id,rad,y,w);
#endif

  t.storeback(&rmap);

  if (rseg->cmp == 0) rseg->cmp = t.genCmp();

#ifdef LISTHITS
  for (ArteTableIterator<GEPT> geptIt; geptIt.valid(); geptIt.next()){
    if (geptIt->hit1 == 0) {
      if (geptIt->mim1 > 0) {
	cout << "plane " << geptIt << " has no hits but mimps " << endl;
      }
    }
  }
#endif

  return 0;
}



int rangerFit(vector<ArtePointer<RSEG> > rsegVec,
	      rangerParticleType pType) {

  rginit_();
  rangerGeom::condUpdate();    // initialization

  vector<rangerRMap> rmapVec;
  rangerTrack t(rsegVec,ModeP,pType,&rmapVec);  // create track from RSEG

  int stat = t.fit();

  if (stat < 0) return stat;

#ifdef rangerHIST
  int id=6;
  float rad = t.totalRadThickness();
  float y = 0;
  float w=1;
  hfill_(id,rad,y,w);
#endif

  t.storeback(&rmapVec);
#ifdef LISTHITS
  for (ArteTableIterator<GEPT> geptIt; geptIt.valid(); geptIt.next()){
    if (geptIt->hit1 == 0) {
      if (geptIt->mim1 > 0) {
	cout << "plane " << geptIt << " has no hits but mimps " << endl;
      }
    }
  }
#endif
  return 0;
}



int rangerFit(ArtePointer<RTRA> rtra,
	      rangerParticleType pType) {

  rginit_();
  rangerGeom::condUpdate();    // initialization

  rangerRMap rmap(rtra);                  // create base of R table mapping
  rangerTrack t(rtra,ModeP,pType,&rmap);  // create track from RSEG

  int stat = t.fit();

  if (stat < 0) {
    rtra->fit = 10;
    return stat;
  }

  t.storeback(&rmap);
  return 0;
}
