// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified: 29-Mar-99
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include <stdio.h>
#include <algo.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include <arte/artefunc.h>
#include "rang/Geom.hh"
#include "rang/GeWa.hh"
#include "rang/Gede.hh"
#include "rang/Gwal.hh"
#include "rang/sortzv.hh"
#undef SHOWSORT

int rangerGeom::nVolume = 0;               // Initialize static members
//rangerVolume** rangerGeom::volume = 0;
//rangerVolume** rangerGeom::PGede = 0;
//rangerVolume** rangerGeom::PGwal = 0;
//rangerVolume* rangerGeom::firstVolume = 0;
//rangerVolume* rangerGeom::lastVolume = 0;
vector<rangerVolume*> rangerGeom::volumes;
vector<vector<rangerVolume*>::iterator> rangerGeom::vGede;
vector<vector<rangerVolume*>::iterator> rangerGeom::vGwal;

void rangerGeom::update() {
  reset();
  build();
  gsort();
  arteMsg << "rangerGeom::update: geometry table (re)built" << 
    " with " << nVolume << " volumes" << endl;
  arteMsg.warn();
};

void rangerGeom::condUpdate() {
  int nVol = ArtePointer<GEDE>::nrow() + ArtePointer<GWAL>::nrow();
  if (nVol != nVolume) {
    update();
  }
};

void rangerGeom::reset() {
//   if (volume) {
//     for (int i=0; i<nVolume; i++) delete volume [i];
//   }
  nVolume = 0;
  //delete PGede;
  //delete PGwal;
  //delete volume;
  for (vector<rangerVolume*>::iterator vIt=volumes.begin();
       vIt != volumes.end(); ++vIt) {
    delete (*vIt);
  }
  volumes.erase(volumes.begin(),volumes.end());
  vGede.erase(vGede.begin(),vGede.end());
  vGwal.erase(vGwal.begin(),vGwal.end());
};

void rangerGeom::build() {
  //PGede = new rangerVolume* [ArtePointer<GEDE>::nrow()];
  //PGwal = new rangerVolume* [ArtePointer<GWAL>::nrow()];
  nVolume = ArtePointer<GEDE>::nrow() + ArtePointer<GWAL>::nrow();
  //volume = new rangerVolume* [nVolume];
  //for (int i=0; i<nVolume; i++) volume [i] = 0;

  //rangerVolume** volumeIt =  volume;    // iterator for volume array
  volumes.reserve(ArtePointer<GEDE>::nrow()+ArtePointer<GWAL>::nrow());
  for ( ArteTable<GEDE>::iterator gede = ArteTable<GEDE>::begin();
	gede != ArteTable<GEDE>::end(); ++gede ) {
    rangerGede* Gede = new rangerGede( gede ); // allocate new Gede
    rangerGeWa* GeWa = Gede;
    rangerVolume* vol = rangerVolume::makeVolume( GeWa );
    //cout << "build: made volume from gede=" << gede 
    //	 << " ms=" << vol->ms() << endl;
    //*volumeIt = vol;

    volumes.push_back(vol);

    //int ge = gede.index();
    //PGede[ge-1] = *volumeIt;
    //printf("build: volume %d stored at %p, pointers address is %p\n",
    //	   vol->plNumber(),*volumeIt,volumeIt);
    //volumeIt++;
  };
  for ( ArteTable<GWAL>::iterator gwal = ArteTable<GWAL>::begin();
	gwal != ArteTable<GWAL>::end(); ++gwal ) {
    rangerGwal* Gwal = new rangerGwal( gwal ); // allocate new Gwal
    rangerGeWa* GeWa = Gwal;
    rangerVolume* vol = rangerVolume::makeVolume( GeWa );
    //cout << "build: made volume from gwal=" << gwal 
    //	 << " ms=" << vol->ms() << endl;
    //*volumeIt = vol;

    volumes.push_back(vol);

    //PGwal[gwal.index()-1] = *volumeIt;
    //printf("build: volume %d stored at %p, pointers address is %p\n",
    //	   vol->plNumber(),*volumeIt,volumeIt);
    //volumeIt++;
  };
};

void rangerGeom::gsort() {
//   float* zArray = new float [nVolume];                  // array for z values
//   rangerVolume** vArray = new rangerVolume* [nVolume];  // array to hold
//                                                        // active volumes
//   long int* index = new long int [nVolume];             // array for index
//   float* zIt = zArray;                             // iterator for zArray
//   rangerVolume** vIt = vArray;                     // iterator for vArray
//   rangerVolume** volumeIt = volume;                // iterator for volume
//   long int nElem = 0;
//   for (int i=0; i<nVolume; i++) {
//     if (*volumeIt) {
//       *zIt = (*volumeIt)->zMean();
//       *vIt = *volumeIt;
// //    if (((*volumeIt)->getGeWa())->shp() == 6) {
// // 	rangerGeWa* geeee = (*volumeIt)->getGeWa();
// // 	cout << "rangerGeom::gsort(): gewa " << geeee->plNumber()
// // 	     << "stored with z=" << *zIt << endl;
// //    }
//       zIt++;
//       vIt++;
//       nElem++;
//     }
//     volumeIt++;
//   }
//   int mode = 1;
//   int way = 0;
//   int nsort = 0;
//   sortzv_(zArray,index,nElem,mode,way,nsort);
//   for (int j=0; j<nElem; j++) {
//     if (j>0) {
//       vArray[index[j]-1]->getPrevious() = vArray[index[j-1]-1];
//     }
//     if (j<(nElem-1)) {
//       vArray[index[j]-1]->getNext() = vArray[index[j+1]-1];
//     }
//   }
//   if (nElem >= 1) {
//     firstVolume = vArray[index[0]-1];
//     lastVolume = vArray[index[nElem-1]-1];
//   }

  sort(volumes.begin(),volumes.end(), compareCall( &rangerVolume::zMean ) );

//   for (vector<rangerVolume*>::iterator vi=volumes.begin();
//        vi != volumes.end(); ++vi) {
//     cout << "vsn " << vi-volumes.begin() 
// 	 << " pl=" << (*vi)->plNumber()
// 	 << "z= " << (*vi)->zMean() << endl;
//   }

  // initialize the vGede and vGwal pointers
  vGede.reserve(ArtePointer<GEDE>::nrow());
  for ( ArteTable<GEDE>::iterator gede = ArteTable<GEDE>::begin();
	gede != ArteTable<GEDE>::end(); ++gede ) {
      vGede.push_back(volumes.end());
  }
  vGwal.reserve(ArtePointer<GWAL>::nrow());
  for ( ArteTable<GWAL>::iterator gwal = ArteTable<GWAL>::begin();
	gwal != ArteTable<GWAL>::end(); ++gwal ) {
      vGwal.push_back(volumes.end());
  }
  for (vector<rangerVolume*>::iterator vi=volumes.begin();
       vi != volumes.end(); ++vi) {
    if ((*vi)->plNumber() > 0) {
      if ((*vi)->plNumber() > (int) ArtePointer<GEDE>::nrow()) {
	cout << "warning: gede::nrow=" << ArtePointer<GEDE>::nrow()
	     << " pln=" << (*vi)->plNumber() << endl;
      }
      vGede[(*vi)->plNumber()-1] = vi;
    }
    else {
      if ((*vi)->plNumber() < 0) {
	if (-(*vi)->plNumber() > (int) ArtePointer<GWAL>::nrow()) {
	  cout << "warning: gwal::nrow=" << ArtePointer<GWAL>::nrow()
	       << " pln=" << (*vi)->plNumber() << endl;
	}
	vGwal[-(*vi)->plNumber()-1] = vi;
      }
      else {
	cout << "plNumber zero!" << endl;
      }
    }
  }

//   for ( ArteTable<GEDE>::iterator gede = ArteTable<GEDE>::begin();
// 	gede != ArteTable<GEDE>::end(); ++gede ) {
//     if (vGede[gede.index()-1] != volumes.end()) {
//       cout << "gede=" << *gede << " z=" << (*gede)->zmin
// 	   << " volume getgede=" << (*vGede[gede.index()-1])->plNumber() << endl;
//     }
//   }

//   for ( ArteTable<GWAL>::iterator gwal = ArteTable<GWAL>::begin();
// 	gwal != ArteTable<GWAL>::end(); ++gwal ) {
//     cout << "gwal=" << *gwal << " z=" << (*gwal)->zmin
// 	 << " volume getgwal=" << (*vGwal[gwal.index()-1])->getGwal() << endl;
//   }




#ifdef SHOWSORT
//   for (rangerVolume* volIt=firstVolume; 
//        volIt; volIt=volIt->next()){
//     cout << "  z=" << volIt->zMean()
// 	 << " plane=" << volIt->plNumber() << endl;
//   }
//   cout << endl << endl << endl << endl;
//   for (rangerVolume* volIt=lastVolume; 
//        volIt; volIt=volIt->previous()){
//     cout << "  z=" << volIt->zMean()
// 	 << " plane=" << volIt->plNumber() << endl;
//   }
#endif





}
  

