// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include <stdio.h>
#include <math.h>
#include <algo.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "rang/Node.hh"
#include "rang/Track.hh"
#include "rang/seg2track.hh"
#include "rang/zfpar.hh"
#include "gean/ZGESL.h"

extern "C" {
  void tfremcn_(const int& it,int& imc,float& fracmc);
}

ArtePointer<RTRA> rangerSeg2track(ArtePointer<RSEG> rseg){
  // ... here we should check if segment is valid

  if ( rseg.index() <= 0 ) {
    arteMsg << "rangerSeg2track: invalid segment " << rseg << endl;
    arteMsg.warn();
    return 0;
  }

  // create new entry in RTRA. 

  mmbook_(ArtePointer<RTRA>::getLink().getRef(),1);
  ArtePointer<RTRA> rtra( ArtePointer<RTRA>::nrow() );

  rtra->zf = rseg->zf;
  rtra->xf = rseg->xf;
  rtra->yf = rseg->yf;
  rtra->txf = rseg->txf;
  rtra->tyf = rseg->tyf;
  rtra->pf = rseg->pf;
  for (int i=0; i<=14; i++) {
    rtra->cvf[i] = rseg->cvf[i];
  }
  rtra->hsi = 0;
  rtra->hin = 0;
  rtra->hou = 0;
  rtra->hri = 0;
  rtra->hca = 0;
  rtra->htr = 0;
  rtra->hmu = 0;
  rtra->fit = rseg->fit;
  rtra->chi = rseg->chi;

  // count the hits
  int nRhit = 0;
  for ( ArteRelationIterator<RHIT> rhitIt = rseg->rhit.begin();
	rhitIt != rseg->rhit.end(); ++rhitIt ) {
    ArtePointer<RHIT> rhit(*rhitIt);
    rhit->rtra = rtra.index();
    nRhit++;
    ArtePointer<GEDE> ge(rhit->hitb->gede);

    switch (ge->cmp) {
    case LGESL_SIL:
      rtra->hsi++;
      break;
    case LGESL_WCH:
      rtra->hou++;
      break;
    case LGESL_MSG:
      rtra->hin++;
      break;
    case LGESL_RIC:
      rtra->hri++;
      break;
    case LGESL_TRD:
      rtra->htr++;
      break;
    case LGESL_ECA:
      rtra->hca++;
      break;
    case LGESL_MUO:
      rtra->hmu++;
      break;
    case LGESL_MUX:
      rtra->hmu++;
      break;
    default:
      cout << " seg2track: bad rhit cmp=" << ge->cmp << endl;
    }
  }

  // since the relation look-up sequence is reverse to that
  // of the booking, we must revert the sequence to book
  // the same hit sequence in the RTRA as in the RSEG

  //vector<ArtePointer<RHIT> > rhVec( nRhit );
  vector<ArtePointer<RHIT> > rhVec;
  copy( rseg->rhit.begin(), rseg->rhit.end(), back_inserter( rhVec ) );

  for( vector<ArtePointer<RHIT> >::reverse_iterator iRhit = rhVec.rbegin();
       iRhit != rhVec.rend(); iRhit++ ) {
    if( (*iRhit)->hitb != 0 ) {
#ifdef FULL
      cout << "single seg relate track" << rtra << " and hitb " 
	   << (*iRhit)->hitb << endl;
#endif
      createRelation( rtra, (*iRhit)->hitb, &RTRA::hitb );
    }
  }

#ifdef FULL
  cout << "The following HITBs are related to rseg=" << rseg << endl;
  for ( ArteRelationIterator<RSEG_HITB> rhitIt( rseg->hitb );
	rhitIt.valid(); rhitIt.next() ) {
    ArtePointer<HITB> hitb(rhitIt);
    cout << hitb << " ";
  }
  cout << "---------------------------------------------------" << endl;


  cout << "The following HITBs are related to rtra=" << rtra << endl;
  {
    ArtePointer<RTRX> rtrx = ArtePointer<RTRX> (rtra);
    for ( ArteRelationIterator<RTRX_HITB> rhitIt( rtrx->hitb );
	  rhitIt.valid(); rhitIt.next() ) {
      ArtePointer<HITB> hitb(rhitIt);
      cout << hitb << " ";
    }
  }
  cout << "---------------------------------------------------" << endl;
#endif

  // copy MTRA relations from RSEG
  for ( ArteRelationIterator<MTRA> mtraIt = rseg->mtra.begin();
 	mtraIt != rseg->mtra.end(); ++mtraIt ) {
    createRelation( rtra, *mtraIt, &RTRA::mtra );
  }

#ifdef FULL
  cout << "The following MTRAs are related to rtra=" << rtra << endl;
#endif
  ArtePointer<RPNT> rpHighZ = 0;

  // copy RPNT relations from RSEG
  for ( ArteRelationIterator<RPNT> rpntIt = rseg->rpnt.begin();
	rpntIt != rseg->rpnt.end(); ++rpntIt ) {
    ArtePointer<RPNT> rp(*rpntIt);
    createRelation( rtra, rp, &RTRA::rpnt );
    if (rp->zf == rseg->ze) {
      rpHighZ = rp;
    }
  }

// set relations to RSEG

  createRelation( rtra, rseg, &RTRA::rseg );

  // create RPNT for last point
  if (rpHighZ == 0) {
    mmbook_(ArtePointer<RPNT>::getLink().getRef(),1);
    rpHighZ = ArtePointer<RPNT>( ArtePointer<RPNT>::nrow() );
    rpHighZ->zf = rseg->ze;
    rpHighZ->xf = rseg->xe;
    rpHighZ->yf = rseg->ye;
    rpHighZ->txf = rseg->txe;
    rpHighZ->tyf = rseg->tye;
    rpHighZ->pf = rseg->pe;
    for (int i=0; i<15; i++) {
      rpHighZ->cvf[i] = rseg->cve[i];
    }
    createRelation( rtra, rpHighZ, &RTRA::rpnt );
  }

  // copy RCCL relations from RSEG
  for ( ArteRelationIterator<RCCL> rcclIt = rseg->rccl.begin();
	rcclIt != rseg->rccl.end(); ++rcclIt ) {
    ArtePointer<RCCL> rccl(*rcclIt);
    createRelation( rtra, rccl, &RTRA::rccl );
#ifdef FULL
    cout << "seg2track(1): relate " << rtra << " and " << rccl << endl;
#endif
  }

#ifdef PULLS
  ArtePointer<RTRX> rtrx = ArtePointer<RTRX> (rtra);
  for ( ArteRelationIterator<RTRX_MTRA> mtraIt( rtrx->mtra );
	mtraIt.valid(); mtraIt.next() ) {
    ArtePointer<MTRA> mtra(mtraIt);
#ifdef FULL
    cout << mtra << " " << endl;
#endif
    int it = rtra.index();
    int imc = mtra.index();
    float pa[4];
    float pamc[4];
    float dpa[4];
    float dpanorm[4];
    float ch2mc;
    int stat;
    tftpar2_(it,imc,pa[0],pamc[0],dpa[0],dpanorm[0],ch2mc,stat);
  
    cout << endl << endl;
    cout << "---------------------------------------------------" << endl;
    printf("%4s %12s %12s %12s %12s \n","i","pRC","pMC","delta","pull");
    cout << "---------------------------------------------------" << endl;
    for (int i=0; i<=3; i++) {
      printf("%4d %12f4  %12f4 %12f4 %12f4 \n",
	     i,pa[i],pamc[i],dpa[i],dpanorm[i]);
    }
    cout << "---------------------------------------------------" << endl;
    cout << "  ch2mc=" << ch2mc << " stat=" << stat << endl;
    cout << "---------------------------------------------------" << endl;
  }
#endif

  return rtra;

}




ArtePointer<RTRA> rangerSeg2track(vector<ArtePointer<RSEG> > rsegVec){
  // ... here we should check if segment is valid

#ifdef FULL
  cout << "rangerSeg2track(rsegVec) has been called with rsegVec=";
  for (vector<ArtePointer<RSEG> >::iterator I=rsegVec.begin();
       I != rsegVec.end(); ++I) {
    cout << " " << *I;
  }
  cout << endl;
#endif

  if ( rsegVec.size() <= 0 ) {
    arteMsg << "rangerSeg2track: invalid segment vector" << endl;
    arteMsg.warn();
    return 0;
  }

  ArtePointer<RSEG> rsLowZ = rsegVec[0];
  ArtePointer<RSEG> rsHighZ = *(rsegVec.rbegin());

  if ( rsLowZ.index() <= 0 ) {
    arteMsg << "rangerSeg2track: invalid first segment " << rsLowZ << endl;
    arteMsg.warn();
    return 0;
  }

  // create new entry in RTRA. 

  mmbook_(ArtePointer<RTRA>::getLink().getRef(),1);
  ArtePointer<RTRA> rtra( ArtePointer<RTRA>::nrow() );

  rtra->zf = rsLowZ->zf;
  rtra->xf = rsLowZ->xf;
  rtra->yf = rsLowZ->yf;
  rtra->txf = rsLowZ->txf;
  rtra->tyf = rsLowZ->tyf;
  rtra->pf = rsLowZ->pf;
  for (int i=0; i<=14; i++) {
    rtra->cvf[i] = rsLowZ->cvf[i];
  }

  rtra->hsi = 0;
  rtra->hin = 0;
  rtra->hou = 0;
  rtra->hri = 0;
  rtra->hca = 0;
  rtra->htr = 0;
  rtra->hmu = 0;
  rtra->fit = 0;
  rtra->chi = 0;

  // count the hits
  int nRhit = 0;
  for (vector<ArtePointer<RSEG> >::const_iterator sIt=rsegVec.begin();
       sIt != rsegVec.end(); sIt++) {
    rtra->chi += (*sIt)->chi;
    if ((*sIt)->fit > rtra->fit) {
      rtra->fit = (*sIt)->fit;
    }
    for ( ArteRelationIterator<RHIT> rhitIt = (*sIt)->rhit.begin();
	  rhitIt != (*sIt)->rhit.end(); ++rhitIt ) {
      ArtePointer<RHIT> rhit(*rhitIt);
      rhit->rtra = rtra.index();
      nRhit++;
      ArtePointer<GEDE> ge(rhit->hitb->gede);
      switch (ge->cmp) {
      case LGESL_SIL:
	rtra->hsi++;
	break;
      case LGESL_WCH:
	rtra->hou++;
	break;
      case LGESL_MSG:
	rtra->hin++;
	break;
      case LGESL_RIC:
	rtra->hri++;
	break;
      case LGESL_TRD:
	rtra->htr++;
	break;
      case LGESL_ECA:
	rtra->hca++;
	break;
      case LGESL_MUO:
	rtra->hmu++;
	break;
      case LGESL_MUX:
	rtra->hmu++;
	break;
      default:
	cout << " seg2track: bad rhit cmp=" << ge->cmp << endl;
      }
    }
  }

  // since the relation look-up sequence is reverse to that
  // of the booking, we must revert the sequence to book
  // the same hit sequence in the RTRA as in the RSEG
  //  ArtePointer<RHIT>* a = new ArtePointer<RHIT> [nRhit];

  vector<ArtePointer<RHIT> > rhVec;
  for (vector<ArtePointer<RSEG> >::const_iterator sIt=rsegVec.begin();
       sIt != rsegVec.end(); sIt++) {
    copy( (*sIt)->rhit.begin(), (*sIt)->rhit.end(),
	  back_inserter( rhVec ) );
  }


  for( vector<ArtePointer<RHIT> >::reverse_iterator iRhit = rhVec.rbegin();
       iRhit != rhVec.rend(); iRhit++ ) {
    if( (*iRhit)->hitb != 0 ) {
#ifdef FULL
      cout << "relate track" << rtra << " and hitb " 
	   << (*iRhit)->hitb << endl;
#endif
      createRelation( rtra, (*iRhit)->hitb, &RTRA::hitb );
    }
  }

#ifdef FULL
  for (vector<ArtePointer<RSEG> >::const_iterator sIt=rsegVec.begin();
       sIt != rsegVec.end(); sIt++) {
    cout << "The following HITBs are related to rseg=" << (*sIt) << endl;
    for ( ArteRelationIterator<RSEG_HITB> rhitIt( (*sIt)->hitb );
	  rhitIt.valid(); rhitIt.next() ) {
      ArtePointer<HITB> hitb(rhitIt);
      cout << hitb << " ";
    }
    cout << "---------------------------------------------------" << endl;
  }

  cout << "The following HITBs are related to rtra=" << rtra << endl;
  {
    ArtePointer<RTRX> rtrx = ArtePointer<RTRX> (rtra);
    for ( ArteRelationIterator<RTRX_HITB> rhitIt( rtrx->hitb );
	  rhitIt.valid(); rhitIt.next() ) {
      ArtePointer<HITB> hitb(rhitIt);
      cout << hitb << " ";
    }
  }
  cout << "---------------------------------------------------" << endl;
#endif

#ifdef OLDREL
  // copy MTRA relations from RSEG
  for (vector<ArtePointer<RSEG> >::const_iterator sIt=rsegVec.begin();
       sIt != rsegVec.end(); sIt++) {
    for ( ArteRelationIterator<MTRA> mtraIt = (*sIt)->mtra.begin();
	  mtraIt != (*sIt)->mtra.end(); ++mtraIt ) {
      ArtePointer<MTRA> mtra(*mtraIt);
      createRelation( rtra, mtra, &RTRA::mtra );
    }
  }
#else
  // determine MTRA once again
  int mtIndex;
  float fracmc;
  tfremcn_(rtra.index(),mtIndex,fracmc);
  if (mtIndex > 0 && fracmc > 0.7) {
    ArtePointer<MTRA> mtra(mtIndex);
    createRelation( rtra, mtra, &RTRA::mtra );
  }
    
#endif

#ifdef FULL
  cout << "The following MTRAs are related to rtra=" << rtra << endl;
#endif

  ArtePointer<RPNT> rpHighZ = 0;

  // copy RPNT relations from RSEG
  for (vector<ArtePointer<RSEG> >::const_iterator sIt=rsegVec.begin();
       sIt != rsegVec.end(); sIt++) {
    for ( ArteRelationIterator<RPNT> rpntIt = (*sIt)->rpnt.begin();
	  rpntIt != (*sIt)->rpnt.end(); ++rpntIt ) {
      createRelation( rtra, *rpntIt, &RTRA::rpnt );
      if ((*rpntIt)->zf == rsHighZ->ze) {
	rpHighZ = *rpntIt;
      }
    }
  }

  // copy RCCL relations from RSEG
  for (vector<ArtePointer<RSEG> >::const_iterator sIt=rsegVec.begin();
       sIt != rsegVec.end(); sIt++) {
    for ( ArteRelationIterator<RCCL> rcclIt = (*sIt)->rccl.begin();
	  rcclIt != (*sIt)->rccl.end(); ++rcclIt ) {
      // check if rccl was already related
      ArteRelationIterator<RCCL> rI = find( rtra->rccl.begin(),
					    rtra->rccl.end(),*rcclIt);
      if (rI == rtra->rccl.end()) {
#ifdef FULL
	cout << "copy relation rccl=" << *rcclIt
	     << " from rs=" << *sIt << " to rtra=" << rtra << endl;
#endif
	createRelation( rtra, *rcclIt, &RTRA::rccl );
      }
      else {
#ifdef FULL
	cout << "was already related: rccl=" << *rcclIt
	     << " to " << rtra << endl;
#endif
      }
    }
  }

// set relations to RSEG

  for (vector<ArtePointer<RSEG> >::reverse_iterator sIt=rsegVec.rbegin();
       sIt != rsegVec.rend(); sIt++) {
    createRelation( rtra, *sIt, &RTRA::rseg );
  }

  if (rpHighZ == 0) {
    mmbook_(ArtePointer<RPNT>::getLink().getRef(),1);
    rpHighZ = ArtePointer<RPNT>( ArtePointer<RPNT>::nrow() );
    rpHighZ->zf = rsHighZ->ze;
    rpHighZ->xf = rsHighZ->xe;
    rpHighZ->yf = rsHighZ->ye;
    rpHighZ->txf = rsHighZ->txe;
    rpHighZ->tyf = rsHighZ->tye;
    rpHighZ->pf = rsHighZ->pe;
    for (int i=0; i<15; i++) {
      rpHighZ->cvf[i] = rsHighZ->cve[i];
    }
    createRelation( rtra, rpHighZ, &RTRA::rpnt );
  }

#ifdef PULLS
  ArtePointer<RTRX> rtrx = ArtePointer<RTRX> (rtra);
  for ( ArteRelationIterator<RTRX_MTRA> mtraIt( rtrx->mtra );
	mtraIt.valid(); mtraIt.next() ) {
    ArtePointer<MTRA> mtra(mtraIt);
#ifdef FULL
    cout << mtra << " " << endl;
#endif
    int it = rtra.index();
    int imc = mtra.index();
    float pa[4];
    float pamc[4];
    float dpa[4];
    float dpanorm[4];
    float ch2mc;
    int stat;
    tftpar2_(it,imc,pa[0],pamc[0],dpa[0],dpanorm[0],ch2mc,stat);
  
    cout << endl << endl;
    cout << "---------------------------------------------------" << endl;
    printf("%4s %12s %12s %12s %12s \n","i","pRC","pMC","delta","pull");
    cout << "---------------------------------------------------" << endl;
    for (int i=0; i<=3; i++) {
      printf("%4d %12f4  %12f4 %12f4 %12f4 \n",
	     i,pa[i],pamc[i],dpa[i],dpanorm[i]);
    }
    cout << "---------------------------------------------------" << endl;
    cout << "  ch2mc=" << ch2mc << " stat=" << stat << endl;
    cout << "---------------------------------------------------" << endl;
  }
#endif

  return rtra;

}
