#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "marple/marple.hh"
#include "reco/Rc99Matches.hh"


void rc99SelVdsRichEcal(ArtePointer<RSEG> rVDS, 
			ArtePointer<RSEG> rRICH, 
			ArtePointer<RCCL> rccl);

float rc99Bremsstrahlung(ArtePointer<RTRA> rtra);

//============================================================
int Rc99MatchComb::makeRTRA()
{
  static float zTarget = -4.7; // set a z position for Tracks

  //=======================================================
  // VDS RICH ECAL 
  //=======================================================
  if( cluster != 0 && svdSegment != 0 && richSegment != 0 )
    {
      cout << " all found! \n";
      // call Thorsten function with refit
      rc99SelVdsRichEcal(svdSegment, richSegment, cluster );

      return 0;
  //=======================================================
  // something one is missed
  //=======================================================
    }else{
      RTRA rtra;

      // empty variabales
      rtra.hou = 0;
      rtra.hri = 0;
      rtra.htr = 0;
      rtra.hmu = 0;

      rtra.flr = 0;
      rtra.flt = 0;
      rtra.fle = 0;

      rtra.lmmu = 0;
      rtra.lmh = 0;
      rtra.flm = 0;

      // --Track parameters--------------------------------
      if( svdSegment != 0 )  
	{
	  rtra.xf  = svdSegment->xf;
	  rtra.yf  = svdSegment->yf;
	  rtra.zf  = svdSegment->zf;
	  rtra.txf = svdSegment->txf;
	  rtra.tyf = svdSegment->tyf;
	  
	  for (int i=0; i<=3; i++){for (int j=i; j<=3; j++)
	    rtra.cvf[c_matrix_pack[i][j]] = 
	      svdSegment->cvf[c_matrix_pack[i][j]];};
	  
	  // put momentum information
	  if( SvdEcal != 0 ) // SVD ECAL match 
	    {
	      rtra.chi = SvdEcal->chi;
	      rtra.pf  = SvdEcal->pf;
	      for (int i=0; i<5; i++){
		rtra.cvf[c_matrix_pack[i][4]] = 
		  SvdEcal->cvf[c_matrix_pack[i][4]];};

	    }else if( SvdRich != 0 ){ // SVD RICH match
	      rtra.chi = SvdRich->chi;
	      rtra.pf  = SvdRich->pf;
	      for (int i=0; i<5; i++){
		rtra.cvf[c_matrix_pack[i][4]] = 
		  SvdRich->cvf[c_matrix_pack[i][4]];};
	    }

	}else if( RichEcal != 0 ){  // RICH ECAL match
	  rtra.chi = RichEcal->chi;

	  RcSegPoint point(RichEcal,0);
	 
	  int iErr = 0;
	  point.prolongate(zTarget, iErr);
	  if( iErr != 0 ) 
	    {
	      cout << " Could not Prolongate track " << ArteTable<RTRA>::size();
	      return 1;
	    }

	  rtra.xf  = point.x();
	  rtra.yf  = point.y();
	  rtra.zf  = point.z();
	  rtra.txf = point.tx();
	  rtra.tyf = point.ty();
	  rtra.pf  = point.pe();
	  
	  for (int i=0; i<5; i++){for (int j=i; j<5; j++)
	    rtra.cvf[c_matrix_pack[i][j]] = point.cov()(i+1,j+1); }
	}

      // --- some addition parameters ---------

      if( svdSegment != 0 )
	{
	  rtra.hsi = int(svdSegment->time+svdSegment->dtim);
	  rtra.hin = int(svdSegment->time);
	  rtra.lte = svdSegment->chi;
	}else{
	  rtra.hsi = 0;
	  rtra.hin = 0; 
	  rtra.lte = 0;
	}
      if( cluster != 0 )
	{	  
	  rtra.hca = int(cluster->wid);
	  rtra.lee = cluster->esum;
	  rtra.lemi= cluster->x;
	  rtra.leh = cluster->y;
	  if( SvdEcal != 0 )      rtra.lth = SvdEcal->tye;
	  else if ( RichEcal != 0 ) rtra.lth = RichEcal->tye;
	}else{
	  rtra.hca = 0;
	  rtra.lee = 0;
	  rtra.lemi= 0;
	  rtra.leh = 0;
	  rtra.lth = 0;
	}
      if( richSegment != 0 )
	{
	  rtra.lre = richSegment->time;
	  rtra.lrmu= richSegment->ze;
	  rtra.lrpi= richSegment->ye;
	  rtra.lrk = richSegment->chi;
	  rtra.lrp = SvdRich->tye;
	}else{
	  rtra.lre = 0;
	  rtra.lrmu= 0;
	  rtra.lrpi= 0;
	  rtra.lrk = 0;
	  rtra.lrp = 0;
	}
      
      ArteTable<RTRA>::push_back( rtra );
      
      ArtePointer<RTRA> Prtra;
      Prtra.setInt( ArteTable<RTRA>::size() );

      if( richSegment != 0 )
	createRelation( Prtra, richSegment, &RTRA::rseg );
      if( svdSegment != 0  )
	createRelation( Prtra, svdSegment,  &RTRA::rseg );
      if( cluster  != 0    )
	createRelation( Prtra, cluster,  &RTRA::rccl );
      
      if( SvdRich  != 0 ){
	Prtra->fit = 30;
	createRelation( Prtra, SvdRich,  &RTRA::rseg );
      }
      if( SvdEcal  != 0 ){
	Prtra->fit = 20;
	createRelation( Prtra, SvdEcal,  &RTRA::rseg );
      }
      if( RichEcal != 0 ){
	Prtra->fit = 100;
	createRelation( Prtra, RichEcal,  &RTRA::rseg );
      }

      if (ArteTable<MTRA>::size() > 0 ) 
	{
	  int found = 0;
	  ArtePointer<MTRA> mtra;
	  if( RichEcal != 0 )
	    {
	      mtra.setInt( richSegment->fit);
	      if ( richSegment->fit != 0)
		for (ArteRelationIterator<MTRA> mccl = 
		       cluster->mtra.begin();
		     !found && mccl != cluster->mtra.end(); mccl++)
		  if (mtra == (*mccl)) found = 1;	    
	    }else if ( SvdEcal != 0 ) {
	      mtra = svdSegment->mtra.begin();
	      if( mtra != svdSegment->mtra.end() )
		for (ArteRelationIterator<MTRA> mccl = cluster->mtra.begin();
		     !found && mccl != cluster->mtra.end(); mccl++)
		  if (mtra == (*mccl)) found = 1;
	    }else if ( SvdRich != 0 ){
	      if( richSegment->fit != 0 && 
		  *richSegment->mtra.begin() == *svdSegment->mtra.begin() )
		{
		  found = 1;
		  mtra = richSegment->mtra.begin();
		}
	    }

	  if( found) createRelation( Prtra, mtra, &RTRA::mtra );
	  Prtra->lmmu = rc99Bremsstrahlung(Prtra);
	  
	    
         };

    }  




  return 0;
}

ostream & operator << (ostream & os, const Rc99MatchComb &rp)
{
  os << "Match = {" << ends;
  if( rp.cluster  != 0   ) os << " ecal_" << rp.cluster << ends;
  if( rp.svdSegment != 0 ) os << " svd_" << rp.svdSegment << ends;
  if( rp.richSegment != 0) os << " rich_" << rp.richSegment << ends;
  os << " } ," << endl ;
  os << "    - constrained from matches RSEG "  << ends;
  if( rp.SvdEcal != 0 ) os << " SvdEcal_" << rp.SvdEcal   << ends;
  if( rp.SvdRich != 0 ) os << " SvdRich_" << rp.SvdRich   << ends;
  if( rp.RichEcal!= 0 ) os << " RichEcal_" << rp.RichEcal << ends;
  os << endl;
  return os;
}


//============================================================
// Rc99MatchVector functions
//============================================================
int Rc99MatchVector::addSvdEcal(  ArtePointer<RSEG> svdEcal)
{
  ArtePointer<RSEG> svd = svdEcal->rend.begin();
  if( !(svd->cmp & Rsegc::vxd ) )
    {
      cout << "SvdEcal Matched RSEG "<< svdEcal 
	   << " has nothing to do with SVD \n";
      return 1;
    }

  ArtePointer<RCCL> ecal = svdEcal->rccl.begin();
  if( ecal == 0  )
    {
      cout << "SvdEcal Matched RSEG "<< svdEcal 
	   << " has nothing to do with ECAL \n";
      return 1;
    }

  unsigned int ipoint = 0;
  unsigned int size = points.size();
  int found = 0;
  vector<Rc99MatchComb *> repeatComb; 
  for( ;  ipoint < size; ++ipoint )
    if(points[ipoint].cluster == ecal )
      {
	found = 1;
	Rc99MatchComb * ptPoint = &(points[ipoint]);
	if(points[ipoint].svdSegment != 0 && points[ipoint].svdSegment != svd)
	  {	
	    int again = 0;
	    for( unsigned int iRepeat = 0; 
		 !again && iRepeat < repeatComb.size(); iRepeat++ )
	      if( repeatComb[iRepeat]->richSegment == ptPoint->richSegment )
		again = 1;
	    if( !again )  repeatComb.push_back( ptPoint);
	  }else if(points[ipoint].svdSegment == 0 ){
	    ptPoint->svdSegment = svd;	    
	    ptPoint->SvdEcal = svdEcal;	    
	  }
      }
  if( !found )
    {
      points.push_back(Rc99MatchComb());
      Rc99MatchComb *   ptPoint = &(points.back());
      ptPoint->cluster = ecal;
      ptPoint->svdSegment = svd;
      ptPoint->SvdEcal = svdEcal;	  
      ptPoint = 0;
    }else{
      for( unsigned int iRepeat = 0; 
	   iRepeat < repeatComb.size(); iRepeat++ )
	{
	  points.push_back(*(repeatComb[iRepeat]));
	  points.back().svdSegment = svd;
	  points.back().SvdEcal = svdEcal;	  
	}
      
    }
  return 0;
}

int Rc99MatchVector::addSvdRich( ArtePointer<RSEG> svdRich )
{
  ArtePointer<RSEG> svd = 0;
  ArtePointer<RSEG> rich = 0;
  int ncandidates = 0;
  for( ArteRelationIterator<RSEG> link = svdRich->rend.begin();
	 link != svdRich->rend.end(); ++link )
      {
	ncandidates++;
	if( (*link)->cmp & Rsegc::vxd ) svd = link;
	else if( (*link)->cmp & Rsegc::rich ) rich = link;
      }
  if( ncandidates != 2 || svd == 0 || rich == 0 )
    {
      cout << " something fishy in SvdRich RSEG " 
	   << svdRich << ", skip it\n";
      return 1;
    }
  unsigned int ipoint = 0;
  unsigned int size = points.size();
  int found = 0;
  vector<Rc99MatchComb *> repeatComb; 
  for( ;  ipoint < size; ++ipoint )
    if(points[ipoint].svdSegment == svd )
      {
	found = 1;
	Rc99MatchComb * ptPoint = &(points[ipoint]);
	if(points[ipoint].richSegment != 0 && 
	   points[ipoint].richSegment != rich)
	  {	
	    int again = 0;
	    for( unsigned int iRepeat = 0; 
		 !again && iRepeat < repeatComb.size(); iRepeat++ )
	      if( repeatComb[iRepeat]->cluster == ptPoint->cluster )
		again = 1;
	    if( !again )  repeatComb.push_back( ptPoint);
	  }else if(points[ipoint].richSegment == 0 ){
	    ptPoint->richSegment = rich;	    
	    ptPoint->SvdRich = svdRich;	    
	  }
      }
  if( !found )
    {
      points.push_back(Rc99MatchComb());
      Rc99MatchComb *   ptPoint = &(points.back());
      ptPoint->richSegment = rich;
      ptPoint->svdSegment = svd;
      ptPoint->SvdRich = svdRich;	  
      ptPoint = 0;
    }else{
      for( unsigned int iRepeat = 0; 
	   iRepeat < repeatComb.size(); iRepeat++ )
	{
	  points.push_back(*(repeatComb[iRepeat]));
	  points.back().richSegment = rich;
	  points.back().SvdRich = svdRich;	  
	}
      
    }
  return 0;
}


int Rc99MatchVector::addRichEcal(  ArtePointer<RSEG> richEcal )
{
  ArtePointer<RSEG> rich = richEcal->rend.begin();
  if( !(rich->cmp & Rsegc::rich ) )
    {
      cout << "RichEcal Matched RSEG "<< richEcal 
	   << " has nothing to do with RICH \n";
      return 1;
    }

  ArtePointer<RCCL> ecal = richEcal->rccl.begin();
  if( ecal == 0 )
    {
      cout << "RichEcal Matched RSEG "<< richEcal 
	   << " has nothing to do with ECAL \n";
      return 1;
    }
  unsigned int ipoint = 0;
  unsigned int size = points.size();
  int found = 0;
  vector<Rc99MatchComb *> repeatComb; 
  for( ;  ipoint < size; ++ipoint )
    if(points[ipoint].cluster == ecal )
      {
	found = 1;
	Rc99MatchComb * ptPoint = &(points[ipoint]);
	if(points[ipoint].richSegment != 0 && 
	   points[ipoint].richSegment != rich)
	  {	
	    int again = 0;
	    for( unsigned int iRepeat = 0; 
		 !again && iRepeat < repeatComb.size(); iRepeat++ )
	      if( repeatComb[iRepeat]->svdSegment == ptPoint->svdSegment )
		again = 1;
	    if( !again )  repeatComb.push_back( ptPoint);
	  }else if(points[ipoint].richSegment == 0 ){
	    ptPoint->richSegment = rich;	    
	    ptPoint->RichEcal = richEcal;	  
	  }
      }
  if( !found )
    {
      points.push_back(Rc99MatchComb());
      Rc99MatchComb *   ptPoint = &(points.back());
      ptPoint->richSegment = rich;
      ptPoint->cluster = ecal;
      ptPoint->RichEcal = richEcal;	  
      ptPoint = 0;
    }else{
      for( unsigned int iRepeat = 0; 
	   iRepeat < repeatComb.size(); iRepeat++ )
	{
	  points.push_back(*(repeatComb[iRepeat]));
	  points.back().richSegment = rich;
	  points.back().RichEcal = richEcal;	  
	}
      
    }
  return 0;
}
