// 
//  Author:  Rainer Mankel                     Created:  20-Feb-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// function rcelid
// perform a rudimentary (!) electron identification
// method should be revised after closer investigation
// likelihoods of standalone (cluster-based) particle id are
//      not taken into account in the rtra likelihoods

// callable from Fortran

#include <iostream.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

extern "C" {
  void rcelid_(){

    static ArtePointer<GESL> ecSl;

    if (! ecSl.index()) {         // locate the ECAL
      for (ArteTable<GESL>::iterator gsIt=ArteTable<GESL>::begin();
	   gsIt != ArteTable<GESL>::end(); ++gsIt) {
	if (! strncmp((*gsIt)->name,"CALW",4)) {
	  ecSl = *gsIt;
	  break;
	}
      }
      if (! ecSl.index()) {
	arteMsg << "rcelid: CALW superlayer not found" << endl;
	arteMsg.warn();
	return;
      }	
      //cout << "ecSL=" << ecSl << endl;
    }
	  
    for (ArteTable<RTRA>::iterator rtIt=ArteTable<RTRA>::begin();
	 rtIt != ArteTable<RTRA>::end(); ++rtIt) {

      // initialize
      (*rtIt)->fle = 0;
      (*rtIt)->lee = 0;
      (*rtIt)->leh = 0;
      (*rtIt)->lemi = 0;

      // get last RPNT
      ArtePointer<RPNT> rp;
      for ( ArteRelationIterator<RPNT> rpntIt = (*rtIt)->rpnt.begin();
	    rpntIt != (*rtIt)->rpnt.end(); ++rpntIt ) {
	if (! rp.index()) {
	  rp = *rpntIt;
	}
	else {
	  if (rp->zf < (*rpntIt)->zf)  rp = *rpntIt;
	}
      }
      if (! rp.index()) continue;
      if (rp->zf < 750) continue;

      // extrapolate

      float xExt = rp->xf + rp->txf * (ecSl->z1 - rp->zf);
      float yExt = rp->yf + rp->tyf * (ecSl->z1 - rp->zf);

      if (xExt < ecSl->x1) continue;  // outer acceptance
      if (xExt > ecSl->x2) continue;
      if (yExt < ecSl->y1) continue;
      if (yExt > ecSl->y2) continue;

      if ((fabs(xExt) < 11.) && (fabs(yExt) < 11.)) continue; 
                                      // inner acceptance
      
      if (xExt > 34 && xExt < 56 && 
	  yExt > -89 && yExt < -67) continue; // electron beam pipe

      (*rtIt)->fle = 1;

      for ( ArteRelationIterator<RCCL> rcclIt = (*rtIt)->rccl.begin();
	    rcclIt != (*rtIt)->rccl.end(); ++rcclIt ) {
	
	double eOverP = (*rcclIt)->esum * fabs( (rp->pf) );

	double mE = 1;
	//double sE = 0.04;
	double sE = 0.2;
	double lhE = exp(-0.5 * (eOverP - mE)*(eOverP - mE) / (sE * sE))
	  / (2.5066 * sE);

	double mH = 0.5;
	double sH = 0.3;
	double lhH = exp(-0.5 * (eOverP - mH)*(eOverP - mH) / (sH * sH))
	  / (2.5066 * sH);

	double lhO = 0.1;

	double lhTot = lhE + lhH + lhO;

	(*rtIt)->lee = lhE / lhTot;
	(*rtIt)->leh = lhH / lhTot;
	(*rtIt)->lemi = 0;              // mip lHood to be defined later
#ifdef FULL
	cout << "rcelid: set lh for rt=" << *rtIt 
	     << " eOverP =" << eOverP 
	     << " lhE=" << lhE
	     << " lhH=" << lhH
	     << " lhO=" << lhO 
	     << " lhTot=" << lhTot << endl;
	cout << " ... rccl=" << *rcclIt << endl;
#endif
      }
    }
    for ( ArteTable<RTRA>::iterator rtI=ArteTable<RTRA>::begin();
	  rtI != ArteTable<RTRA>::end(); ++rtI) {
      char cc[13];
      float pmc = 0;
      for ( ArteRelationIterator<MTRA> mtIt=((*rtI)->mtra).begin();
	    mtIt != ((*rtI)->mtra).end(); ++mtIt) {
	strncpy(cc,(*mtIt)->topp->name,12);
	cc[12] = 0;
	pmc = sqrt( (*mtIt)->px * (*mtIt)->px 
		    + (*mtIt)->py * (*mtIt)->py
		    + (*mtIt)->pz * (*mtIt)->pz
		    );
      } 

#ifdef FULL
      cout << "rcelid: rt=" << *rtI
	   << " " << cc << " p=" << pmc
	   << " lee=" << (*rtI)->lee
	   << " leh=" << (*rtI)->leh
	   << " lemi=" << (*rtI)->lemi << endl;
#endif
    }
  }
}

