#include "rich/RICHDAQ.h"
#include "rich/PhotonDetection.h"
#include "rich/ArteTablesInit.h"
#include "rich/RICHDB.h"

extern int RICHdebug;

// --------------------------------------------------------------------

RICHDAQ *RICH_DAQ=0;

// --------------------------------------------------------------------
M16Base::M16Base(char* path) {
  DAQ_INITIALIZATION* Init=0;
  Init= new DAQ_INITIALIZATION(path);
  Init->sours_selector("m16");
  int pmi,pmc,asd,asdchan;
  numchan=0;
  char s[256];
  char sasd[5];
  while (Init->m16_scan(s)) { 
    //printf("1: %d  %s\n",numchan,s);
    if (sscanf(s,"%d %d %s %d",&pmi,&pmc,sasd,&asdchan)==4) {
      sasd[0]=sasd[3];
      sasd[1]='\0';
      asd=atoi(sasd);
      pm[index(asd,asdchan)]=pmi;
      pmChannel[index(asd,asdchan)]=pmc;
      asdID     [pmi*16+pmc]=asd;
      asdChannel[pmi*16+pmc]=asdchan;
      numchan++;
    }
  }
  if (numchan!=64) {
    fprintf(stderr,"M16Base: missing channels: check file m16wiremap\n");
    Exit(1);
  }
  delete Init;
}
  
  
// --------------------------------------------------------------------
M4Base::M4Base(char* path) {
  DAQ_INITIALIZATION* Init=0;
  Init= new DAQ_INITIALIZATION(path);
  Init->sours_selector("m4");
  int pmi,pmc,asdchan;
  numchan=0;
  char s[256];
  while (Init->m4_scan(s)) {
    if (sscanf(s,"%d %d %d",&pmi,&pmc,&asdchan)==3) {
      //printf("%d  %s\n",numchan,s);
      pm[index(asdchan)]=pmi;
      pmChannel[index(asdchan)]=pmc;
      asdChannel[pmi*4+pmc]=asdchan;
      numchan++;
    }
  }
  if (numchan!=16) {
    fprintf(stderr,"M4Base: missing channels: check file m4wiremap\n");
    Exit(1);
  }
  delete Init;
}
  

// -------------------------------------------------------------------------

RICHDAQ::RICHDAQ(char *path) {
 
  if (!path) {
    if (!ArteInitFromDB) {
      ArteInitFromDB=new ARTE_INIT_FROM_DB();
      ArteInitFromDB->ARTETablesInitialization();
    }
    delete ArteInitFromDB;
  }


  if (!RICH_Detector)  RICH_Detector =new RICHDetector(path);
  if (!RICH_Vessel)    RICH_Vessel   =new RICHVessel(path);
  if (!RICH_LambdaPhi) RICH_LambdaPhi=new LambdaPhi();

  RICHDetector& det=*RICH_Detector;

  nhit=0;
  numfed=0;
  numchannel=RICH_Detector->NumChannel();
  allcha =new RICHChannel[numchannel];
  hitlist=new unsigned short[numchannel];
  numhit =new unsigned long [numchannel];
  if (!numhit) { // is enough to check, since it is last and largest array
    fprintf(stderr,"RICHDAQ: Out of memory: Memory allocation error.\n");
    Exit(1);
  }
  ZeroNumHit();
  memset(allcha,0xFF,sizeof(RICHChannel)*numchannel);

  int sm,r,c,p,chan_id; 
  short chan_stat;
  DAQ_INITIALIZATION* init_status=0;
  init_status=new DAQ_INITIALIZATION(path);
  init_status->sours_selector("status");
  for (det.NewChannelLoop();det.ChannelLoop(&sm,&r,&c,&p);) {
    RICHChannel& ch=allcha[det.ChannelIndex()];
    ch.sm() =sm;
    ch.row()=r; 
    ch.col()=c;
    ch.pm_channel()=p;
    ch.pm_numchannel()=det.pm().numchannel();
  }
  while (init_status->channelstatus_scan(&chan_id,&chan_stat)) {
    if (det.ChannelExist(chan_id)) {
      allcha[det.ChannelIndex()].status()=chan_stat;
    }
  }
  delete init_status;

  m4bas  = new M4Base(path);
  m16bas = new M16Base(path);

  DAQ_INITIALIZATION* init=new DAQ_INITIALIZATION(path);
  init->sours_selector("asd");

  int *fid=0;
  numfed=0;
  int *cnt=new int[NumChannel()];
  memset(cnt,0,sizeof(int)*NumChannel());
  int lupfedid,d,con;
  char asd[5], tag[256];
  asd[0]='\0'; tag[0]='\0';
  while (init->asd_scan(tag,asd,&sm,&r,&c,&lupfedid,&d,&con)) {
    asd[0]=asd[3];
    asd[1]='\0';
    int asdid=atoi(asd);
    r*=2;
    c*=2;   
    if (!strcmp(tag,"M16")) {    
      int asdchannel;
      for (asdchannel=0;asdchannel<NumChannelPerASD();asdchannel++) {
	int pm =m16bas->pmt(asdid,asdchannel);
	int p  =m16bas->pmtChannel(asdid,asdchannel);
	if (sm>6) { // on lower detector baseboard are rotated by 180 degree
	  pm=3-pm; 
	  p=15-p;
	}
	int rr=r+pm/2;
	int cc=c+pm%2;
	
	if (!det.ChannelExist(sm,rr,cc,p)) {
	  fprintf(stderr,"RICHDAQ(char*) initialisation error: \n");
	  fprintf(stderr,"  ASD.dat requires channel without pmt: %2d %2d %2d %2d\n",
		  sm,rr,cc,p);
	  //fprintf(stderr,"  last line: %s\n",s);
	  Exit(1);
	}
	
	if (++cnt[det.ChannelIndex()]>1) {
	  printf("Channel double assigned: %2d %2d %2d %2d asd: %d %2d  ncount: %d\n",
		 sm,rr,cc,p,asdid,asdchannel,cnt[det.ChannelIndex()]);
	}
	
	allcha[det.ChannelIndex()].fedid()=lupfedid;
	
	int i=(asdchannel&0x8)|(7-(asdchannel&0x7)); // hardware map on asd card
	
	allcha[det.ChannelIndex()].bit()
	  =(d*NumConnectorPerDaughter()+con)*NumChannelPerConnector()+i;
      }
    }
    else if (!strcmp(tag,"M4")) {
      int asdchannel;
      for (asdchannel=0;asdchannel<NumChannelPerASD();asdchannel++) {
	int pm =m4bas->pmt(asdchannel);
	int p  =m4bas->pmtChannel(asdchannel);
	if (sm>6) { // on lower detector baseboard are rotated by 180 degree
	  pm=3-pm; 
	  p=3-p;
	}
	
	int rr=r+pm/2;
	int cc=c+pm%2;
	
	if (!det.ChannelExist(sm,rr,cc,p)) {
	  fprintf(stderr,"RICHDAQ(char*) initialisation error: \n");
	  fprintf(stderr,"  ASD.dat requires channel without pmt: %2d %2d %2d %2d\n",
		  sm,rr,cc,p);
	  //fprintf(stderr,"  last line: %s\n",s);
	  Exit(1);
	}
	
	if (++cnt[det.ChannelIndex()]>1) {
	  printf("Channel double assigned: %2d %2d %2d %2d asd: %d %2d  ncount: %d\n",
		 sm,rr,cc,p,asdid,asdchannel,cnt[det.ChannelIndex()]);
	}
	allcha[det.ChannelIndex()].fedid()=lupfedid;
	
	int i=(asdchannel&0x8)|(7-(asdchannel&0x7)); // hardware map on asd card
	
	allcha[det.ChannelIndex()].bit()
	  =(d*NumConnectorPerDaughter()+con)*NumChannelPerConnector()+i;
      }
    } 
    else {
      fprintf(stderr,"RICHDAQ: Unknown Base type: %s\n",tag);
      Exit(1);
    }

    // see if new fed is involved
    int i;
    for (i=0;i<numfed;i++) 
      if (fid[i]==lupfedid) break;
    if (i==numfed) {
      numfed++;
      fid=(int*)realloc(fid,sizeof(int)*numfed);
      fid[numfed-1]=lupfedid;
    }
  }
  free(fid);
  delete init;

  int i;
  for (i=0;i<NumChannel();i++) {
    if (allcha[i].bit()==(short)0xFFFF) {
      allcha[i].fedid()=numfed;
      allcha[i].bit()=0;
      allcha[i].status()=0;
      printf("Channel not assigned: %2d %2d %2d %2d\n",
	     allcha[i].sm(),allcha[i].row(),allcha[i].col(),allcha[i].pm_channel());
      fflush(stdout);
    }
  }
  fedbitindex= new unsigned short[numfed*NumChannelPerFED()+1];
  memset(fedbitindex,0xFF,sizeof(unsigned short)*(numfed*NumChannelPerFED()));
  
  for (i=0;i<NumChannel();i++) {
    if (!ChannelExist(i)) continue;
    fedbitindex[fedid()*NumChannelPerFED()+bit()]=i;
    }
  
  int n=0;
  for (i=0;i<numfed*NumChannelPerFED();i++) {
    if (fedbitindex[i]!=(unsigned short)0xFFFF) n++;
  }
  if (n!=NumChannel()) {
    fprintf(stderr,"RICHDAQ: error initializing fedbitindex\n");
    Exit(1);
  }
  
  etb_fed=new ETB_fed[numfed];
  
  for (i=0;i<256;i++) {
    pattern[i].bit[7]=(i&0x01);
    pattern[i].bit[6]=(i&0x02);
    pattern[i].bit[5]=(i&0x04);
    pattern[i].bit[4]=(i&0x08);
    pattern[i].bit[3]=(i&0x10);
    pattern[i].bit[2]=(i&0x20);
    pattern[i].bit[1]=(i&0x40);
    pattern[i].bit[0]=(i&0x80);
  }
}


// -------------------------------------------------------------------------


int RICHDAQ::NextEvent() {

  // -- Check if MC event
  if (!ArteTable<MTRA>::nrow()) {
    fprintf(stderr," $$$RICH: RICHDAQ::NextEvent(): Function can be used in Monte Carlo only!\n");
    return 1;
  }

  static int old_ev_no=-1;
  int ev_no=ArtePointer<EVHD>(1)->evt;
  if (old_ev_no==ev_no) return 2;
  old_ev_no=ev_no;

  // -- Fill RICHDAQ event variables
  numevent++;
  nhit=0;
  RICHDetector &rd=*RICH_Detector;
  for (ArteTable<DIGB>::iterator idigb=RichRun::gede->dig1;
       idigb!=RichRun::gede->digl; ++idigb) {
    int id=(*idigb)->id;
    rd.ChannelExist(id);
    int ci=rd.ChannelIndex();
    numhit[ci]++;
    hitlist[nhit++]=ci;
  }

  for (int i=0;i<numfed; i++) {
    etb_fed[i].data=new unsigned char[NumChannelPerFED()/8];
    memset(etb_fed[i].data,0x0,NumChannelPerFED()/8);
  }


  for (int i=0; i<NumHitChannel(); i++) {
    HitChannelExist(i);
    int fed_id   = fedid();
    int fed_bit  = bit();
    int fed_byte = fed_bit/8;
    int fed_bit_in_byte = fed_bit-fed_byte*8;

    //cout<<i<<" fedid="<<fed_id<<" bit="<<fed_bit<<" fed_byte="<<fed_byte<<" fed_bit_in_byte="<<fed_bit_in_byte<<endl;
    //cout<<" byte before="<<hex<<int(*(unsigned char*)(etb_fed[fed_id].data+fed_byte))<<dec<<endl;
    unsigned char *byte=(unsigned char*)((unsigned char*)etb_fed[fed_id].data+fed_byte);
    *byte=*byte|(128>>fed_bit_in_byte);
    //cout<<" byte after="<<hex<<int(*(unsigned char*)(etb_fed[fed_id].data+fed_byte))<<dec<<endl;
  }

  return 0;
}



// -------------------------------------------------------------------------
int RICHDAQ::NextEventL4(FILE *fp) {
  int ret=0;
#ifndef NO_ARTE
  static int* evtmem=0;
  static EVENT_HDR* event_ptr;
  extern int  SEG_LEN;
  
  if (!evtmem) {
    if (feof(fp)) return 0;
    evtmem=new int[SEG_LEN];
    event_ptr=(EVENT_HDR*) evtmem;
    ZeroNumHit();
  }

  int bc = 0;
  LogHdr hdr;
  l4aGetEvt( fp, event_ptr, SEG_LEN, &bc, &hdr);

  if (feof(fp)||!(ret=NextEvent(event_ptr))) {
    free(evtmem);
    evtmem=0;
  }
#endif
  return ret;
}


// -------------------------------------------------------------------------
int RICHDAQ::NextEvent(EVENT_HDR* event_ptr) {
  event=event_ptr;
  fltevt=event->sltevt;
  numevent++;
  ETB_get_feds(event, COMP_ID_RICH, numfed, etb_fed, &numfedevt);
  
  register int j1=NumChannelPerFED()/8;
  
  nhit=0;
  for (int i=0;i<numfedevt;i++)
    {
      
      if (etb_fed[i].hdr.fedid!=i)
	{
	  static int errorcount=0;
	  errorcount++;
	  if (errorcount<5)
	    {
	      cerr << "RICH:  Error with FED " << i << "\n";
	    }
	  else if (errorcount == 5)
	    {
	      cerr << "RICH:  Error with FED " << i << "\n";
	      cerr << "RICH:  Error message will be suppressed now.\n";
	    }
	  continue;
	}
      register int  f0=etb_fed[i].hdr.fedid*NumChannelPerFED();
      register unsigned char *data=(unsigned char*)etb_fed[i].data;
      register int bit=0;
      for (register int j=0;j<j1;j++)
	{
	  if (*data)
	    {
	      register unsigned char* h=&pattern[*data].bit[0];
	      //cout<<"RICHDAQ: data="<<hex<<int(*data)<<dec<<" "<<hex<<int(*h)<<dec<<endl;
	      for (register int k=0;k<8;k++)
		{
		  if (*h)
		    {
		      register unsigned short ci=fedbitindex[f0+bit];
		      if ((ci!=(unsigned short)0xFFFF))
			{
			  numhit[ci]++;
			  hitlist[nhit++]=ci;
			}
		    }
		  bit++;
		  h++;
		}
	    }
	  else bit+=8;
	  data++;
	}
    }
  return 1;
}
 
 
// -------------------------------------------------------------------------
int RICHDAQ::NextEvent(FILE *fp) {
  int ret=0;
  static int* evtmem=0;
  static EVENT_HDR* event_ptr;
  
  if (!evtmem) {
    if (feof(fp)) return 0;
    evtmem=new int[0x4000];
    event_ptr=(EVENT_HDR*) evtmem;
    ZeroNumHit();
  }
  ETB_read_event(fp,event_ptr,0x4000);
  if (feof(fp)||!(ret=NextEvent(event_ptr))) {
    free(evtmem);
    evtmem=0;
  }
  return ret;
}


// -------------------------------------------------------------------------
void RICHDAQ::printevent(FILE *fp) {

  fprintf(fp,"\nEvent Header:\n");
  fprintf(fp,"  Length ...........: %08X  %6d\n",event->length,event->length);
  fprintf(fp,"  Record ID ........: %04X\n",event->recid);
  fprintf(fp,"  reserved .........: %04X\n",event->format);
  fprintf(fp,"  SLT Event No .....: %08X\n",event->sltevt);
  fprintf(fp,"  Component Mask ...: %08X\n",event->compmask);
  fprintf(fp,"  Event Mask .......: %08X\n",event->evtmask);
  fflush(fp);
  int i;
  for (i=0;i<numfedevt;i++) {
    SLB_SUBHDR* slb_header=&etb_fed[i].hdr;
    char* data=(char*)etb_fed[i].data;
    if (slb_header==0) {
      fprintf(fp,"\n No data in this event for fed %08X.\n",i);
      return;
      }
    fprintf(fp,"\n");
    fprintf(fp,"    Length ................: %04X %6d\n",
  	  slb_header->length,slb_header->length);
    fprintf(fp,"    MotherID (BoardID) ....: %04X\n",slb_header->fedid);
    fprintf(fp,"    RecordID ..............: %04X\n",slb_header->recid);
    fprintf(fp,"    BX-ID .................: %04X\n",slb_header->bxid);
    fflush(fp);
    int i;
    for (i=0;i<NumChannelPerFED()/8;i++) {
      fprintf(fp,"%02X",data[i]);
      if (i%32==31) fprintf(fp,"\n");
      else if (i%2==1) fprintf(fp," ");
      }
    fflush(fp);
    }
  }


// -------------------------------------------------------------------------

extern "C" {
  int ARTE_get_feds_DRIC( int max, ETB_fed *feds, int *n );
	   }

int RICHDAQ::fillHITBfromDRIC() {

#ifndef NO_ARTE
  static int noDric=0;
  if (!ArteTable<DRIC>::nrow()) {
     if (noDric<10) { 
       fprintf(stderr," $$$RICH: DRIC table not filled\n");
       if (noDric==9) fprintf(stderr," $$$RICH: the last message of this kind ...\n"); 
       noDric++;
     }
     return 0;
  }

  ArtePointer<HITB> hitb;
  ArtePointer<DIGB> digb;
  RichRun::gede->hit1=ArteTable<HITB>::end();
  RichRun::gede->dig1=ArteTable<DIGB>::end();
 
  RICHDetector &rd=*RICH_Detector;
  HITB hdat;
  DIGB hdig;
  numevent++;

  ARTE_get_feds_DRIC(numfed,etb_fed,&numfedevt); // similar to ETB_get_feds()

  register int l=NumChannelPerFED()/8;

  nhit=0;
  for( int i=0; i<numfedevt; i++ ) {

    if (etb_fed[i].hdr.fedid!=i)
      {
        static int errorcount=0;
        errorcount++;
        if (errorcount<5)
          {       
            cerr << "RICH:  Error with FED " << i << "\n";
          }
        else if (errorcount == 5)
          {
            cerr << "RICH:  Error with FED " << i << "\n";
            cerr << "RICH:  Error message will be suppressed now.\n";
          }
	continue;
      }


    register int  f0 = etb_fed[i].hdr.fedid*NumChannelPerFED();   
    register unsigned char *p = (unsigned char *)etb_fed[i].data;
    register int  bit= 0;
    for (int j=0; j<l; j++) {
      register unsigned char adc=*(p+j);
      if (adc) {
        //cout << "j=" << j << " 0x" << hex << int( adc ) << dec <<endl;
        register unsigned char* h=&pattern[adc].bit[0];
	//cout<<"RICHOnline: adc="<<hex<<int(adc)<<dec<<" "<<hex<<int(*h)<<dec<<endl;
        for (register int k=0;k<8;k++) {
          if (*h) {
            register unsigned short ci=fedbitindex[f0+bit];
	    if ((ci!=(unsigned short)0xFFFF)) {
	      ChannelExist(ci);
	      if (!bad()) {
		numhit[ci]++;
		hitlist[nhit++]=ci;
		rd.ChannelExist(sm(),row(),col(),pm_channel());
		Vec3 x=rd.pos();
		hdat.z=x.z()/10.;
		hdat.x=x.x()/10.;
		hdat.y=x.y()/10.;
		hdat.q=RICH_LambdaPhi->phi(ci);
		hdat.r=RICH_LambdaPhi->lambda(ci);
		hdat.gede=RichRun::gede.index();
		hdig.id=sm()*1000000+row()*10000+col()*100+pm_channel();
		digb=ArteTable<DIGB>::push_back(hdig);
		hitb=ArteTable<HITB>::push_back(hdat);
		createRelation(hitb,digb,&HITB::digb);
	      }
	    }
	  }
          bit++;
	  h++;
	}
      }
      else bit+=8;
    }
  }
  RichRun::gede->hitl=ArteTable<HITB>::end();
  RichRun::gede->digl=ArteTable<DIGB>::end();
#endif
  return 1;
}

// -------------------------------------------------------------------------
