#include "arte/arte.hh"
#include "rich/PhiLambda.h"
#include "rise/RICHFit.h"
#include "CLHEP/Matrix/Matrix.h"

RICHFit::~RICHFit() {delete histo;}

RICHFit::RICHFit(int nbin, float binmin, float binmax) {
  nx=nbin;
  xmi=binmin;
  xma=binmax;
  histo=new Histo(nx,xmi,xma);

  sigma=7e-4;
  n=0;
  b=0;
  s=0;
  s3s=0;
}


void RICHFit::fillRadiusHistogram(float p,float l,float r) {
  
  tPhi=p;
  tLam=l;
  tRad=r;
  float isigma=1/sigma;

  resetRadiusHistogram();
  for (ArteTable<HITB>::iterator ihitb=ArteTable<HITB>::begin();
       ihitb!=ArteTable<HITB>::end(); ++ihitb) {
    float dphi=tPhi-(*ihitb)->q;
    float dlam=tLam-(*ihitb)->r;
    float drad=(tRad-sqrt(dphi*dphi+dlam*dlam))*isigma;

    if (drad<xmi || drad>xma) continue;
    histo->fill(drad);
  }
}



void RICHFit::estimateSN() {

  float sumBg=0;
  float sumSg=0;
  int   nBg=0,nSg=0;
  for (int i=0; i<histo->N(); i++) {
    if (fabs(histo->getX(i))>4) { 
      sumBg+=histo->getY(i);
      nBg++;
    }
    else if (fabs(histo->getX(i))<3) {
      sumSg+=histo->getY(i);
      nSg++;
    }
  }
  sumBg/=float(nBg);
  
  b=sumBg*(histo->N()-nBg);
  s=sumSg-b;

  cout<<"S="<<s<<"  B="<<b<<"  sumBg="<<sumBg<<endl;
  for (int i=0; i<histo->N(); i++) {
    float newValue=histo->getY(i)-sumBg;
    cout<<"new:"<<i<<" "<<histo->getY(i)<<" -> "<<newValue<<endl;
    histo->setY(i,newValue);
  }
}


void RICHFit::fitS() {
  HepMatrix alpha(3,3);
  HepMatrix beta(3,1);
}
