#ifndef _RINGSEARCHTIMING_H_
#define _RINGSEARCHTIMING_H_

#include "rich/rich.h"

#ifdef RISE_ROOT
#include "TStopwatch.h"
#include "TH2.h"
#include "TFile.h"
#endif

class RingSearchTiming {
#ifdef RISE_ROOT
 private:
  TStopwatch sw;
  double     old_time,new_time;
  TH2F       *occup_t;
  TFile      *ftime;

 public:
  /// Constructor
  inline RingSearchTiming()    { 
    new_time=old_time=0;
    ftime=new TFile("rise_timing.root","RECREATE","Timing file");
    occup_t=new TH2F("occup_t","Time vs. Occupancy",50,0,2000,50,0,5);
  }

  /// Destructor
  inline ~RingSearchTiming() { ftime->Write(); }

  /// Start Stopwatch
  inline void   startTiming()  { 
    sw.Start(); 
    old_time=sw.GetRealTime();
  }

  /// Stop Stopwatch
  inline void   stopTiming()   { 
    sw.Stop(); 
    new_time=sw.GetRealTime();
  }

  /// Get elapsed time
  inline float getDeltaTime() { return float(new_time-old_time); }

  /// Fill occupancy-time histogram
  inline void   fillTimingHistograms(float occupancy) {
    cout<<occupancy<<" "<<getDeltaTime()<<endl;
    occup_t->Fill(occupancy,getDeltaTime(),1);
  }

  /// Fill occupancy-time histogram, occupancy taken from HITB table
  inline void   fillTimingHistograms() {
    int n0=RichRun::gede->hit1.index();
    int n1=RichRun::gede->hitl.index();
    fillTimingHistograms(float(n1-n0));
  }
#endif
};


#endif
